      subroutine qval_space(xp,yp,zp,q)
      implicit none

      double precision xp,yp,zp,q,qx,qy,qz
      logical compute_grad

      integer n, i, j
      double precision qv,qxv,qyv,qzv

      integer m, n_locations
      double precision w(3), a
      integer ichoice, get_qinit_choice

      double precision b,c

      ichoice = get_qinit_choice()

      n = n_locations()

      q = 0
      do m = 1,n
         call get_locations(m,w)
         if (ichoice .eq. 1 .or. ichoice .eq. 4) then
            call cosine_bell(xp,yp,zp,w,qv)
            q = q + qv
         elseif (ichoice .eq. 2) then
            call gaussian(xp,yp,zp,w,qv)
            q = q + qv
         elseif (ichoice .eq. 3) then
            call slotted_disk(xp,yp,zp,w,qv,m)
            q = q + qv
         endif
      enddo

      end

      subroutine cosine_bell(x,y,z,w,q)
      implicit none

      double precision x,y,z,w(3),q
      double precision ca,wn
      double precision  p(3), pn
      integer m

      double precision hmax, h1, b, c, r, ri, pi
      double precision get_gc_distance

      common /compi/ pi

      hmax = 1.d0
      r = 0.5d0

      ri = get_gc_distance(x,y,z,w)

      if (ri .le. r) then
         h1 = hmax*(1 + cos(pi*ri/r))/2.d0
      else
         h1 = 0.d0
      endif
      q = h1

      end

      subroutine gaussian(x,y,z,w,q)
      implicit none

      double precision x,y,z,w(3),q, b, r2, hmax

      b = 5.d0
      hmax = 1.d0

      r2 = (x-w(1))**2 + (y-w(2))**2 + (z-w(3))**2

      q = hmax*exp(-b*r2)

      end

      subroutine slotted_disk(x,y,z,w,q,which_disk)
      implicit none

      integer which_disk
      double precision x,y,z,w(3), q
      double precision r, pi
      double precision th, lambda, thi, li
      double precision get_gc_distance, ri

      common /compi/ pi

      r = 0.5d0

      ri = get_gc_distance(x,y,z,w)

      call map2polar(w(1),w(2),w(3),lambda,th)
      call map2polar(x,y,z,li,thi)

c     # Default value
      q = 0

c     # Now check to if we are in a disk
      if (ri .lt. r) then
         if (abs(lambda - li) .gt. r/6.d0) then
            q = 1.d0
         else
            if (which_disk .eq. 1) then
               if (th - thi .lt. -5.d0*r/12) then
                  q = 1.d0
               endif
            else
               if (th - thi .gt. 5.d0*r/12) then
                  q = 1.d0
               endif
            endif
         endif
      endif

      end




      subroutine set_locations(w,n)
      implicit none

      integer i, n, m
      double precision w(3,n)

      double precision wc_com(3,100)
      integer n_com

      common /comexp1/ wc_com
      common /comexp2/ n_com

c     # Locations of initial exponentials
      do i = 1,n
         do m = 1,3
            wc_com(m,i) = w(m,i)
         enddo
      enddo
      n_com = n
      end

      subroutine get_locations(i,w)
      implicit none

      integer i,m
      double precision w(3)

      double precision wc_com(3,100)
      integer n_com
      common /comexp1/ wc_com
      common /comexp2/ n_com

      if (i .gt. n_com) then
         write(6,*) 'get_locations : i .gt. n_com'
         stop
      endif

      do m = 1,3
         w(m) = wc_com(m,i)
      enddo

      end

      integer function n_locations()
      implicit none

      integer n_com
      common /comexp2/ n_com

      n_locations = n_com


      end


c     # ------------------------------------------------
c     # Great circle distance on sphere
c     # ------------------------------------------------
      double precision function get_gc_distance(x,y,z,w)
      implicit none

      double precision x,y,z,w(3)
      double precision p(3), pn, wn, ca
      double precision rsphere, get_rsphere
      integer m

      rsphere = get_rsphere()

      p(1) = x
      p(2) = y
      p(3) = z
      pn = sqrt(p(1)*p(1) + p(2)*p(2) + p(3)*p(3))
      if (abs(pn - rsphere) .ge. 1e-3) then
         write(6,*) 'cosine_bell : Point is not on the sphere; '
         write(6,*) x, y, z, abs(pn-rsphere)
         stop
      endif

      wn = sqrt(w(1)*w(1) + w(2)*w(2) + w(3)*w(3))
      do m = 1,3
         w(m) = w(m)/wn
      enddo
      ca = (w(1)*x + w(2)*y + w(3)*z)/pn
      if (abs(ca) .gt. 1) then
         write(6,*) 'cosine_bell : abs(ca) > 1; ', ca
         stop
      endif
      get_gc_distance = acos(ca)


      end


      subroutine map2polar(x,y,z,lambda,th)
      implicit none

      double precision x,y,z,th,lambda
      double precision r, pi

      common /compi/ pi

      r = sqrt(x**2 + y**2 + z**2)
      th = asin(z/r)

      lambda = atan2(y,x)
      if (lambda .lt. 0) then
         lambda = lambda + 2*pi
      endif

      end
