      subroutine setup_mesh_advection(mx,my,mbc,xlower,ylower,dx,dy)
      implicit none

      integer mx,my,mbc
      double precision xlower,ylower,dx,dy

      integer meqn2
      parameter(meqn2 = 20)
      logical plist(meqn2)
      integer mq, mcapa, idx, method, meqn

c     # This tries to avoid calling too many things that have to do with
c     # the parabolic solver.

      include 'para.i'

      if (meqn2 .lt. meqn_com) then
         write(6,*) 'setaux : Increase size of meqn2 to ', meqn
         stop
      endif

      meqn = meqn_com
      method = method_com

c     # Indicate which equations in the system have a parabolic
c     # term
      do mq = 1,meqn
         plist(mq) = .true.
      enddo


c     # Set up pointers in common blocks in 'para.i'
      call setup_common_sphere(mx,my,mbc,xlower,ylower,dx,dy,meqn,
     &      plist,method)

c     # Set up mesh.  Use pointers defined in para.i
      call setup_mesh(mx,my,mbc,xlower,ylower,dx,dy,
     &      work(xp_ptr),work(yp_ptr),work(zp_ptr),
     &      work(xd_ptr),work(yd_ptr),work(zd_ptr))

c     # Set up approximate metric terms (lengths of boundary edges; mesh
c     # cell areas)
      call compute_area(mx,my,mbc,
     &      work(xd_ptr),work(yd_ptr),work(zd_ptr),
     &      work(area_ptr))

      end


      subroutine setup_advection(mx,my,mbc,meqn)
      implicit none

      integer mx,my,mbc, meqn

      include 'para.i'
      include 'para_exact.i'
c      include 'work_ncar.i'

c     # Set up indices into work array used for the exact solution.
      call setup_common_exact(mx,my)

c      call set_ncar_common(mx,my,mbc)

c      call set_trig_values(maxmx, maxmy, mx,my,mbc,
c     &      work(xp_ptr),work(yp_ptr),work(zp_ptr),
c     &      work(xd_ptr),work(yd_ptr),work(zd_ptr),
c     &      work_ncar(trig_p_ptr),work_ncar(trig_d_ptr))

c     # Initialize q0, the spatial portion of the exact solution.
      call compute_q0(mx,my,mbc,
     &      work(xp_ptr), work(yp_ptr), work(zp_ptr),
     &      work_exact(q0_ptr))

      end


c      subroutine set_trig_values(maxmx, maxmy, mx, my, mbc,
c     &      xp,yp,zp,xd,yd,zd,trig_p,trig_d)
c      implicit none
c
c      integer mx,my,mbc,maxmx, maxmy, maux
c
c      double precision xp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
c      double precision yp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
c      double precision zp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
c
c      double precision xd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
c      double precision yd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
c      double precision zd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
c
c      double precision trig_p(-mbc:mx+mbc+1,-mbc:my+mbc+1,4)
c      double precision trig_d(-mbc:mx+mbc+2,-mbc:my+mbc+2,4)
c
c      integer i,j
c      double precision l,th
c
c      do j = -mbc,my+mbc+2
c         do i = -mbc,mx+mbc+2
c            call map2polar(xd(i,j),yd(i,j),zd(i,j),l,th)
c            trig_d(i,j,1) = cos(l)
c            trig_d(i,j,2) = sin(l)
c            trig_d(i,j,3) = cos(th)
c            trig_d(i,j,4) = sin(th)
c         enddo
c      enddo
c
c      do j = 1-mbc,my+mbc
c         do i = 1-mbc,mx+mbc
c            call map2polar(xp(i,j),yp(i,j),zp(i,j),l,th)
c            trig_p(i,j,1) = cos(l)
c            trig_p(i,j,2) = sin(l)
c            trig_p(i,j,3) = cos(th)
c            trig_p(i,j,4) = sin(th)
c         enddo
c      enddo
c
c
c      end

      subroutine set_ncar_common(mx,my,mbc)
      implicit none

      integer mx, my, mbc
      integer total, np, nd

      include 'ncar.i'

      np = (mx+mbc+1)*(my+mbc+1)
      nd = (mx+mbc+2)*(my+mbc+2)

      trig_p_ptr = 1
      trig_d_ptr = trig_p_ptr + np
      total = trig_d_ptr + nd

      total = total - 1

      if (total .gt. mwork_ncar) then
         write(6,*) 'set_ncar_common : Increase size of mwork_ncar to ',
     &         mwork_ncar
         stop
      endif

      end

c     # --------------------------------------------------------
c     # This is to initialize q0 needed for exact solution.
c     # --------------------------------------------------------
      subroutine compute_q0(mx,my,mbc,xp,yp,zp,q0)
      implicit none

      integer mx,my,mbc
      double precision xp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision yp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision zp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision q0(0:mx+1,0:my+1)

      integer i,j, ichoice, get_init_choice
      double precision x,y,z,t
      double precision gaussian_sum
      double precision cosine_bell_sum
      double precision slotted_disk_sum
      double precision w(3)

      ichoice = get_init_choice()

      t = 0
      do i = 0,mx+1
         do j = 0,my+1
            x = xp(i,j)
            y = yp(i,j)
            z = zp(i,j)
            if (ichoice .eq. 1) then
               q0(i,j) = gaussian_sum(x,y,z)
            elseif (ichoice .eq. 2 .or. ichoice .eq. 3) then
               q0(i,j) = cosine_bell_sum(x,y,z)
            elseif (ichoice .eq. 4) then
               q0(i,j) = slotted_disk_sum(x,y,z)
            endif
         enddo
      enddo

      end

      double precision function compute_exact_sum(mq,t)
      implicit none

      integer mq
      double precision t
      double precision get_initial_sum

c     #
      compute_exact_sum = get_initial_sum(mq)

      end
