      subroutine set_velocity(maxmx,maxmy,mx,my,mbc,meqn,
     &      xp,yp,zp,xd,yd,zd, xlower,ylower,dx,dy,t,
     &      normals,aux,maux)
      implicit none

      integer maxmx, maxmy, mx,my,mbc,maux,meqn
      double precision xlower,ylower,dx,dy, t
      double precision aux(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc,maux)

      double precision normals(-mbc:mx+mbc+2,-mbc:my+mbc+2,6)

      double precision xp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision yp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision zp(-mbc:mx+mbc+1,-mbc:my+mbc+1)

      double precision xd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
      double precision yd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
      double precision zd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
      logical isdisk

      integer get_vflag, vflag

c      include 'para.i'

      vflag = get_vflag()

      if (vflag .eq. 1) then

c        # Set velocities using a stream-function.
c        # Use stream function to get average normal velocity
c        # Scale by ratio of edge length to dx or dy.
c        # Store only the normal velocity in the aux array.
c        # Normals stored in aux array are not needed.
         call compute_velocity_psi(maxmx, maxmy,mx,my,mbc,meqn,
     &         dx,dy,t,xd,yd,zd,aux,maux)

      else if (vflag .eq. 2) then
c        # Set edge-centered velocities
c        # Using mapping function to get location of the center
c        # of cell edges. Call 'get_vel' at these edge locations.
c        # Use scaled normals stored in aux array to get normal
c        # velocity, and store only the normal velocity in the
c        # aux array.
         call compute_velocity_ec(maxmx,maxmy, mx,my,mbc,meqn,
     &         xlower,ylower,dx,dy,t,normals,aux,maux)

      else if (vflag .eq. 3) then
c        # Set cell-centered velocities.
c        # Evaluate velocity 'get_vel' at cell-centered
c        # locations xp,yp,zp.
c        # Store all three components of the velocity
c        # in the aux array
         call compute_velocity_cc(maxmx,maxmy,mx,my,mbc,meqn,
     &         xp,yp,zp,t,normals,aux,maux)
      endif

      end

      subroutine compute_velocity_cc(maxmx,maxmy,mx,my,mbc,meqn,
     &      xp,yp,zp,t,normals,aux,maux)
      implicit none

      integer mx,my,mbc, maxmx, maxmy, maux, meqn
      double precision t
      double precision xp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision yp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision zp(-mbc:mx+mbc+1,-mbc:my+mbc+1)

      double precision normals(-mbc:mx+mbc+2,-mbc:my+mbc+2,6)

      double precision aux(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc,maux)

      integer i,j,m, mq
      double precision vel(3), nv(3), vl, vr, vb, vt
      double precision nvl(3), nvr(3), nvb(3), nvt(3)

c     # Get cell centered velocities.

      do i = 1-mbc,mx+mbc
         do j = 1-mbc,my+mbc
            do mq = 1,meqn

c              # -----------------------------------------------------
c              # Cell (i,j) : Project out surface normals
               call get_vel(xp(i,j),yp(i,j),zp(i,j),vel,mq,t)

               do m = 1,3
                  nvl(m) = normals(i,j,m)
                  nvr(m) = normals(i+1,j,m)
                  nvb(m) = normals(i,j,3+m)
                  nvt(m) = normals(i,j+1,3+m)
               enddo
               call closure_cell(nvl,nvb,nvr,nvt,vel)

c              # -----------------------------------------------------
c              # Compute vl at left (x) edge of cell (i,j)
               vl = 0
               do m = 1,3
c                 # Components of normal to x edge
                  nv(m) = normals(i,j,m)

c                 # Use the same normal for both left and right
c                 # cell centered velocities
                  vl = vl + nv(m)*vel(m)
               enddo

c              # Compute (vr) at right (x) edge of cell (i,j)
               vr = 0
               do m = 1,3
c                 # Components of normal to x edge
                  nv(m) = normals(i+1,j,m)

c                 # Use the same normal for both left and right
c                 # cell centered velocities
                  vr = vr + nv(m)*vel(m)
               enddo

c              # Bottom (y) edge of cell (i,j)
               vb = 0
               do m = 1,3
c                 # Components of normal to y edge
                  nv(m) = normals(i,j,3+m)

c                 # Use the same normal for both left and right
c                 # cell centered velocities
                  vb = vb + nv(m)*vel(m)
               enddo

c              # Top (y) edge of cell (i,j)
               vt = 0
               do m = 1,3
c                 # Components of normal to y edge
                  nv(m) = normals(i,j+1,3+m)

c                 # Use the same normal for both left and right
c                 # cell centered velocities
                  vt = vt + nv(m)*vel(m)
               enddo

c              # For each cell, store (vl,vr) at left edge
               aux(i,j,2) = vl
               aux(i,j,3) = vr
               aux(i,j,4) = vb
               aux(i,j,5) = vt
            enddo
         enddo
      enddo

      end


      subroutine compute_velocity_ec(maxmx,maxmy, mx,my,mbc,meqn,
     &      xlower,ylower,dx,dy,t,normals,aux,maux)
      implicit none

      integer mx,my,mbc, maxmx, maxmy, maux, meqn
      double precision xlower,ylower,dx,dy, t

      double precision normals(-mbc:mx+mbc+2,-mbc:my+mbc+2,6)

      double precision aux(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc,maux)

      integer i,j,m, mq
      double precision vel(3), xc,yc,xe,ye,ze,v,nv(3)

c     # Get edge centered velocities.

      do i = 2-mbc,mx+mbc
         do j = 1-mbc,my+mbc
c           # Do x-edges
            xc = xlower + (i-1)*dx
            yc = ylower + (j-0.5)*dy

            call mapc2m(xc,yc,xe,ye,ze)
            call get_vel(xe,ye,ze,vel,t)

            v = 0
            do m = 1,3
               nv(m) = normals(i,j,m)
               v = v + nv(m)*vel(m)
            enddo
c           # Solution is already scaled by lengths
            aux(i,j,2) = v
         enddo
      enddo

      do j = 2-mbc,my+mbc
         do i = 1-mbc,mx+mbc
c           # Do y-face
            xc = xlower + (i-0.5)*dx
            yc = ylower + (j-1)*dy

            call mapc2m(xc,yc,xe,ye,ze)
            call get_vel(xe,ye,ze,vel,t)

            v = 0
            do m = 1,3
               nv(m) = normals(i,j,3+m)
               v = v + nv(m)*vel(m)
            enddo
            aux(i,j,3) = v
         enddo
      enddo

      end

      subroutine compute_velocity_psi(maxmx, maxmy,mx,my,mbc,meqn,
     &      dx,dy,t,xd,yd,zd,aux,maux)

      integer maxmx, maxmy, mx,my,mbc,maux, meqn
      double precision dx,dy, t

      double precision xd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
      double precision yd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
      double precision zd(-mbc:mx+mbc+2,-mbc:my+mbc+2)

      double precision xd1(3),xd2(3)
      double precision aux(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc,maux)

      integer i,j, mq, a
      double precision vn, dxdy
      double precision l1,l2,th1,th2

      dxdy = dx+dy

      do i = 2-mbc,mx+mbc
         do j = 1-mbc,my+mbc
c           # x-faces
            xd1(1) = xd(i,j+1)
            xd1(2) = yd(i,j+1)
            xd1(3) = zd(i,j+1)

            xd2(1) = xd(i,j)
            xd2(2) = yd(i,j)
            xd2(3) = zd(i,j)

            call get_vel_psi(xd1,xd2,dy,vn,t)
            aux(i,j,2) = vn
         enddo
      enddo

      do j = 2-mbc,my+mbc
         do i = 1-mbc,mx+mbc
c           # y-faces
            xd1(1) = xd(i+1,j)
            xd1(2) = yd(i+1,j)
            xd1(3) = zd(i+1,j)

            xd2(1) = xd(i,j)
            xd2(2) = yd(i,j)
            xd2(3) = zd(i,j)

            call get_vel_psi(xd1,xd2,dx,vn,t)

            aux(i,j,3) = -vn
         enddo
      enddo

      end


      subroutine get_vel_psi(xd1,xd2,ds,vn,t)
      implicit none

      double precision xd1(3),xd2(3), ds, vn, psi,t

      vn = (psi(xd1(1),xd1(2),xd1(3),t) -
     &      psi(xd2(1),xd2(2),xd2(3),t))/ds

      end

      subroutine set_vflag(vflag)
      implicit none
      integer vflag, vflag_com

      common /comvel/ vflag_com

      vflag_com = vflag

      end

      integer function get_vflag()
      implicit none
      integer vflag_com

      common /comvel/ vflag_com

      get_vflag = vflag_com

      end
