      subroutine set_velocity(maxmx,maxmy,mx,my,mbc,meqn,
     &      xp,yp,zp,xd,yd,zd, xlower,ylower,dx,dy,t,
     &      aux,maux)
      implicit none

      integer maxmx, maxmy, mx,my,mbc,maux,meqn
      double precision xlower,ylower,dx,dy, t
      double precision aux(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc,maux)

      double precision xp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision yp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision zp(-mbc:mx+mbc+1,-mbc:my+mbc+1)

      double precision xd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
      double precision yd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
      double precision zd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
      logical isdisk

      integer get_vflag, vflag

      include 'para.i'

      vflag = get_vflag()

      if (vflag .eq. 1) then

c        # Set velocities using a stream-function.
c        # Use stream function to get average normal velocity
c        # Scale by ratio of edge length to dx or dy.
c        # Store only the normal velocity in the aux array.
c        # Normals stored in aux array are not needed.
         call compute_velocity_psi(maxmx, maxmy,mx,my,mbc,meqn,
     &         dx,dy,t,xd,yd,zd,aux,maux)
         call compute_velocity_cross(maxmx, maxmy,mx,my,mbc,meqn,
     &         dx,dy,t,xp,yp,zp,aux,maux)

      else if (vflag .eq. 2) then
c        # Set edge-centered velocities
c        # Using mapping function to get location of the center
c        # of cell edges. Call 'get_vel' at these edge locations.
c        # Use scaled normals stored in aux array to get normal
c        # velocity, and store only the normal velocity in the
c        # aux array.
         call compute_velocity_ec(maxmx,maxmy, mx,my,mbc,meqn,
     &         xlower,ylower,dx,dy,t,aux,maux)

      else if (vflag .eq. 3) then
c        # Set cell-centered velocities.
c        # Evaluate velocity 'get_vel' at cell-centered
c        # locations xp,yp,zp.
c        # Store all three components of the velocity
c        # in the aux array
         call compute_velocity_cc(maxmx,maxmy,mx,my,mbc,meqn,
     &         xp,yp,zp,t,aux,maux)
         if (.not. isdisk()) then
            call closure_error(maxmx, maxmy, mx,my,mbc,meqn,
     &            aux,maux)
         endif
      endif

      end

      subroutine compute_velocity_cc(maxmx,maxmy,mx,my,mbc,meqn,
     &      xp,yp,zp,t,aux,maux)
      implicit none

      integer mx,my,mbc, maxmx, maxmy, maux, meqn
      double precision t
      double precision xp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision yp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision zp(-mbc:mx+mbc+1,-mbc:my+mbc+1)

      double precision aux(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc,maux)

      integer i,j,m, mq
      double precision vel(3)

c     # Get cell centered velocities.

      do i = 1-mbc,mx+mbc
         do j = 1-mbc,my+mbc

            do mq = 1,meqn
               call get_vel(xp(i,j),yp(i,j),zp(i,j),vel,mq,t)
               do m = 1,3
                  aux(i,j,10+(mq-1)*3 + m) = vel(m)
               enddo
            enddo
         enddo
      enddo

      end


      subroutine compute_velocity_ec(maxmx,maxmy, mx,my,mbc,meqn,
     &      xlower,ylower,dx,dy,t,aux,maux)
      implicit none

      integer mx,my,mbc, maxmx, maxmy, maux, meqn
      double precision xlower,ylower,dx,dy, t

      double precision aux(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc,maux)

      integer i,j,m, mq
      double precision vel(3), xc,yc,xe,ye,ze,v,nv(3)

c     # Get cell centered velocities.

      do mq = 1,meqn
         do i = 2-mbc,mx+mbc
            do j = 1-mbc,my+mbc
c              # Do x-edges
               xc = xlower + (i-1)*dx
               yc = ylower + (j-0.5)*dy

               call mapc2m(xc,yc,xe,ye,ze)
               call get_vel(xe,ye,ze,vel,mq,t)

               v = 0
               do m = 1,3
                  nv(m) = aux(i,j,1+m)
                  v = v + nv(m)*vel(m)
               enddo
c              # Solution is already scaled by lengths
               aux(i,j,10 + (mq-1)*2 + 1) = v
            enddo
         enddo

         do j = 2-mbc,my+mbc
            do i = 1-mbc,mx+mbc
c              # Do y-face
               xc = xlower + (i-0.5)*dx
               yc = ylower + (j-1)*dy

               call mapc2m(xc,yc,xe,ye,ze)
               call get_vel(xe,ye,ze,vel,mq,t)

               v = 0
               do m = 1,3
                  nv(m) = aux(i,j,4+m)
                  v = v + nv(m)*vel(m)
               enddo
               aux(i,j,10 + (mq-1)*2 + 2) = v
            enddo
         enddo
      enddo

      end

      subroutine compute_velocity_psi(maxmx, maxmy,mx,my,mbc,meqn,
     &      dx,dy,t,xd,yd,zd,aux,maux)

      integer maxmx, maxmy, mx,my,mbc,maux, meqn
      double precision dx,dy, t

      double precision xd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
      double precision yd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
      double precision zd(-mbc:mx+mbc+2,-mbc:my+mbc+2)

      double precision xd1(3),xd2(3)
      double precision aux(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc,maux)

      integer i,j, mq, a, m
      double precision vn, dxdy
      double precision trig_values1(4),trig_values2(4)

      dxdy = dx+dy

      do mq = 1,meqn
         do i = 2-mbc,mx+mbc
            do j = 1-mbc,my+mbc
c              # x-faces

               do m = 1,4
                  trig_values1(m) = aux(i,j+1,16+m)
                  trig_values2(m) = aux(i,j,16+m)
               enddo

               call get_vel_psi(trig_values1,trig_values2,dy,vn,mq,t)
               aux(i,j,11) = vn
            enddo
         enddo

         do j = 2-mbc,my+mbc
            do i = 1-mbc,mx+mbc
c              # y-faces

               do m = 1,4
                  trig_values1(m) = aux(i+1,j,16+m)
                  trig_values2(m) = aux(i,j,16+m)
               enddo

               call get_vel_psi(trig_values1,trig_values2,dx,vn,mq,t)

               aux(i,j,12) = -vn
            enddo
         enddo

c        # Fix corner velocities
      enddo

      end


      subroutine compute_velocity_cross(maxmx, maxmy,mx,my,mbc,meqn,
     &      dx,dy,t,xp,yp,zp,aux,maux)

      integer maxmx, maxmy, mx,my,mbc,maux, meqn
      double precision dx,dy, t

      double precision xp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision yp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision zp(-mbc:mx+mbc+1,-mbc:my+mbc+1)

      double precision xp1(3),xp2(3)
      double precision aux(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc,maux)

      integer i,j, mq, m
      double precision vn, trig_values1(4),trig_values2(4)

      do mq = 1,meqn
         do i = 2-mbc,mx+mbc
            do j = 2-mbc,my+mbc
               do m = 1,4
                  trig_values1(m) = aux(i,j,20+m)
                  trig_values2(m) = aux(i-1,j,20+m)
               enddo

               call get_vel_psi(trig_values1,trig_values2,dx,vn,mq,t)
c              # Store both '+' and '-' velocities in case
c              # we are at a discontinuity
               aux(i,j,13) = -vn
               aux(i,j,14) = -vn
            enddo
         enddo

         do j = 2-mbc,my+mbc
            do i = 2-mbc,mx+mbc
c              # y-faces
               do m = 1,4
                  trig_values1(m) = aux(i,j,20+m)
                  trig_values2(m) = aux(i,j-1,20+m)
               enddo

               call get_vel_psi(trig_values1,trig_values2,dy,vn,mq,t)

c              # Store both '+' and '-' velocities in case
c              # we are at a discontinuity
               aux(i,j,15) = vn
               aux(i,j,16) = vn
            enddo
         enddo
      enddo

      end


      subroutine get_vel_psi(tv1,tv2,ds,vn,mq,t)
      implicit none

      integer mq
      double precision tv1(4),tv2(4), ds, vn, psi,t

      vn = (psi(tv1,t) - psi(tv2,t))/ds

      end

      subroutine set_vflag(vflag)
      implicit none
      integer vflag, vflag_com

      common /comvel/ vflag_com

      vflag_com = vflag

      end

      integer function get_vflag()
      implicit none
      integer vflag_com

      common /comvel/ vflag_com

      get_vflag = vflag_com

      end
