function plot_all_errors(pc,n2plt)

% To print cosine bells for unlimited and limited schemes,  and get
% effective angle
%
%             plot_all_errors([1,2],2);
%
% To print gaussian hills for unlimited and limited schemes
%
%             plot_all_errors([3,4],2);
%

close all;

fname{1} = 'cosinebells/errfile.out.mthlim4';
fname{2} = 'cosinebells/errfile.out.mthlim0';

fname{3} = 'gaussianhills/errfile.out.mthlim4';
fname{4} = 'gaussianhills/errfile.out.mthlim0';

s.color = {'r','g','b','m'};
s.sym = {'^','^','.','.'};
s.size = [10 10 30 30];
nend = [240 960];

p_lines = cell(4,1);
p_syms = cell(4,1);
p_myeff = cell(4,1);
for i = 1:4,
  opt = set_options_ncar(fname{i});
  opt.clear_plot = false;
  opt.plot_ne = true;
  opt.n2plt = n2plt;
  opt.nend = nend;
  opt.ylim = [1e-3 1];
  opt = plot_errors_ncar(opt);
  p_syms{i} = opt.plot_handle_syms(n2plt);
  p_lines{i} = opt.plot_handle_lines(n2plt);
  p_myeff{i} = opt.plot_handle_myeff;
  set(p_syms{i},'marker',s.sym{i});
  set(p_syms{i},'color',s.color{i},'markersize',s.size(i),'linewidth',4);
  slopes(i) = opt.slope(n2plt);
  myeff(i) = opt.myeff;
  hold on;
end;

set([p_myeff{:} p_lines{:}],'visible','off');
set([p_syms{:} p_lines{:}],'visible','off');
if (n2plt == 2 & pc(1) == 1 & pc(2) == 2)
  set([p_myeff{pc} p_lines{pc}],'visible','on');
end;
set([p_syms{pc} p_lines{pc}],'visible','on');

hold on;
xlim = get(gca,'xlim');
plot(xlim,[0.033 0.033],'k--');

set(gca,'ylim',[2e-3 1]);

lstr{1} = sprintf('Cosine hill (sp); Rate = %4.1f',slopes(1));
lstr{2} = sprintf('Cosine hill (un); Rate = %4.1f',slopes(2));
lstr{3} = sprintf('Gaussian (sp); Rate = %4.1f',slopes(3));
lstr{4} = sprintf('Gaussian (un); Rate = %4.1f',slopes(4));

lh = legend([p_syms{pc}],lstr{pc},4);

switch n2plt;
case 1
  title('1-norm errors','fontsize',18);
case 2
  title('2-norm errors','fontsize',18);
case 3
  title('Inf-norm errors','fontsize',18);
end;

% -----------------------------------------------------
function opt = plot_errors_ncar(varargin)

%  PLOT_ERRORS plots errors from Python convergence test 'convtest_<advection,diffusion>.py'
%
%  PLOT_ERRORS with no arguments, plots results of routine 'convtest_<advection,diffusion>.py'
%
%  PLOT_ERRORS(MAPTYPE) plots errors from a mapping MAPTYPE, where MAPTYPE
%  is one of {'disk','hemisphere','sphere'}. In this case, various
%  parameters controlling the plot are default values.
%
%  PLOT_ERRORS(OPTIONS_IN) allows the user to specify options.   Calling
%
%           options_in = set_options(maptype)
%
%  will return a structure whose fields are default values the user can
%  modify.   The fields are
%
%           maptype       Maptype ('disk','hemisphere','sphere')
%           nend          Range (in my) over which convergence rate should be
%                         computed.    Default:  my in [128 1024]
%           ylim          2x1 vector containing y range for plot
%           fname         string containing file name where errors are
%                         stored
%           n2plot        Nx1 vectors containing errors to plot (1 = 1-norm,
%                         2 = 2-norm; 3 = inf-norm; 4 = conservation error).
%           ytick         Tick marks for y-axis
%
%
%  This routine assumes that the user has run 'convtest_advection.py' or
%  'convtest_diffusion.py'.  See these files for options that are available.
%

if (nargin < 1)
  error(['plot_errors : You must specify an input argument ',...
	'using either ''set_options'' or ''maptype''']);
else
  if (isstruct(varargin{1}))
    options_in = varargin{1};
  elseif (ischar(varargin{1}))
    options_in = set_options(varargin{1});
  end
end;


maptype = options_in.maptype;
nend = options_in.nend;
ylim = options_in.ylim;
fname = options_in.fname;
n2plt = options_in.n2plt;
legpos = options_in.legpos;
tstr = options_in.title;
plot_ne = options_in.plot_ne;

clr_plot = options_in.clear_plot;

if (clr_plot)
  close all;
end;


leny = 2;

nvec = [8 16 32 64 128 256 512];
nvec = [30 60 120 240 480 960];

xtick = leny./fliplr(nvec);
y1 = log10(ylim(1));
y2 = log10(ylim(2));
ylen = y2-y1+1;
ytick = logspace(y1,y2,ylen);

if (ylen > 9)
  if (ylen/2 > 9)
    ytick = ytick(1:3:ylen);
  else
    ytick = ytick(1:2:ylen);
  end;
end;

ftitle = maptype;

% Plot errors on solution on fixed grid
ep = load(fname);
my = ep(:,1);
[h,i] = sort(leny./my);

ep = ep(i,2:5);
my = my(i);

if (length(nend)==0)
  nmin = my(1);
  nmax = my(end);
else
  nmin = nend(1);
  nmax = nend(2);
end;

% Symbols for each norm plotted.
sym_ep = {'b.', 'g.', 'rs', 'g^-'};

for n = 1:4,
  [s,b] = compute_slope(leny,h,ep(:,n),nmin,nmax);
  slopes(n) = s;
  y_intercept(n) = b;
  lines_h(n) = loglog(h,b*h.^s,'k');
  hold on;
  syms_h(n) = loglog(h,ep(:,n),sym_ep{n});
end;

set([lines_h, syms_h],'visible','off');

% Set size and thickness of symbols and lines
set(lines_h,'linewidth',2);
set(syms_h(1:2),'markersize',20);
set(syms_h(3),'markersize',5,'linewidth',5);  % To get a solid square.
set(syms_h(4),'markersize',5,'linewidth',5);  % To get a solid square.


% Set up legend entries
str_p{1} = sprintf('one-norm : Rate = %4.2f',slopes(1));
str_p{2} = sprintf('two-norm : Rate = %4.2f',slopes(2));
str_p{3} = sprintf('inf-norm : Rate = %4.2f',slopes(3));
str_p{4} = sprintf('Cons.    : Rate = %4.2f',slopes(4));

set(lines_h(n2plt),'visible','on');
set(syms_h(n2plt),'visible','on');
pleg = syms_h(n2plt);
sleg = {str_p{n2plt}};

if (max(my) < nmin)
  set(lines_h(n2plt),'visible','off');
end;


lh = legend(pleg(end:-1:1),sleg(end:-1:1),legpos);
set(lh,'fontname','fixed','fontsize',18)

set(gca,'xtick',xtick);
set(gca,'xticklabel',fliplr(nvec));
set(gca,'ytick',ytick);
set(gca,'Box','on')


% Draw dashed lines indicated which values were used to compute slopes
hold on;
plot([leny./nmin leny./nmin],ylim, 'k--');
plot([leny./nmax leny./nmax],ylim, 'k--');
set(gca,'ylim',ylim);

nlog = log(nvec)/log(2);
xlim = 2.^([nlog(end)+1 nlog(1)-1]);
set(gca,'xlim',leny./xlim);

title(sprintf('%s (%s)',tstr,maptype),'fontsize',18);
xlabel('Number of grid points on a patch edge (= 90 deg.)','fontsize',18);
ylabel('Errors','fontsize',18)

if (plot_ne)
  he = exp((log(0.033) - log(y_intercept(2)))/slopes(2));
  my_eff = 2./he;
  str = sprintf('\\theta eff. ~ %6.3f^o',90/my_eff);
  fprintf('my effective : %d\n',round(my_eff));
  xlim = get(gca,'xlim');
  % loglog(xlim,0*ylim + 0.033,'y-','linewidth',2,'erasemode','background');
  % gh = gtext(str,'fontweight','bold','fontsize',16,'background','w','interpreter','tex');
  myeff_h = loglog(he,0.0333,'k.','markersize',30);
end;


set(gca,'fontsize',18);
if (nargout > 0)
  opt = options_in;
  opt.slope = slopes;
  opt.plot_handle_syms = syms_h;
  opt.plot_handle_lines = lines_h;
  opt.plot_handle_myeff = myeff_h;
  opt.myeff = round(my_eff);
end;

% --------------------------------------------------
function [s,b] = compute_slope(lenx,h,e,nmin,nmax)

m = lenx./h >= nmin & lenx./h <= nmax;

e = e(m);
h = h(m);

y = log(e);
x = log(h);

A = [x ones(length(x),1)];

sb = A\y;
s = sb(1);
b = exp(sb(2));
% b = e(1)/h(1).^s;

% ----------------------------------------------------------------
function options_in = set_options_ncar(fname)

maptype = 'sphere';
nend = [240 960];
nvec = [30 60 120 240 480 960];
n0 = 30;

ylim = [1e-7 10];

n2plt = [2 3];
legpos = 4;

clr_plot = true;
plot_ne = true;

options_in = struct('maptype',maptype,...
                    'nvec',nvec,...
		    'n0',n0,...
                    'nend',nend,...
		    'ylim',ylim,...
		    'fname',fname,...
		    'n2plt',n2plt,...
		    'legpos',legpos,...
		    'title','Errors',...
		    'clear_plot',clr_plot,...
		    'plot_ne',plot_ne);
