!!
!! Interpolate two-patch sphere grid data to a long-lat grid.
!!
!!   %> gfortran -o interp_data interp_data.f90 mapc2m.f mapc2m_sphere.f phys2comp.f90 cellave.f
!!

PROGRAM interp_data
  IMPLICIT NONE

  INTEGER :: lx, ly, iframe

  INTEGER :: i,j, m, mq
  INTEGER :: ngrid, level, mx, my, meqn
  DOUBLE PRECISION :: xlow, ylow, dx, dy, t, pi, lc,tc,xc,yc,xp,yp,zp

  DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: q

  DOUBLE PRECISION :: t0, t1, compute_err, err, err_norm(3), tol
  DOUBLE PRECISION :: interp_bilinear, qij,xc1, yc1, area, area_total

  CHARACTER(100) :: fname1, fname2
  CHARACTER :: c
  INTEGER :: nstp, ipos, idigit, l1, l2

  COMMON /compi/ pi

  pi = 4.d0*ATAN(1.d0)

!! Size of Lat-long grid
  OPEN(10,file='latlong.dat')
  READ(10,*) iframe
  READ(10,*) lx
  READ(10,*) ly
  CLOSE(10)

  fname1 = 'fort.qxxxx'
  fname2 = 'fort.txxxx'
  nstp = iframe
  DO ipos = 10, 7, -1
     idigit = MOD(nstp,10)
     fname1(ipos:ipos) = CHAR(ICHAR('0') + idigit)
     fname2(ipos:ipos) = CHAR(ICHAR('0') + idigit)
     nstp = nstp / 10
  ENDDO

  OPEN(10,file=fname2)
  READ(10,*) t
  READ(10,*) meqn
  CLOSE(10)

  OPEN(10,file=fname1)
  READ(10,*) ngrid
  READ(10,*) level
  READ(10,*) mx
  READ(10,*) my
  READ(10,*) xlow
  READ(10,*) ylow
  READ(10,*) dx
  READ(10,*) dy

  tol = 0.01d0*dx

  ALLOCATE(q(0:mx+1,0:my+1,meqn))

  DO j = 1,my
     DO i = 1,mx
        READ(10,*) (q(i,j,m),m = 1,meqn)
     ENDDO
  ENDDO

!! Fill in ghost cell values needed for interpolation
  DO m = 1,meqn
     DO i = 1,mx
        q(i,0,m) = q(mx+1-i,1,m)
        q(i,my+1,m) = q(mx+1-i,my,m)
     ENDDO

     DO j = 1,my
        q(0,j,m) = q(mx,j,m)
        q(mx+1,j,m) = q(1,j,m)
     ENDDO
  ENDDO

  !! x : longitude
  !! y : latitude
  dx = 2*pi/lx
  dy = pi/ly

  OPEN(10,file='interp.out')
  CALL CPU_TIME(t0)
  DO i = 1,lx
     DO j = 1,ly
        lc = (i-0.5)*dx
        tc = -pi/2.d0 + (j-0.5)*dy
        xp = COS(tc)*cos(lc)
        yp = COS(tc)*sin(lc)
        zp = SIN(tc)
        CALL phys2comp(xp,yp,zp,xc,yc,tol)
        qij = interp_bilinear(q,mx,my,meqn,xc,yc)
        WRITE(10,120) lc,tc, qij
     ENDDO
     WRITE(10,*) ' '
  ENDDO
  CALL CPU_TIME(t1)
  CLOSE(10)
  WRITE(6,100) 'Elapsed time ', t1-t0
120 FORMAT(3F24.16)

100 FORMAT(A,E10.4)

END PROGRAM interp_data

DOUBLE PRECISION FUNCTION compute_err(xp,yp,zp,xc,yc)
  IMPLICIT NONE

  DOUBLE PRECISION xp,yp,zp,xc,yc
  DOUBLE PRECISION xp_est, yp_est, zp_est

  CALL mapc2m(xc,yc,xp_est,yp_est,zp_est)

  compute_err = SQRT((xp-xp_est)**2 + (yp-yp_est)**2 + (zp-zp_est)**2)

END FUNCTION compute_err

DOUBLE PRECISION FUNCTION interp_bilinear(q,mx,my,meqn,xc,yc)
  IMPLICIT NONE

  INTEGER :: mx, my, meqn
  DOUBLE PRECISION :: q(0:mx+1,0:my+1,meqn), xc, yc, quad(0:1,0:1)

  INTEGER :: i,j
  DOUBLE PRECISION :: dx, dy, u, v, qij,a00, a01, a10, a11
  DOUBLE PRECISION :: xlow, ylow

  dx = 4.d0/mx
  dy = 2.d0/my

  i = NINT((xc + 3.d0 - dx/2.d0)/dx)
  j = NINT((yc + 1.d0 - dy/2.d0)/dy)

  IF (i .LT. 0 .OR. i .GT. mx+1) THEN
     WRITE(6,*) 'Problem with i; i = ', i
     WRITE(6,*) 'xc = ', xc
     STOP
  ENDIF

  IF (j .LT. 0 .OR. j .GT. my+1) THEN
     WRITE(6,*) 'Problem with j; j = ', j
     STOP
  ENDIF

  quad(0,0) = q(i,j,1)
  quad(1,0) = q(i+1,j,1)
  quad(0,1) = q(i,j+1,1)
  quad(1,1) = q(i+1,j+1,1)

  a00 = quad(0,0)
  a01 = quad(1,0) - quad(0,0)
  a10 = quad(0,1) - quad(0,0)
  a11 = quad(1,1) - quad(1,0) - (quad(0,1) - quad(0,0))

  xlow = -3.d0 + (i-0.5)*dx
  ylow = -1.d0 + (j-0.5)*dy
  u = (xc - xlow)/dx
  v = (yc - ylow)/dy


  interp_bilinear = a00 + a01*u + a10*v + a11*u*v


END FUNCTION interp_bilinear
