c     # -------------------------------------------------------
c     # Apply the boundary conditions
c     #
C     # This assumes that we have physical boundary conditions on all
c     # four edges of Cartesian grid.
c     #
c     # This requires that the tridiagonal system has been
c     # set up by 'initialize_bc(...)'
c     # ----------------------------------------------------------
      subroutine apply_bc(mx,my,meqn,work_tri,ldtri,ipiv,c_bc,ldbc,q)
      implicit none

      integer mx,my,meqn,ldtri,ldbc
      double precision c_bc(ldbc,4,meqn)

c     # The last dimension is the number of equations
c     # containing parabolic terms; this is <= meqn
      double precision q(0:mx+1,0:my+1,*)
      double precision work_tri(ldtri,*)
      integer ipiv(2*(mx+my),*)

      integer m,mq
      logical active_parabolic_term
      integer q_idx


      do mq = 1,meqn
         if (active_parabolic_term(mq)) then
            m = q_idx(mq)
            call apply_bc_internal(mx,my,work_tri(1,mq),ldtri,
     &            ipiv(1,mq),ldbc,c_bc(1,1,mq),q(0,0,m))
         endif
      enddo

      end

      subroutine apply_bc_internal(mx,my,work_tri,mwork_tri,
     &      ipiv,ldbc,c_bc,qp)
      implicit none

      integer mx,my,mwork_tri,miwork_tri, ldbc
      double precision qp(0:mx+1,0:my+1)
      double precision work_tri(mwork_tri)
      double precision ipiv(2*(mx+my))

      double precision c_bc(ldbc,4)

      double precision fa

      integer pcval,ucval,bcval
      integer trisys_du, trisys_d, trisys_dl, trisys_du2, qh
      integer coeffs_alpha, dval, pval, rhs, rv,tv

      integer ntri, nrhs, ldb, info
      character trans

      integer nbc

      nbc = 2*(mx + my)

c      mwork_tri = 15*nbc + 6
      call get_tri_ptrs(mwork_tri,mx,my,pcval,ucval,bcval,
     &      trisys_du,trisys_d,trisys_dl,trisys_du2,qh,
     &      coeffs_alpha,pval,dval,rhs,rv,tv)

      call update_vectors(mx,my,ldbc,c_bc,qp,work_tri(bcval),
     &      work_tri(pval))

c     # Get time dependent right hand side and scalar fa
      call get_tri_rhs(mx,my,
     &      work_tri(pcval),work_tri(ucval),work_tri(bcval),
     &      work_tri(pval),work_tri(rhs),work_tri(rv),fa)

c     # Solve tridiagonal system to get dual bc values.
      trans = 'n'
      nrhs = 1
      ntri = nbc - 1
      ldb = ntri
      call dgttrs(trans, ntri,nrhs,work_tri(trisys_dl),
     &      work_tri(trisys_d),work_tri(trisys_du),
     &      work_tri(trisys_du2),ipiv,
     &      work_tri(rhs),ldb,info)

c     # Assign primal values
      call assign_bc_tri(mx,my,
     &      work_tri(pcval),work_tri(ucval),work_tri(bcval),
     &      work_tri(pval),work_tri(dval),
     &      work_tri(qh),work_tri(rhs),
     &      work_tri(coeffs_alpha),fa,qp)
      end


c     # -------------------------------------------------------
c     # Compute the right hand side based on current solution
c     # -------------------------------------------------------
      subroutine get_tri_rhs(mx,my,pcval,ucval,bcval,
     &      pval,rhs,rv,fa)
      implicit none

      integer mx,my

      double precision pcval(2*(mx+my))
      double precision ucval(2*(mx+my))
      double precision bcval(2*(mx+my),3)

      double precision rhs(2*(mx+my))
      double precision pval(2*(mx+my))
      double precision fa

      double precision a,b,c,p

      double precision rv(0:2*(mx + my))

      integer k, nbc

      nbc = 2*(mx + my)

      do k = 1,nbc
         a = bcval(k,1)
         b = bcval(k,2)
         c = bcval(k,3)
         p = pcval(k)
         rv(k) = (c + b*p*pval(k))/(a + b*p)
      enddo
      rv(0) = rv(nbc)

      do k = 1,nbc
         rhs(k) = -(rv(k-1) + rv(k))
      enddo
      fa = rhs(nbc)

      end


      subroutine update_vectors(mx,my,ldbc,c_bc,sp,bcval,pval)
      implicit none

      integer mx,my, ldbc
      double precision c_bc(ldbc,4)
      double precision sp(0:mx+1,0:my+1)

      double precision bcval(2*(mx+my),3)
      double precision pval(2*(mx+my))

      integer i, j, n, nbc

      nbc = 2*(mx + my)

      n = 1
      do i = 1,mx
         pval(n) = sp(i,1)
         bcval(n,3) = c_bc(i,2)
         n = n + 1
      enddo

      do j = 1,my
         pval(n) = sp(mx,j)
         bcval(n,3) = c_bc(j,3)
         n = n + 1
      enddo

      do i = mx,1,-1
         pval(n) = sp(i,my)
         bcval(n,3) = c_bc(i,4)
         n = n + 1
      enddo

      do j = my,1,-1
         pval(n) = sp(1,j)
         bcval(n,3) = c_bc(j,2)
         n = n + 1
      enddo

      end


      subroutine assign_bc_tri(mx,my,pcval,ucval,
     &      bcval,pval,dval,qh,qinh,coeffs_alpha,fa,qp)
      implicit none

      integer mx,my
      double precision qh(2*(mx+my)-1)
      double precision qinh(2*(mx+my)-1)
      double precision coeffs_alpha(3), fa

      double precision qp(0:mx+1,0:my+1)

      double precision pcval(2*(mx+my))
      double precision ucval(2*(mx+my))
      double precision bcval(2*(mx+my),3)
      double precision dval(2*(mx+my)+1)
      double precision pval(2*(mx+my))

      integer i,j,n, nbc
      double precision alpha, a,b,c

      nbc = 2*(mx + my)

      alpha = coeffs_alpha(1)*fa + coeffs_alpha(2)*qinh(1) +
     &      coeffs_alpha(3)*qinh(nbc-1)

      if (1./alpha .eq. 0) then
         alpha = 0
      endif

      do n = 1,nbc-1
         dval(n) = qinh(n) + alpha*qh(n)
      enddo
      dval(nbc) = alpha
      dval(nbc+1) = dval(1)

c     # Overwrite pval...
      do n = 1,nbc
         a = bcval(n,1)
         b = bcval(n,2)
         c = bcval(n,3)
         pval(n) = (c + b*(pcval(n)*pval(n) -
     &         ucval(n)*(dval(n+1)-dval(n))))/(a + b*pcval(n))
      enddo

c     # Set primal boundary values
      n = 1
      do i = 1,mx
         qp(i,0) = pval(n)
         n = n + 1
      enddo

      do j = 1,my
         qp(mx+1,j) = pval(n)
         n = n + 1
      enddo

      do i = mx,1,-1
         qp(i,my+1) = pval(n)
         n = n + 1
      enddo

      do j = my,1,-1
         qp(0,j) = pval(n)
         n = n + 1
      enddo
      end


c     # -------------------------------------------------------
c     # Initialize tridiagonal systems needed for applying BCs
c     #
c     # Assume BCs of the form
c     #
c     #        A q + B q_n = C(t)
c     #
c     # where A, B, and C depend on space.  Only C can depend on
c     # time.
c     #
c     # This assumes that we have already set up 'coeffs_p', and
c     # the user has supplied BC coefficients A and B in arrays
c     # a_bc, b_bc.  The user also supplies C at time t = 0.
c     # If C is time dependent, the user can update the values
c     # during calls from the RKC solver.
c     # ----------------------------------------------------------
      subroutine initialize_bc(mx,my,meqn,coeffs_p,
     &      a_bc,b_bc,c_bc,ldbc,lenp,work_tri,ldtri,ipiv)
      implicit none

      integer mx,my, meqn, ldbc,ldtri

      double precision coeffs_p(0:mx+1,0:my+1,2,2)

      double precision a_bc(ldbc,4,meqn)
      double precision b_bc(ldbc,4,meqn)
      double precision c_bc(ldbc,4,meqn)

      double precision lenp(2*(mx+my))
      double precision work_tri(ldtri,*)
      integer ipiv(2*(mx+my),*)

      integer mq
      logical has_parabolic_term

      do mq = 1,meqn
         if (has_parabolic_term(mq)) then
c           # Initialize tridiagonal system needed for imposing BCs
            call initialize_bc_internal(mx,my,coeffs_p,lenp,
     &            a_bc(1,1,mq),b_bc(1,1,mq),c_bc(1,1,mq),ldbc,
     &            ipiv(1,mq),work_tri(1,mq),ldtri)
         endif
      enddo

      end


      subroutine initialize_bc_internal(mx,my,coeffs_p,lenp,
     &      a_bc,b_bc,c_bc,ldbc,ipiv,work_tri,mwork_tri)
      implicit none

      integer mx,my,mwork_tri,miwork_tri,ldbc
      double precision coeffs_p(0:mx+1,0:my+1,2,2)
      double precision work_tri(mwork_tri)
      integer ipiv(*)

      double precision lenp(2*(mx+my))
      double precision a_bc(ldbc,4)
      double precision b_bc(ldbc,4)
      double precision c_bc(ldbc,4)

      integer trisys_du,trisys_d,trisys_dl,trisys_du2
      integer pcval, ucval, bcval,qh, coeffs_alpha
      integer pval,dval,rhs,rv,tv
      integer nbc

      call get_tri_ptrs(mwork_tri,mx,my,pcval,ucval,bcval,
     &      trisys_du,trisys_d,trisys_dl,trisys_du2,qh,
     &      coeffs_alpha,pval,dval,rhs,rv,tv)

      call set_vectors(mx,my,coeffs_p,a_bc,b_bc,c_bc,ldbc,lenp,
     &      work_tri(pcval),work_tri(ucval),work_tri(bcval))

      call construct_sys(mx,my,work_tri(pcval),
     &      work_tri(ucval),work_tri(bcval),work_tri(tv),
     &      work_tri(trisys_dl),work_tri(trisys_d),
     &      work_tri(trisys_du),work_tri(trisys_du2),
     &      ipiv,work_tri(qh),work_tri(coeffs_alpha))

      end

      subroutine copy_c_bc(mx,my,meqn,c_bc,ldbc,c_bc_store)
      implicit none

      integer mx,my,meqn,ldbc

      double precision c_bc(ldbc,4,meqn)
      double precision c_bc_store(ldbc,4,meqn)

      integer i,j, mq
      logical has_parabolic_term

      do mq = 1,meqn
c        # Only copy those terms which have been properly defined.
         if (has_parabolic_term(mq)) then
            do j = 1,my
               c_bc_store(j,1,mq) = c_bc(j,1,mq)
               c_bc_store(j,3,mq) = c_bc(j,3,mq)
            enddo

            do i = 1,mx
               c_bc_store(i,2,mq) = c_bc(i,2,mq)
               c_bc_store(i,4,mq) = c_bc(i,4,mq)
            enddo
         endif
      enddo

      end


      subroutine set_vectors(mx,my,coeffs_p,a_bc,b_bc,c_bc,ldbc,
     &      lenp,pcval,ucval,bcval)
      implicit none

      integer mx,my,ldbc
      double precision coeffs_p(0:mx+1,0:my+1,2,2)

      double precision lenp(2*(mx+my))
      double precision a_bc(ldbc,4)
      double precision b_bc(ldbc,4)
      double precision c_bc(ldbc,4)

      double precision pcval(2*(mx+my))
      double precision ucval(2*(mx+my))
      double precision bcval(2*(mx+my),3)

      integer i,j,n

      n = 1
      do i = 1,mx
         pcval(n) =  -coeffs_p(i,1,2,1)/lenp(n)
         ucval(n) =  coeffs_p(i,1,2,2)/lenp(n)
         bcval(n,1) = a_bc(i,2)
         bcval(n,2) = b_bc(i,2)
         bcval(n,3) = c_bc(i,2)
         n = n + 1
      enddo


      do j = 1,my
         pcval(n) = coeffs_p(mx+1,j,1,1)/lenp(n)
         ucval(n) = coeffs_p(mx+1,j,1,2)/lenp(n)
         bcval(n,1) = a_bc(j,3)
         bcval(n,2) = b_bc(j,3)
         bcval(n,3) = c_bc(j,3)
         n = n + 1
      enddo

      do i = mx,1,-1
         pcval(n) = -coeffs_p(i,my+1,2,1)/lenp(n)
         ucval(n) = coeffs_p(i,my+1,2,2)/lenp(n)
         bcval(n,1) = a_bc(i,4)
         bcval(n,2) = b_bc(i,4)
         bcval(n,3) = c_bc(i,4)
         n = n + 1
      enddo

      do j = my,1,-1
         pcval(n)   = -coeffs_p(1,j,1,1)/lenp(n)
         ucval(n)   = -coeffs_p(1,j,1,2)/lenp(n)
         bcval(n,1) = a_bc(j,1)
         bcval(n,2) = b_bc(j,1)
         bcval(n,3) = c_bc(j,1)
         n = n + 1
      enddo

      end

      subroutine construct_sys(mx,my,pcval,ucval,bcval,tv,
     &      trisys_dl,trisys_d,trisys_du,trisys_du2,
     &      ipiv, qh,coeffs_alpha)
      implicit none

      integer mx,my
      double precision   coeffs_p(0:mx+1,0:my+1,2,2)
      double precision  trisys_dl(2*(mx+my))
      double precision   trisys_d(2*(mx+my))
      double precision  trisys_du(2*(mx+my))
      double precision trisys_du2(2*(mx+my))
      integer ipiv(2*(mx+my))

      double precision     qh(2*(mx+my))
      double precision coeffs_alpha(3)

      double precision pcval(2*(mx+my))
      double precision ucval(2*(mx+my))
      double precision bcval(2*(mx+my),3)

      double precision tv(0:2*(mx + my))

      double precision a,b,u,p

      double precision aa,v(2),w(2)
      integer k, km, nbc, ntri, ldb, nrhs, info, m
      character trans

      nbc = 2*(mx+my)

      do k = 1,nbc
         a = bcval(k,1)
         b = bcval(k,2)
         u = ucval(k)
         p = pcval(k)
         tv(k) = b*u/(a + b*p)
      enddo
      tv(0) = tv(nbc)

      do k = 1,nbc-1
         trisys_dl(k) = tv(k)
      enddo

      do k = 1,nbc
         trisys_d(k) = tv(k) - tv(k-1) - 2.d0
      enddo

      do k = 1,nbc-1
         trisys_du(k) = -tv(k)
      enddo

c     # Define vectors w and v. We only store v(1) and v(nbc-1); the rest
c     # are zero.  Same for w.
      w(1) = tv(nbc)
      v(1) = -tv(nbc)
      w(2) = -tv(nbc-1)
      v(2) = tv(nbc-1)
      u = tv(nbc) - tv(nbc-1) - 2.d0

c     # Factor tridiagonal system.
      ntri = nbc - 1
      call dgttrf(ntri,trisys_dl, trisys_d, trisys_du,
     &      trisys_du2, ipiv,info)

      info = 0
      if (info .ne. 0) then
         write(6,'(A,I5)') 'construct_sys(1) : info .ne. 0; ',
     &         info
         stop
      endif

c     # w - equation (2.41)
      do k = 1,nbc-1
         qh(k) = 0.d0
      enddo
      qh(1)    = -w(1)
      qh(nbc-1) = -w(2)

      trans = 'n'
      ldb      = ntri
      nrhs     = 1
      call dgttrs(trans,ntri,nrhs,trisys_dl,trisys_d,
     &      trisys_du,trisys_du2,ipiv,qh,ldb,info)

      if (info .ne. 0) then
         write(6,'(A,I5)') 'construct_sys(2) : info .ne. 0; ',
     &         info
         stop
      endif

  105 format(3E16.8)
      coeffs_alpha(1) = 1.d0/(u + v(1)*qh(1) + v(2)*qh(nbc-1))
      coeffs_alpha(2) = -coeffs_alpha(1)*v(1)
      coeffs_alpha(3) = -coeffs_alpha(1)*v(2)

      end

c     # -------------------------------------------------------------
c     # Miscellaneous routines
c     # -------------------------------------------------------------

      subroutine get_tri_ptrs(mwork,mx,my,pcval,ucval,bcval,
     &      trisys_du,trisys_d,trisys_dl,trisys_du2,qh,
     &      coeffs_alpha,pval,dval,rhs,rv,tv)
      implicit none

      integer mwork,mx,my
      integer trisys_du,trisys_d,trisys_dl,trisys_du2,qh
      integer pcval,ucval,bcval,coeffs_alpha
      integer pval,dval,rhs,rv,tv

      integer total, nbc

      nbc = 2*(mx + my)

      pcval        = 1
      ucval        = pcval        + nbc
      bcval        = ucval        + nbc
      trisys_du    = bcval        + 3*nbc
      trisys_d     = trisys_du    + nbc
      trisys_dl    = trisys_d     + nbc
      trisys_du2   = trisys_dl    + nbc
      qh           = trisys_du2   + nbc
      coeffs_alpha = qh           + nbc
      pval         = coeffs_alpha + 3
      dval         = pval         + nbc

c     # dval(2*(mx+my)+1)
      rhs          = dval         + nbc + 1
      rv           = rhs          + nbc
c     # rv(0:2*(mx+my))
      tv           = rv           + nbc + 1

c     # total  = 15*nbc + 6
      total        = tv           + nbc + 1

c     # Actual memory needed :
      total = total - 1

      if (mwork .lt. 0) then
c        # This is being called from the RKC solver - we already
c        # know that we have enough space
         return
      endif
      if (mwork .lt. total) then
         write(6,*) 'Increase memory in work_tri to ',total
         stop
      endif

      end
