      subroutine parabolic_main(maxmx, maxmy,meqn,mbc,q)
      implicit none

      integer maxmx, maxmy, meqn, mbc
      double precision q(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc,meqn)

      integer mx,my
      double precision dx,dy
      double precision xlower,ylower,xupper,yupper

      logical rest

      integer nout, nstep, iframe, ierror, n
      integer verbose, step_counter
      double precision dt, t0, tinitial, ti

      integer mq
      logical plist(meqn)


      open(55,file='para.data')

c     domain variables
      read(55,*) mx
      read(55,*) my

c     # Read nstep,ntotal
c     # dt
      read(55,*) nstep,nout
      read(55,*) dt

      read(55,*) xlower
      read(55,*) xupper
      read(55,*) ylower
      read(55,*) yupper

      read(55,*) verbose

      read(55,*) iframe
      close(55)
c     # Done reading in data


c     # ----------------------------------------------
c     # Basic sanity checks

      if (mx .gt. maxmx) then
         write(6,200) 'Increase size of maxmx to ', mx
         stop
      endif

      if (my .gt. maxmy) then
         write(6,200) 'Increase size of maxmy to ', my
         stop
      endif
  200 format(A,I5)

c     # ----------------------------------------------

      rest = iframe .ge. 0

c     # grid spacing
      dx = (xupper - xlower)/mx
      dy = (yupper - ylower)/my

c     # Read in any parameters
      call setprob

c     # Initialize solution using either data from a restart file
c     # or by calling initialization routine 'qinit'
      if (rest) then
         call parabolic_restart(maxmx,maxmy,meqn,mbc,mx,my,
     &         q,iframe,t0)
         write(6,400) iframe,t0
  400    format('Restarting from file ',I4,' at t = ',F12.4,/)
      else
         call qinit(maxmx,maxmy,meqn,mbc,mx,my,xlower,ylower,
     &         dx,dy,q)
         t0 = 0
         iframe = 0
         call parabolic_output(maxmx,maxmy,meqn,mbc,mx,my,
     &         xlower,ylower,dx,dy,q,t0,iframe)
         if (verbose .ge. 2) then
            write(6,601) iframe,t0
         endif
      endif

c     # Assume that all equations have parabolic terms.
      do mq = 1,meqn
         plist(mq) = .true.
      enddo

      call setup_parabolic(mx,my,meqn,plist,xlower,ylower,dx,dy)

c     # Start time stepping
      step_counter = 1
      do n = 0,nout-1
         ti = t0 + dt*n
         call parabolic_step_rkc(maxmx,maxmy,meqn,mbc,mx,my,plist,
     &         q,ti,dt,ierror)
         if (verbose .ge. 2) then
            call rkc_stats_short(ti,dt,step_counter)
         endif
         if (verbose .eq. 3) then
            call rkc_stats(ti,dt)
         endif

         if (ierror .ge. 2) then
            write(6,100)'parabolic_step : Problem with rkc solver; ',
     &            ' ierror = ',ierror
         endif
  100    format(A,A,I5)

         if (mod(n+1,nstep) .eq. 0) then
            iframe = iframe + 1
            call parabolic_output(maxmx,maxmy,meqn,mbc,mx,my,
     &            xlower,ylower,dx,dy,q,ti,iframe)
            if (verbose .eq. 2) then
               write(6,601) iframe,ti
            endif
            step_counter = 0
         endif
         step_counter = step_counter + 1
      enddo
  500 format('Step ', i4, ' at time t = ',F12.4,
     &      ' (max stages = ',I3,'; fevals = ',I6,')')
  601 format('Frame ',i4,' done at time t = ',F12.4,/)

      end
