      subroutine apply_rhs_cn(mx,my,meqn,qn,t,dt,rhs)
      implicit none

      integer mx,my,meqn
      double precision t,dt
      double precision qn(0:mx+1,0:my+1,*)
      double precision rhs(0:mx+1,0:my+1,*)

      integer i,j,m, mq, q_idx
      logical active_parabolic_term
      double precision xlower,ylower, dx,dy
      double precision lambda, tp
      integer ldbc,ldtri

      include 'para.i'

      xlower = xlower_com
      ylower = ylower_com
      dx = dx_com
      dy = dy_com
      ldbc = ldbc_com
      ldtri = ldtri_com

c     # Compute dual cell values (user defined)
      call compute_dual_phys(mx,my,meqn,qn,work(qd_ptr))

c     # Update inhomogenous boundary data 'c_bc'.  This can
c     # be a dummy routine if 'c_bc' does not depend on time.
c     # (user defined)
c     # Here, BCs are needed at time level t
      tp = t
      call update_bc(mx,my,meqn,xlower,ylower,dx,dy,t,
     &      work(c_bc_ptr),ldbc)
c
c     # Apply boundary conditions; both primal and dual
c     # boundary values will get set by this routine.
      call apply_bc(mx,my,meqn,work(work_tri_ptr),ldtri,ipiv,
     &      work(c_bc_ptr),ldbc,qn,work(qd_ptr))

c     # Compute Laplacian using primal and dual values
c     # (user defined)
      call apply_LB(mx,my,meqn,work(coeffs_ptr),
     &      work(area_ptr),qn,work(qd_ptr),rhs)

      lambda = -4.d0/dt
      do mq = 1,meqn
         if (active_parabolic_term(mq)) then
            m = q_idx(mq)
            do j = 1,my
               do i = 1,mx
                  rhs(i,j,m) = 0.5*(-lambda*qn(i,j,m) + rhs(i,j,m))
               enddo
            enddo
         endif
      enddo

c     # Time center the source term for Crank-Nicolson
      tp = t + dt/4.d0
      call src_reaction(mx,my,meqn,tp,qn,rhs)

      do mq = 1,meqn
         if (active_parabolic_term(mq)) then
            m = q_idx(mq)
            do j = 1,my
               do i = 1,mx
                  rhs(i,j,m) = -2.d0*rhs(i,j,m)
               enddo
            enddo
         endif
      enddo

      end


      subroutine matvec(mx,my,meqn,q,lambda,Aq,ishomogeneous)
      implicit none

      integer mx,my,meqn
      double precision q(0:mx+1,0:my+1,*)
      double precision Aq(0:mx+1,0:my+1,*)
      double precision lambda
      logical ishomogeneous

      integer ldbc,ldtri
      double precision xlower,ylower,dx,dy
      double precision t_curr, dt_curr, tp

      integer i,j,m,mq,q_idx
      logical active_parabolic_term

      include 'para.i'

      xlower = xlower_com
      ylower = ylower_com
      dx = dx_com
      dy = dy_com
      ldbc = ldbc_com
      ldtri = ldtri_com
      t_curr = current_time_com
      dt_curr = current_dt_com

      call increment_fcounter()

c     # Compute dual cell values (user defined)
      call compute_dual_phys(mx,my,meqn,q,work(qd_ptr))

c     # Update inhomogeneous boundary data 'c_bc'.  This can
c     # be a dummy routine if 'c_bc' does not depend on time.
c     # (user defined)
      if (ishomogeneous) then
         call update_homogeneous(mx,my,meqn,ldbc,work(c_bc_ptr))
      else
c        # This is called at the first computation of the
c        # residual
         tp = t_curr + dt_curr
         call update_bc(mx,my,meqn,xlower,ylower,dx,dy,tp,
     &         work(c_bc_ptr),ldbc)
      endif

c
c     # Apply boundary conditions; both primal and dual
c     # boundary values will get set by this routine.
      call apply_bc(mx,my,meqn,work(work_tri_ptr),ldtri,ipiv,
     &      work(c_bc_ptr),ldbc,q,work(qd_ptr))

c     # Compute Laplacian using primal and dual values
c     # (user defined)
      call apply_LB(mx,my,meqn,work(coeffs_ptr),work(area_ptr),q,
     &      work(qd_ptr),Aq)

      do mq = 1,meqn
         if (active_parabolic_term(mq)) then
            m = q_idx(mq)
            do j = 1,my
               do i = 1,mx
                  Aq(i,j,m) = Aq(i,j,m) + lambda*q(i,j,m)
               enddo
            enddo
         endif
      enddo

      call set_zeros(mx,my,meqn,Aq)

      end
