      subroutine parabolic_step(maxmx, maxmy,meqn,mbc,mx,my,
     &      nv,q_claw,t,dt,method,verbose,step_counter)
      implicit none

      integer maxmx, maxmy, meqn,mbc,mx,my
      double precision q_claw(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc,meqn)
      logical nv(meqn)
      double precision t, dt
      integer method,verbose,step_counter

      integer ierror

      if (method .eq. 1) then
c        # --------------------------------------------------
c        # Forward Euler
c        # --------------------------------------------------
         call parabolic_step_feuler(maxmx, maxmy,meqn,mbc,mx,my,
     &         nv,q_claw,t,dt,ierror)

      elseif (method .eq. 2) then
c        # --------------------------------------------------
c        # Backward Euler
c        # --------------------------------------------------
         call parabolic_step_beuler(maxmx, maxmy,meqn,mbc,mx,my,
     &         nv,q_claw,t,dt,ierror)

      elseif (method .eq. 3) then
c        # --------------------------------------------------
c        # Crank-Nicolson
c        # --------------------------------------------------
         call parabolic_step_cn(maxmx, maxmy,meqn,mbc,mx,my,
     &         nv,q_claw,t,dt,ierror)

      elseif (method .eq. 4) then
c        # --------------------------------------------------
c        # TR-BDF2
c        # --------------------------------------------------
         call parabolic_step_trbdf(maxmx, maxmy,meqn,mbc,mx,my,
     &      nv,q_claw,t,dt,ierror)
         if (verbose .ge. 2) then
            call trbdf_stats_short(t,dt,step_counter)
         endif
         if (verbose .eq. 3) then
c            call trbdf_stats(t,dt)
         endif

      elseif (method .eq. 5) then
c        # --------------------------------------------------
c        # ROCK4
c        # --------------------------------------------------
         call parabolic_step_rock4(maxmx, maxmy,meqn,mbc,mx,my,
     &         nv,q_claw,t,dt,ierror)

         if (verbose .eq. 5) then
            call rkc_stats(t,dt)
         endif
         if (ierror .ge. 2) then
            write(6,100)'parabolic_step : Problem with rkc solver; ',
     &            ' ierror = ',ierror
         endif
      elseif (method .eq. 6) then
c        # --------------------------------------------------
c        # RKC
c        # --------------------------------------------------
         call parabolic_step_rkc(maxmx, maxmy,meqn,mbc,mx,my,
     &      nv,q_claw,t,dt,ierror)
         if (verbose .eq. 2) then
            call rkc_stats_short(t,dt,step_counter)
         endif
         if (verbose .eq. 3) then
            call rkc_stats(t,dt,step_counter)
         endif
         if (ierror .ge. 2) then
            write(6,100)'parabolic_step : Problem with rkc solver; ',
     &            ' ierror = ',ierror
         endif
      elseif (method .eq. 7) then
c        # --------------------------------------------------
c        # IRKC
c        # --------------------------------------------------
         call parabolic_step_irkc(maxmx, maxmy, meqn,mbc,mx,my,
     &         nv,q_claw,t,dt,ierror)
         if (verbose .eq. 2) then
            call irkc_stats_short(t,dt,step_counter)
         endif
         if (verbose .eq. 3) then
            call irkc_stats(t,dt,step_counter)
         endif
      endif
  100 format(A,A,I5)

      end



      subroutine set_zeros(mx,my,meqn,rhs)
      implicit none

      integer mx,my,meqn
      double precision rhs(0:mx+1,0:my+1,*)

      integer i,j,m,mq
      logical active_parabolic_term
      integer q_idx


c     # --------------------------------------------
c     # Primary cells
c     # --------------------------------------------
c     # Left and right ghost cells
      do mq = 1,meqn
         if (active_parabolic_term(mq)) then
            m = q_idx(mq)

            do j = 1,my
               rhs(0,j,m)    = 0.d0
               rhs(mx+1,j,m) = 0.d0
            enddo

c           # Bottom and top ghost cells
            do i = 1,mx
               rhs(i,0,m)    = 0.d0
               rhs(i,my+1,m) = 0.d0
            enddo

c           # corners
            rhs(0,0,m) = 0
            rhs(0,my+1,m) = 0
            rhs(mx+1,0,m) = 0
            rhs(mx+1,my+1,m) = 0
         endif
      enddo

      end


      subroutine setup_mesh(mx,my,xlower,ylower,dx,dy,
     &      xp,yp,zp,xd,yd,zd)
      implicit none

      integer mx,my
      double precision xlower, ylower, dx,dy
      double precision xp(0:mx+1,0:my+1)
      double precision yp(0:mx+1,0:my+1)
      double precision zp(0:mx+1,0:my+1)

      double precision xd(mx+1,my+1)
      double precision yd(mx+1,my+1)
      double precision zd(mx+1,my+1)

      double precision dxf,dyf,xc,yc,xd1,yd1,zd1
      double precision xupper, yupper

      logical issphere

      integer i,j

c     # ---------------------------------------------------------
c     # Get points at dual nodes of fine mesh.
c     # ---------------------------------------------------------

      xupper = xlower + mx*dx
      yupper = ylower + my*dy

      dxf = dx/2.d0
      dyf = dy/2.d0
      do i = 0,2*mx+2
         do j = 0,2*my+2
            if (mod(i,2) .ne. mod(j,2)) then
               cycle
            endif
            xc = xlower + (i-1)*dxf
            yc = ylower + (j-1)*dyf

c           # To get primary mesh points at computational
c           # boundary

            call fix_edges(xc,yc,xlower,ylower,xupper,yupper)

            call mapc2m(xc,yc,xd1,yd1,zd1)
            if (mod(i,2) .eq. 1) then
               xd((i-1)/2 + 1, (j-1)/2 + 1) = xd1
               yd((i-1)/2 + 1, (j-1)/2 + 1) = yd1
               zd((i-1)/2 + 1, (j-1)/2 + 1) = zd1
            else
               xp(i/2,j/2) = xd1
               yp(i/2,j/2) = yd1
               zp(i/2,j/2) = zd1
            endif
         enddo
      enddo

      end

c     # ---------------------------------------------------------
c     # Get areas and lengths using trilinear approximations
c     # ---------------------------------------------------------
      subroutine setup_metric(mx,my,xlower,ylower,dx,dy,
     &      xp,yp,zp,xd,yd,zd,area,lengths_bnd)
      implicit none

      integer mx,my
      double precision xlower,ylower,dx,dy
      double precision area(0:mx+1,0:my+1)

      double precision xd(mx+1,my+1)
      double precision yd(mx+1,my+1)
      double precision zd(mx+1,my+1)

      double precision xp(0:mx+1,0:my+1)
      double precision yp(0:mx+1,0:my+1)
      double precision zp(0:mx+1,0:my+1)

      logical issphere

c     # This is only used for lengths in boundary terms, so it
c     # is not used if we are on the sphere
      double precision lengths_bnd(*)

      integer deg

c     # Compute lengths of edges on the boundary - needed
c     # for boundary conditions
      if (.not. issphere()) then
         call compute_lengths(mx,my,xd,yd,zd,lengths_bnd)
      endif

c     # Compute areas using quadrature rule
      deg = 4
c      call compute_area(mx,my,xlower,ylower,dx,dy,
c     &      xp,yp,zp,xd,yd,zd,deg,area)
      call compute_area2(mx,my,xd,yd,zd,area)

      end

c     # -----------------------------------------------------
c     # Compute lengths of edges of primal cells
c     # This is used in initialization of the tridiagonal
c     # system used at physical boundaries.
c     # -----------------------------------------------------
      subroutine compute_lengths(mx,my,xd,yd,zd,lenp)
      implicit none

      integer mx,my
      double precision xd(mx+1,my+1)
      double precision yd(mx+1,my+1)
      double precision zd(mx+1,my+1)

      double precision lenp(2*(mx+my))

      integer i,j, k
      double precision dx1, dy1, dz1

c     # 'lenp' are the lengths at boundary edges of primal
c     # cells.

      k = 1
      do i = 1,mx
         dx1 = xd(i+1,1) - xd(i,1)
         dy1 = yd(i+1,1) - yd(i,1)
         dz1 = zd(i+1,1) - zd(i,1)

         lenp(k) = sqrt(dx1*dx1 + dy1*dy1 + dz1*dz1)
         k = k + 1
      enddo

      do j = 1,my
         dx1 = xd(mx+1,j+1) - xd(mx+1,j)
         dy1 = yd(mx+1,j+1) - yd(mx+1,j)
         dz1 = zd(mx+1,j+1) - zd(mx+1,j)

         lenp(k) = sqrt(dx1*dx1 + dy1*dy1 + dz1*dz1)
         k = k + 1
      enddo

      do i = mx,1,-1
         dx1 = xd(i+1,my+1) - xd(i,my+1)
         dy1 = yd(i+1,my+1) - yd(i,my+1)
         dz1 = zd(i+1,my+1) - zd(i,my+1)

         lenp(k) = sqrt(dx1*dx1 + dy1*dy1 + dz1*dz1)
         k = k + 1
      enddo


      do j = my,1,-1
         dx1 = xd(1,j+1) - xd(1,j)
         dy1 = yd(1,j+1) - yd(1,j)
         dz1 = zd(1,j+1) - zd(1,j)

         lenp(k) = sqrt(dx1*dx1 + dy1*dy1 + dz1*dz1)
         k = k + 1
      enddo

      end

      subroutine compute_area(mx,my,xlower,ylower,dx,dy,
     &      xp,yp,zp,xd,yd,zd,deg,ap)
      implicit none

      integer mx,my, deg

      external one
      double precision xlower,ylower,dx,dy

      double precision xp(0:mx+1,0:my+1)
      double precision yp(0:mx+1,0:my+1)
      double precision zp(0:mx+1,0:my+1)

      double precision xd(mx+1,my+1)
      double precision yd(mx+1,my+1)
      double precision zd(mx+1,my+1)

      double precision ap(0:mx+1,0:my+1)

      integer i,j,ii,jj, k
      double precision a,b,c,d
      double precision quad(0:1,0:1,3)
      double precision a00(3),a01(3),a10(3), a11(3)
      double precision quad_square, quad_triangle

      double precision xcenter(3), area, xlow,ylow
      logical isavg, isdiag
      integer m

      common /comtri/ a00,a01,a10,a11

      deg = 4
      isavg = .false.

      open(10,file='area_old.out')

c     # Primary cells.  Note that we don't do anything special
c     # in the diagonal cells - so the areas there are less accurate
c     # than in the rest of the mesh.
      do j = 1,my
         do i = 1,mx
c           # Approximate surfaces of the cells using a trilinear
c           # map
            xlow = xlower + (i-1)*dx
            ylow = ylower + (j-1)*dy
            a = xlow
            b = xlow + dx
            c = ylow
            d = ylow + dy
            do ii = 0,1
               do jj = 0,1
                  quad(ii,jj,1) = xd(i+ii,j+jj)
                  quad(ii,jj,2) = yd(i+ii,j+jj)
                  quad(ii,jj,3) = zd(i+ii,j+jj)
               enddo
            enddo
            do m = 1,3
               a00(m) = quad(0,0,m)
               a01(m) = quad(1,0,m) - quad(0,0,m)
               a10(m) = quad(0,1,m) - quad(0,0,m)
               a11(m) = quad(1,1,m) - quad(1,0,m) -
     &               quad(0,1,m) + quad(0,0,m)
            enddo
            ap(i,j) = quad_square(one,a,b,c,d,deg,isavg,area)
            write(10,120) i,j,ap(i,j)
         enddo
      enddo
  120 format(2I5,E24.16)
      close(10)
      end



c     # This is the area element based on a linear approximation to
c     # the surface defined by the corners of the mesh cell.
      double precision function area_element(xc,yc)
      implicit none

      double precision xc,yc

      double precision a00(3), a01(3),a10(3), a11(3)
      double precision Txi(3), Teta(3)
      double precision quad(0:1,0:1,3)
      double precision quad_tmp(0:1,0:1,3), xcenter(3)
      double precision norm_cross, a,b,c,d, xi,eta
      integer m,ii,jj

c     # These are set in either quad_square or quad_triangle
      common /comquad/ a,b,c,d

c     # this is set in compute_area_p
      common /comtri/ a00,a01,a10,a11

c     # This trilinear map assumes an local mapping in (xi,eta)
c     # in [0,1]x[0,1] over each mesh cell.  But args (xc,yc)
c     # are in [a,b]x[c,d].  So we map (xc,yc) to (xi,eta) below.

c     # Here, (a,b,c,d) are all values in computational space, and
c     # are stored in a common block 'comquad', above
      xi =  (xc-a)/(b-a)
      eta = (yc-c)/(d-c)

c     # Final division here assures units work out : (phys. space)/(comp. space)
      do m = 1,3
         txi(m)  = (a01(m) + a11(m)*eta)/(b-a)
         teta(m) = (a10(m) + a11(m)*xi)/(d-c)
      enddo

      area_element = norm_cross(txi,teta)

      end

      subroutine compute_area2(mx,my,xd,yd,zd,ap)
      implicit none

      integer mx,my

      double precision xd(mx+1,my+1)
      double precision yd(mx+1,my+1)
      double precision zd(mx+1,my+1)

      double precision ap(0:mx+1,0:my+1)

      integer i,j,ii,jj
      double precision quad(0:1,0:1,3)
      double precision get_area_approx

c     # Primary cells.  Note that we don't do anything special
c     # in the diagonal cells - so the areas there are less accurate
c     # than in the rest of the mesh.
      open(10,file='area_new.out')
      do j = 1,my
         do i = 1,mx
c           # Approximate surfaces of the cells using a trilinear
c           # map
            do ii = 0,1
               do jj = 0,1
                  quad(ii,jj,1) = xd(i+ii,j+jj)
                  quad(ii,jj,2) = yd(i+ii,j+jj)
                  quad(ii,jj,3) = zd(i+ii,j+jj)
               enddo
            enddo
            ap(i,j) = get_area_approx(quad)
            write(10,120) i,j,ap(i,j)
         enddo
      enddo
  120 format(2I5,E24.16)
      close(10)
      end


c     # This is the area element based on a bilinear approximation to
c     # the surface defined by the corners of the mesh cell.
      double precision function get_area_approx(quad)
      implicit none

      double precision quad(0:1,0:1,3)

      double precision a00(3), a01(3),a10(3), a11(3)
      double precision Txi(3), Teta(3)
      double precision norm_cross, xi,eta

      integer m

c     # Coefficients for bilinear approximation to surface
      do m = 1,3
         a00(m) = quad(0,0,m)
         a01(m) = quad(1,0,m) - quad(0,0,m)
         a10(m) = quad(0,1,m) - quad(0,0,m)
         a11(m) = quad(1,1,m) - quad(1,0,m) -
     &         quad(0,1,m) + quad(0,0,m)
      enddo

      eta = 0.5d0
      xi  = 0.5d0

c     # Mesh square is approximated by
c     #       T(xi,eta) = a00 + a01*xi + a10*eta + a11*xi*eta
c     #
c     # Area is the length of the cross product of T_xi and T_eta.
c     # computed at the center of the mesh cell.
      do m = 1,3
         txi(m)  = (a01(m) + a11(m)*eta)
         teta(m) = (a10(m) + a11(m)*xi)
      enddo

      get_area_approx = norm_cross(txi,teta)

      end


c     # ----------------------------------------------------------------
c     # Setup the coefficients needed for the boundary conditions operator
c     # ----------------------------------------------------------------
      subroutine compute_lb_coeffs(mx,my,xp,yp,zp,xd,yd,zd,
     &      coeffs_p)
      implicit none

      integer mx,my
      double precision xp(0:mx+1,0:my+1)
      double precision yp(0:mx+1,0:my+1)
      double precision zp(0:mx+1,0:my+1)

      double precision xd(mx+1,my+1)
      double precision yd(mx+1,my+1)
      double precision zd(mx+1,my+1)

      double precision coeffs_p(0:mx+1,0:my+1,2,2)

      integer i,j,k,m
      double precision taup_head(3),taup_tail(3), taup(3), sp
      double precision taud_head(3),taud_tail(3), taud(3), sd
      double precision a1, a2, ca, sa, cta

      integer k1, k2

c     % Loop over each primary cell
      do j = 1,my+1
         do i = 1,mx+1
            taud_head(1) = xp(i,j)
            taud_head(2) = yp(i,j)
            taud_head(3) = zp(i,j)

            if (i .le. mx .and. j .le. my) then
               k1 = 1
               k2 = 2
            elseif (i .le. mx .and. j .eq. my+1) then
               k1 = 2
               k2 = 2
            else
               k1 = 1
               k2 = 1
            endif

            do k = k1,k2
               if (k .eq. 1) then
                  taud_tail(1) = xp(i-1,j)
                  taud_tail(2) = yp(i-1,j)
                  taud_tail(3) = zp(i-1,j)

                  taup_head(1) = xd(i,j+1)
                  taup_head(2) = yd(i,j+1)
                  taup_head(3) = zd(i,j+1)
               else
                  taud_tail(1) = xp(i,j-1)
                  taud_tail(2) = yp(i,j-1)
                  taud_tail(3) = zp(i,j-1)

                  taup_head(1) = xd(i+1,j)
                  taup_head(2) = yd(i+1,j)
                  taup_head(3) = zd(i+1,j)
               endif

               taup_tail(1) = xd(i,j)
               taup_tail(2) = yd(i,j)
               taup_tail(3) = zd(i,j)

               sp = 0
               sd = 0
               do m = 1,3
                  taup(m) = taup_head(m) - taup_tail(m)
                  taud(m) = taud_head(m) - taud_tail(m)

                  sp = sp + taup(m)*taup(m)
                  sd = sd + taud(m)*taud(m)
               enddo
               sp = sqrt(sp)
               sd = sqrt(sd)

               ca = (taup(1)*taud(1) + taup(2)*taud(2) +
     &               taup(3)*taud(3))/(sp*sd)
               sa = sqrt(1.d0 - ca**2)
               cta = ca/sa

               a1 = 1.d0/sa
               a2 = -cta

               coeffs_p(i,j,k,1) = (sp/sd)*a1
               coeffs_p(i,j,k,2) = a2
            enddo
         enddo
      enddo

      end


      subroutine compute_coeffs2(mx,my,xp,yp,zp,xd,yd,zd,coeffs)
      implicit none

      integer mx,my
      double precision xd(mx+1,my+1)
      double precision yd(mx+1,my+1)
      double precision zd(mx+1,my+1)

      double precision xp(0:mx+1,0:my+1)
      double precision yp(0:mx+1,0:my+1)
      double precision zp(0:mx+1,0:my+1)

      double precision coeffs(0:mx+1,0:my+1,2,2)

      integer i,j,m
      double precision taud(3),taup(3), sp
      double precision nv(3), w(3)
      double precision wlen, tlen
      double precision c1, c2

c     # Compute normals at all interior edges.

c     # Get x-face normals
      do j = 1,my
         do i = 1,mx+1
            taud(1) = xp(i,j) - xp(i-1,j)
            taud(2) = yp(i,j) - yp(i-1,j)
            taud(3) = zp(i,j) - zp(i-1,j)

            taup(1) = xd(i,j+1) - xd(i,j)
            taup(2) = yd(i,j+1) - yd(i,j)
            taup(3) = zd(i,j+1) - zd(i,j)

c           # sp is the length of taup
            call get_coeffs(taup,taud,c1,c2)
            coeffs(i,j,1,1) = c1
            coeffs(i,j,1,2) = c2
         enddo
      enddo


      do j = 1,my+1
         do i = 1,mx
c           # Now do y-faces
            taud(1) = xp(i,j) - xp(i,j-1)
            taud(2) = yp(i,j) - yp(i,j-1)
            taud(3) = zp(i,j) - zp(i,j-1)

            taup(1) = xd(i+1,j) - xd(i,j)
            taup(2) = yd(i+1,j) - yd(i,j)
            taup(3) = zd(i+1,j) - zd(i,j)

            call get_coeffs(taup,taud,c1,c2)
            coeffs(i,j,2,1) = c1
            coeffs(i,j,2,2) = c2
         enddo
      enddo


      end

      subroutine get_coeffs(taup,taud,c1,c2)
      implicit none

      double precision taup(3),taud(3)

      integer m
      double precision nlen,aii,ai2,c1,c2, nv(3)
      double precision sp

c     # we leave out any scaling by the determinant of the metric,
c     # since we just normalize the vector anyway.
      aii = 0
      ai2 = 0
      do m = 1,3
         aii = aii + taup(m)*taup(m)
         ai2 = ai2 - taud(m)*taup(m)
      enddo

      nlen = 0
      do m = 1,3
         nv(m) = aii*taud(m) + ai2*taup(m)
         nlen = nlen + nv(m)*nv(m)
      enddo
      nlen = sqrt(nlen)
      sp = sqrt(aii)

c     # Multiply by length of primal edge to approximae
c     # integral
      c1 = sp*aii/nlen
      c2 = sp*ai2/nlen

      end

c     # ---------------------------------------------------------
c     # Utility routines used by both problems with physical
c     # boundaries and problems on the sphere.
c     # ---------------------------------------------------------

      subroutine ps_copy_from_claw(maxmx, maxmy, mbc,meqn,mx,my,
     &      q_claw, q)
      implicit none

      integer maxmx, maxmy, mbc,meqn,mx,my,nv
      double precision q_claw(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc,meqn)
      double precision q(0:mx+1,0:my+1,*)

      integer i,j,m,mq
      logical active_parabolic_term
      integer q_idx

      do mq = 1,meqn
         if (active_parabolic_term(mq)) then
            m = q_idx(mq)
            do j = 0,my+1
               do i = 0,mx+1
                  q(i,j,m) = q_claw(i,j,mq)
               enddo
            enddo
         endif
      enddo

      end

      subroutine ps_copy_to_claw(maxmx, maxmy, mbc,meqn,mx,my,
     &      q,q_claw)
      implicit none

      integer maxmx, maxmy, mbc,meqn,mx,my, nv
      double precision q(0:mx+1,0:my+1,*)
      double precision q_claw(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc,meqn)

      integer i,j, m,mq
      logical active_parabolic_term
      integer q_idx

      do mq = 1,meqn
         if (active_parabolic_term(mq)) then
            m = q_idx(mq)
            do j = 0,my+1
               do i = 0,mx+1
                  q_claw(i,j,mq) = q(i,j,m)
               enddo
            enddo
         endif
      enddo

      end

      subroutine write_line(iunit,len)
      implicit none

      integer iunit,len,i
      character(1000) line

      do i = 1,len
         line(i:i) = '-'
      enddo

      write(iunit,'(A)') line(1:len)

      end


      logical function has_parabolic_term(mq)
      implicit none

      integer mq
      include "para.i"

      has_parabolic_term = plist_com(mq)

      end

      logical function active_parabolic_term(mq)
      implicit none

      integer mq
      include "para.i"

      active_parabolic_term = map_active_com(mq) .gt. 0

      end

      integer function q_idx(mq)
      implicit none

      integer mq
      logical active_parabolic_term
      include "para.i"

      if (active_parabolic_term(mq)) then
         q_idx = map_active_com(mq)
      else
         write(6,100) 'q_idx : Equation ''mq'' is not an ',
     &         'active term; mq = ', mq
         stop
      endif
  100 format(A,A,I2)

      end

      integer function mq_idx(meqn,m_in)
      implicit none

      integer m_in, m, mq, meqn, q_idx
      logical active_parabolic_term

      do mq = 1,meqn
         if (active_parabolic_term(mq)) then
            m = q_idx(mq)
            if (m .eq. m_in) then
               mq_idx = mq
               return
            endif
         endif
      enddo
      write(6,*) 'mq_idx : Index m not in list of active terms; m = ',m
      stop

      end

      subroutine set_nv(meqn,nv,ntotal)
      implicit none

      integer meqn, ntotal
      logical nv(meqn)

      integer m, mq
      logical has_parabolic_term

      include 'para.i'

      m = 0
      do mq = 1,meqn
         map_active_com(mq) = 0
         if (nv(mq) .and. .not. has_parabolic_term(mq)) then
            write(6,100) 'setup_parabolic_step : There is no ',
     &            'parabolic term in equation ', mq
            write(6,*) ' '
            stop
         endif
         if (nv(mq)) then
            m = m + 1
            map_active_com(mq) = m
         endif
      enddo
      ntotal = m
  100 format(A,A,I2)

      end


c     # These are kept separate from RKC counters above, since I wanted to
c     # use this for all time stepping routines.
      subroutine initialize_fcounter()
      implicit none
      include 'para.i'

      nfevals_total = 0

      end

      subroutine increment_fcounter()
      implicit none

      include 'para.i'

      nfevals_total = nfevals_total + 1

      end


      integer function get_fcounter()
      implicit none

      include 'para.i'

      get_fcounter = nfevals_total

      end



      subroutine set_disk()
      implicit none
      logical isdisk_com, ishemisphere_com,issphere_com
      common /comsurf1/ isdisk_com
      common /comsurf2/ ishemisphere_com
      common /comsurf3/ issphere_com

      isdisk_com = .true.
      ishemisphere_com = .false.
      issphere_com = .false.
      end

      subroutine set_hemisphere()
      implicit none
      logical isdisk_com, ishemisphere_com,issphere_com
      common /comsurf1/ isdisk_com
      common /comsurf2/ ishemisphere_com
      common /comsurf3/ issphere_com

      isdisk_com = .false.
      ishemisphere_com = .true.
      issphere_com = .false.
      end

      subroutine set_sphere()
      implicit none
      logical isdisk_com, ishemisphere_com,issphere_com
      common /comsurf1/ isdisk_com
      common /comsurf2/ ishemisphere_com
      common /comsurf3/ issphere_com

      isdisk_com = .false.
      ishemisphere_com = .false.
      issphere_com = .true.
      end


      logical function isdisk()
      implicit none
      logical isdisk_com
      common /comsurf1/ isdisk_com
      isdisk = isdisk_com
      end

      logical function ishemisphere()
      implicit none
      logical ishemisphere_com
      common /comsurf2/ ishemisphere_com
      ishemisphere = ishemisphere_com
      end

      logical function issphere()
      implicit none
      logical issphere_com
      common /comsurf3/ issphere_com
      issphere = issphere_com
      end
