      subroutine bicg(mx,my,meqn,mpara,fp,qp,lambda, maxiter,tol,
     &      work,mwork,prt_info,ierror)
      implicit none

      integer mx,my,mwork,mpara,meqn
      integer maxiter,ierror,prt_info(2)
      double precision tol,lambda

      double precision qp(0:mx+1,0:my+1,mpara)
      double precision fp(0:mx+1,0:my+1,mpara)

      double precision work(mwork)

      double precision alpha, beta, rho, rho_1, rho_2, a
      double precision normb,normr, resid
      double precision pq, omega, norms, rv, tt,ts

      double precision dot_bicg, norm_bicg

      integer pp,rp,qqp,ap,rtp,phatp,vp,sp,tp, shatp

      logical ishomogeneous
      integer k, prt_mod
      logical prt,prt_file

c     # prt_info(1) = 0 : don't print anything
c     # prt_info(1) = 1 : print to screen
c     # prt_info(1) = 2 : print to file (its_cg.out)
c     # prt_info(1) = 3 : print to both file and screen
c
c     # prt_info(2) = n : print every n outputs to screen
c
      prt = prt_info(1) .eq. 1 .or. prt_info(1) .eq. 3
      prt_file = prt_info(1) .eq. 2 .or. prt_info(1) .eq. 3
      prt_mod = prt_info(2)

      call get_bicg_ptrs(mwork,mx,my,mpara,pp,rp,
     &      ap,rtp,phatp,vp,sp,tp,shatp)

      normb = norm_bicg(mx,my,mpara,fp)
      if (normb .le. 1e-12) then
         normb = 1
      endif

      ishomogeneous = .false.
      call matvec(mx,my,meqn,qp,lambda,work(Ap),ishomogeneous)

c     # Do r = -1*Ax + b
      a = -1.d0
      call axpy0_bicg(mx,my,mpara,a,work(Ap),fp,work(rp))

      normr = norm_bicg(mx,my,mpara,work(rp))

      resid = normr/normb
      if (resid .le. tol) then
         tol = resid
         maxiter = 0
         ierror = 0
         return
      endif

      call copy_bicg(mx,my,mpara,work(rp),work(rtp))

      if (prt) then
         write(6,*) '---------------------------'
         write(6,'(A)') '  k        Resid. '
         write(6,*) '---------------------------'
         write(6,1001) 1, normr
      endif
      if (prt_file) then
         open(10,file='its_bicg.out')
         write(10,1001) 1,normr
      endif

c     # Idea is that unless we exit for some other reason,
c     # we will get ierror = 1 (maxits exceeded).
      ishomogeneous = .true.
      ierror = 0
      do k = 1,maxiter
         rho_1 = dot_bicg(mx,my,mpara,work(rtp),work(rp))
         if (rho_1 .eq. 0) then
            resid = normr/normb
            ierror = 2
            exit
         endif

         if (k .eq. 1) then
            call copy_bicg(mx,my,mpara,work(rp),work(pp))
         else
            beta = (rho_1/rho_2)*(alpha/omega)
c           # p = -omega*v + p
            a = -omega
            call axpy2_bicg(mx,my,mpara,a,work(vp),work(pp))

c           # p = beta*p + r
            a = beta
            call axpy1_bicg(mx,my,mpara,a,work(pp),work(rp))
         endif

c        # Preconditioning step here...
         call copy_bicg(mx,my,mpara,work(pp),work(phatp))


c        # First matrix-vector multpiply
         call matvec(mx,my,meqn,work(phatp),lambda,work(vp),
     &         ishomogeneous)

         rv = dot_bicg(mx,my,mpara,work(rtp),work(vp))
         alpha = rho_1/rv


c        # s = -alpha*v + r
         a = -alpha
         call axpy0_bicg(mx,my,mpara,a,work(vp),work(rp),work(sp))

         norms = norm_bicg(mx,my,mpara,work(sp))
         resid = norms/normb

         if (resid .le. tol) then
c           # x = alpha*phat + x
            a = alpha
            call axpy2_bicg(mx,my,mpara,alpha,work(phatp),qp)
            exit
         endif

c        # Second preconditioning step
         call copy_bicg(mx,my,mpara,work(sp),work(shatp))

c        # Second matrix-vector multpiply
         call matvec(mx,my,meqn,work(shatp),lambda,work(tp),
     &         ishomogeneous)
         ts = dot_bicg(mx,my,mpara,work(tp),work(sp))
         tt = dot_bicg(mx,my,mpara,work(tp),work(tp))

         omega = ts/tt
c        # x = x + alpha*phat + omega*shat
c        # x* = alpha*phat + x
c        # x  = omega*shat + x*
         a = alpha
         call axpy2_bicg(mx,my,mpara,a,work(phatp),qp)
         a = omega
         call axpy2_bicg(mx,my,mpara,a,work(shatp),qp)

c        # r = -omega*t + s
         a = -omega
         call axpy0_bicg(mx,my,mpara,a,work(tp),work(sp),work(rp))

         rho_2 = rho_1

         normr = norm_bicg(mx,my,mpara,work(rp))
         resid = normr/normb
         if (prt) then
            if (mod(k,prt_mod) .eq. 0) then
               write(6,1001) k, resid
            endif
         endif
         if (prt_file) then
            write(10,1001) k,resid
         endif

         if (resid .le. tol) then
            ierror = 0
            exit
         endif

         if (omega .eq. 0) then
            resid = normr/normb
            exit
         endif
      enddo
      tol = resid
      if (k .ge. maxiter) then
         write(6,*) 'maxiter = ', maxiter
         ierror = 1
      endif
      maxiter = k

      if (prt) then
         write(6,'(A)') '-----------------------------'
         write(6,1001) k, resid
         write(6,*) ' '
      endif
      if (prt_file) then
         write(10,1001) k, resid
         close(10)
      endif

      if (prt_file) then
         write(10,1001) k, resid
         close(10)
      endif

      if (ierror .eq. 0) then
      elseif (ierror .eq. 1) then
         if (prt) then
            write(6,*) 'Maximum number of iterations exceeded'
         endif
      elseif (ierror .ge. 2) then
         if (prt) then
            write(6,'(A)') 'Bad exit'
         endif
      endif


c      close(10)
c      tol = resid
c      ierror = 1

 1001 format(I6,E16.8)

      end

      double precision function norm_bicg(mx,my,mpara,xp)
      implicit none

      integer mx,my,mpara
      double precision xp(0:mx+1,0:my+1,mpara)
      double precision scale, ssq, absai, aij
      double precision max_xp,min_xp
      logical novalue

      integer i,j, m


      scale = 0
      ssq = 1.d0

      do m = 1,mpara
         do j = 1,my
            do i = 1,mx
               aij = xp(i,j,m)
               if (aij .ne. 0) then
                  absai = abs(aij)
                  if (scale .lt. absai) then
                     ssq = 1.d0 + ssq*(scale/absai)**2
                     scale = absai
                  else
                     ssq = ssq + (absai/scale)**2
                  endif
               endif
            enddo
         enddo
      enddo

      norm_bicg = scale*sqrt(ssq)

      end

      subroutine axpy0_bicg(mx,my,mpara,a,xp,yp,rp)
      implicit none

      integer mx,my, mpara
      double precision xp(0:mx+1,0:my+1,mpara)
      double precision yp(0:mx+1,0:my+1,mpara)
      double precision rp(0:mx+1,0:my+1,mpara)

      double precision a
      logical novalue

      integer  i,j, m

      do m = 1,mpara
         do j = 1,my
            do i = 1,mx
               rp(i,j,m) = a*xp(i,j,m) + yp(i,j,m)
            enddo
         enddo
      enddo

      end


      subroutine axpy1_bicg(mx,my,mpara,a,xp,yp)
      implicit none

      integer mx,my,mpara
      double precision a
      double precision xp(0:mx+1,0:my+1,mpara)
      double precision yp(0:mx+1,0:my+1,mpara)

      integer  i,j, m

      do m = 1,mpara
         do j = 1,my
            do i = 1,mx
               xp(i,j,m) = a*xp(i,j,m) + yp(i,j,m)
            enddo
         enddo
      enddo

      end

      subroutine axpy2_bicg(mx,my,mpara,a,xp,yp)
      implicit none

      integer mx,my, mpara
      double precision a
      double precision xp(0:mx+1,0:my+1,mpara)
      double precision yp(0:mx+1,0:my+1,mpara)

      integer  i,j,m

      do m = 1,mpara
         do j = 1,my
            do i = 1,mx
               yp(i,j,m) = a*xp(i,j,m) + yp(i,j,m)
            enddo
         enddo
      enddo

      end


      subroutine copy_bicg(mx,my,mpara,xp,yp)
      implicit none

      integer mx,my, mpara
      double precision xp(0:mx+1,0:my+1,mpara)
      double precision yp(0:mx+1,0:my+1,mpara)

      integer i,j,m

      do m = 1,mpara
         do j = 1,my
            do i = 1,mx
               yp(i,j,m) = xp(i,j,m)
            enddo
         enddo
      enddo

      end

      double precision function dot_bicg(mx,my,mpara,xp,yp)
      implicit none

      integer mx,my,mpara
      double precision xp(0:mx+1,0:my+1,mpara)
      double precision yp(0:mx+1,0:my+1,mpara)

      integer i,j,m
      double precision sum
      logical novalue

      sum = 0
      do m = 1,mpara
         do j = 1,my
            do i = 1,mx
               sum = sum + xp(i,j,m)*yp(i,j,m)
            enddo
         enddo
      enddo

      dot_bicg = sum

      end

      subroutine get_bicg_ptrs(mwork,mx,my,mpara,pp,rp,
     &      ap,rtp,phatp,vp,sp,tp,shatp)
      implicit none

      integer mwork,mx,my, mpara
      integer pp,rp,rd,ap,rtp,phatp,vp,sp,tp,shatp

      integer Np,total

      Np = (mx+2)*(my+2)

      pp    = 1
      rp    = pp       + Np*mpara
      ap    = rp       + Np*mpara
      rtp   = ap       + Np*mpara
      phatp = rtp      + Np*mpara
      vp    = phatp    + Np*mpara
      sp    = vp       + Np*mpara
      tp    = sp       + Np*mpara
      shatp = tp       + Np*mpara

      total = shatp    + Np*mpara

      total = total - 1

      if (mwork .lt. total) then
         write(6,*) 'bicg : Increase size of work_bicg to ', total
         stop
      endif

      end

      integer function get_bicg_mem(mx,my,mpara)
      implicit none

      integer mx,my,mpara, np

      Np = (mx+2)*(my+2)

      get_bicg_mem = Np*mpara*9

      end
