c     # --------------------------------------------------------------
c     # Right hand side for explicit time step routines
c     # --------------------------------------------------------------
      subroutine f_exp(neqn,t,q,rhs)
      implicit none

      integer neqn
      double precision t, q(neqn), rhs(neqn)

      integer mx,my,meqn,mbc
      logical ishomogeneous

      include "para.i"

      ishomogeneous = .false.

      mx = mx_com
      my = my_com

      meqn = meqn_com
      mbc = mbc_com

c     # Compute dual cell values (user defined)
      call compute_dual_phys(mx,my,meqn,q,work(qd_ptr))

c     # Apply boundary conditions, updating 'c_bc' if it is
c     # time dependent
      call apply_bc(mx,my,meqn,q,work(qd_ptr),t,ishomogeneous)

c     # Compute Laplacian using primal and dual values
c     # (user defined)
      call apply_LB(mx,my,mbc,meqn,work(coeffs_ptr),work(area_ptr),
     &      q,work(qd_ptr),work(diff_ptr))

      call src_reaction(mx,my,meqn,t,q,work(reac_ptr))
      call src_diffusion(mx,my,meqn,t,q,work(diff_ptr))

      call compute_rhs_exp(mx,my,meqn,work(diff_ptr),
     &      work(reac_ptr),rhs)

c     # Zero out rhs ghost cell values so RKC won't update them
      call set_zeros(mx,my,meqn,rhs)


      end

      subroutine compute_rhs_exp(mx,my,meqn,diff,reac,rhs)
      implicit none

      integer mx,my,meqn
      double precision diff(0:mx+1,0:my+1,*)
      double precision reac(0:mx+1,0:my+1,*)
      double precision rhs(0:mx+1,0:my+1,*)

      integer i,j,mq,q_idx,m
      logical active_parabolic_term

      do mq = 1,meqn
         if (active_parabolic_term(mq)) then
            m = q_idx(mq)
            do j = 1,my
               do i = 1,mx
                  rhs(i,j,m) = diff(i,j,m) + reac(i,j,m)
               enddo
            enddo
         endif
      enddo

      end

c     # --------------------------------------------------------------
c     # IRKC
c     # --------------------------------------------------------------
      subroutine f_exp_irkc(neqn,t,q,rhs)
      implicit none

      integer neqn
      double precision t, q(neqn), rhs(neqn)

      integer mx,my,meqn,mbc,nactive,m
      logical ishomogeneous

      include "para.i"

      ishomogeneous = .false.

      mx = mx_com
      my = my_com
      meqn = meqn_com
      mbc = mbc_com
      nactive = nactive_com

c     # we have to copy to a new array, since the ordering for IRKC is
c     # different than for all other routines.
      call copy_to_work_irkc(mx,my,meqn,nactive,q,work(q_ptr))
      call copy_to_work_irkc(mx,my,meqn,nactive,rhs,work(rhs_ptr))

      call compute_dual_phys(mx,my,meqn,work(q_ptr),work(qd_ptr))

      call apply_bc(mx,my,meqn,work(q_ptr),work(qd_ptr),t,ishomogeneous)

      call apply_LB(mx,my,mbc,meqn,work(coeffs_ptr),work(area_ptr),
     &      work(q_ptr),work(qd_ptr),work(diff_ptr))

c     # No reaction term here, since it is handled implicitly
      call src_diffusion(mx,my,meqn,t,q,work(diff_ptr))

      call set_zeros(mx,my,meqn,work(diff_ptr))

      call copy_from_work_irkc(mx,my,meqn,nactive,work(diff_ptr),rhs)

      end


      subroutine f_imp_irkc(ieqn,nactive,t,q,rhs,compute_jac,jac)
      implicit none

      integer ieqn, nactive
      double precision t, q(nactive), rhs(nactive)
      double precision jac(nactive,nactive)
      logical compute_jac

      integer mx,my

      integer i,j,m, np, r1,ii,jj, n

      include "para.i"

      mx = mx_com
      my = my_com

c     # For ieqn = 1,P+1,2*P+1, 3*P+1, where P = nactive  = number of PDEs
c     #
c     # ieqn = (N-1)*P + 1
c     # N = (mx+2-1)*j + i

      n = (ieqn-1)/nactive + 1

      i = mod(n-1,mx+2) + 1
      j = (n - i)/(mx+2) + 1

c     # Subtract 1, since we grid points start at (0,0)
      i = i - 1
      j = j - 1

      if (i .eq. 0 .OR. i .eq. mx+1 .or. j .eq. 0 .or. j .eq. my+1) then
         do m = 1,nactive
            rhs(m) = 0
         enddo
         do ii = 1,nactive
            do jj = 1,nactive
               jac(ii,jj) = 0.d0
            enddo
         enddo
         return
      endif

      call src_reaction_irkc(i,j,nactive,t,q,rhs,compute_jac,jac)

      end

c     # --------------------------------------------------------------
c     # Crank-Nicolson
c     # --------------------------------------------------------------
      subroutine compute_rhs_cn(mx,my,meqn,qn,t,dt,diff,reac,rhs)
      implicit none

      integer mx,my,meqn
      double precision t,dt
      double precision qn(0:mx+1,0:my+1,*)
      double precision rhs(0:mx+1,0:my+1,*)
      double precision diff(0:mx+1,0:my+1,*)
      double precision reac(0:mx+1,0:my+1,*)

      integer i,j,m, mq, q_idx
      logical active_parabolic_term
      double precision lambda, tp
      integer mbc
      logical ishomogeneous

      include 'para.i'

      ishomogeneous = .false.
      mbc = mbc_com

c     # Compute dual cell values (user defined)
      call compute_dual_phys(mx,my,meqn,qn,work(qd_ptr))

      call apply_bc(mx,my,meqn,qn,work(qd_ptr),t,ishomogeneous)

c     # Compute Laplacian using primal and dual values
c     # (user defined)
      call apply_LB(mx,my,mbc,meqn,work(coeffs_ptr),
     &      work(area_ptr),qn,work(qd_ptr),diff)

      call src_diffusion(mx,my,meqn,t,qn,diff)
c     # Time center the source term for Crank-Nicolson
      tp = t + dt/2.d0
      call src_reaction(mx,my,meqn,tp,qn,reac)

      lambda = -2.d0/dt
      do mq = 1,meqn
         if (active_parabolic_term(mq)) then
            m = q_idx(mq)
            do j = 1,my
               do i = 1,mx
                  rhs(i,j,m) = lambda*qn(i,j,m) - diff(i,j,m) -
     &                  2.d0*reac(i,j,m)
               enddo
            enddo
         endif
      enddo

      end

c     # --------------------------------------------------------------
c     # Matrix vector multiply for CN, TR-BDF2 and BE.
c     # --------------------------------------------------------------
      subroutine matvec(mx,my,meqn,q,lambda,Aq,ishomogeneous)
      implicit none

      integer mx,my,meqn
      double precision q(0:mx+1,0:my+1,*)
      double precision Aq(0:mx+1,0:my+1,*)
      double precision lambda
      logical ishomogeneous

      double precision t_curr, dt_curr, tp

      integer i,j,m,mq,q_idx,mbc
      logical active_parabolic_term

      include 'para.i'

      t_curr = current_time_com
      dt_curr = current_dt_com
      mbc = mbc_com

c     # Compute dual cell values (user defined)
      call compute_dual_phys(mx,my,meqn,q,work(qd_ptr))

      call apply_bc(mx,my,meqn,q,work(qd_ptr),t_curr,ishomogeneous)

c     # Compute Laplacian using primal and dual values
      call apply_LB(mx,my,mbc,meqn,work(coeffs_ptr),work(area_ptr),q,
     &      work(qd_ptr),Aq)

      call src_diffusion(mx,my,meqn,t_curr,q,Aq)

      do mq = 1,meqn
         if (active_parabolic_term(mq)) then
            m = q_idx(mq)
            do j = 1,my
               do i = 1,mx
                  Aq(i,j,m) = Aq(i,j,m) + lambda*q(i,j,m)
               enddo
            enddo
         endif
      enddo

      call set_zeros(mx,my,meqn,Aq)

      end

      subroutine print_q(mx,my,meqn,qp,qd)
      implicit none

      integer mx,my,meqn
      double precision qp(0:mx+1,0:my+1,*)
      double precision qd(1:mx+1,1:my+1,*)

      integer i,j,m, mq, q_idx
      logical active_parabolic_term

      do mq = 1,meqn
         if (active_parabolic_term(mq)) then
            m = q_idx(mq)
            do j = 1,my
               do i = 1,mx
                  write(6,120) i,j,m,qp(i,j,m)
               enddo
            enddo
            write(6,*) ' '
         endif
      enddo
  120 format(3I5,2E16.8)


      end
