      subroutine compute_coeffs(mx,my,mbc,xp,yp,zp,xd,yd,zd,
     &      coeffs)
      implicit none

      integer mx,my,mbc

      double precision xp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision yp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision zp(-mbc:mx+mbc+1,-mbc:my+mbc+1)

      double precision xd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
      double precision yd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
      double precision zd(-mbc:mx+mbc+2,-mbc:my+mbc+2)

      double precision coeffs(0:mx+1,0:my+1,2,2)

      integer i,j,m
      double precision taud(3),taup(3), sp
      double precision nv(3), w(3)
      double precision wlen, tlen
      double precision c1, c2

c     # Get x-face normals
      do j = 1,my
         do i = 1,mx+1
            taud(1) = xp(i,j) - xp(i-1,j)
            taud(2) = yp(i,j) - yp(i-1,j)
            taud(3) = zp(i,j) - zp(i-1,j)

            taup(1) = xd(i,j+1) - xd(i,j)
            taup(2) = yd(i,j+1) - yd(i,j)
            taup(3) = zd(i,j+1) - zd(i,j)

c           # sp is the length of taup
            call get_coeffs(taup,taud,c1,c2,sp)
            coeffs(i,j,1,1) = c1
            coeffs(i,j,1,2) = c2
         enddo
      enddo


      do j = 1,my+1
         do i = 1,mx
c           # Now do y-faces
            taud(1) = xp(i,j) - xp(i,j-1)
            taud(2) = yp(i,j) - yp(i,j-1)
            taud(3) = zp(i,j) - zp(i,j-1)

            taup(1) = xd(i+1,j) - xd(i,j)
            taup(2) = yd(i+1,j) - yd(i,j)
            taup(3) = zd(i+1,j) - zd(i,j)

            call get_coeffs(taup,taud,c1,c2,sp)
            coeffs(i,j,2,1) = c1
            coeffs(i,j,2,2) = c2
         enddo
      enddo


      end

      subroutine get_coeffs(taup,taud,c1,c2,sp)
      implicit none

      double precision taup(3),taud(3), sp

      integer m
      double precision nlen,aii,ai2,c1,c2, nv(3)

c     # we leave out any scaling by the determinant of the metric,
c     # since we just normalize the vector anyway.
      aii = 0
      ai2 = 0
      do m = 1,3
         aii = aii + taup(m)*taup(m)
         ai2 = ai2 - taud(m)*taup(m)
      enddo

      nlen = 0
      do m = 1,3
         nv(m) = aii*taud(m) + ai2*taup(m)
         nlen = nlen + nv(m)*nv(m)
      enddo
      nlen = sqrt(nlen)
      sp = sqrt(aii)

c     # Multiply by length of primal edge to approximate
c     # integral
      c1 = sp*aii/nlen
      c2 = sp*ai2/nlen

      end

c     # ---------------------------------------------------------
c     # Utility routines used by both problems with physical
c     # boundaries and problems on the sphere.
c     # ---------------------------------------------------------

      subroutine ps_copy_from_claw(maxmx, maxmy, mbc,meqn,mx,my,
     &      q_claw, q)
      implicit none

      integer maxmx, maxmy, mbc,meqn,mx,my,nv
      double precision q_claw(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc,meqn)
      double precision q(0:mx+1,0:my+1,*)

      integer i,j,m,mq
      logical active_parabolic_term
      integer q_idx

      do mq = 1,meqn
         if (active_parabolic_term(mq)) then
            m = q_idx(mq)
            do j = 0,my+1
               do i = 0,mx+1
                  q(i,j,m) = q_claw(i,j,mq)
               enddo
            enddo
         endif
      enddo

      end

      subroutine ps_copy_to_claw(maxmx, maxmy, mbc,meqn,mx,my,
     &      q,q_claw)
      implicit none

      integer maxmx, maxmy, mbc,meqn,mx,my, nv
      double precision q(0:mx+1,0:my+1,*)
      double precision q_claw(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc,meqn)

      integer i,j, m,mq
      logical active_parabolic_term
      integer q_idx

      do mq = 1,meqn
         if (active_parabolic_term(mq)) then
            m = q_idx(mq)
            do j = 0,my+1
               do i = 0,mx+1
                  q_claw(i,j,mq) = q(i,j,m)
               enddo
            enddo
         endif
      enddo

      end

      logical function has_parabolic_term(mq)
      implicit none

      integer mq
      include "para.i"

      has_parabolic_term = plist_com(mq)

      end

      logical function active_parabolic_term(mq)
      implicit none

      integer mq
      include "para.i"

      active_parabolic_term = map_active_com(mq) .gt. 0

      end

      integer function q_idx(mq)
      implicit none

      integer mq
      logical active_parabolic_term
      include "para.i"

      if (active_parabolic_term(mq)) then
         q_idx = map_active_com(mq)
      else
         write(6,100) 'q_idx : Equation ''mq'' is not an ',
     &         'active term; mq = ', mq
         stop
      endif
  100 format(A,A,I2)

      end

      integer function mq_idx(meqn,m_in)
      implicit none

      integer m_in, m, mq, meqn, q_idx
      logical active_parabolic_term

      do mq = 1,meqn
         if (active_parabolic_term(mq)) then
            m = q_idx(mq)
            if (m .eq. m_in) then
               mq_idx = mq
               return
            endif
         endif
      enddo
      write(6,*) 'mq_idx : Index m not in list of active terms; m = ',m
      stop

      end

      subroutine set_nv(meqn,nv,ntotal)
      implicit none

      integer meqn, ntotal
      logical nv(meqn)

      integer m, mq
      logical has_parabolic_term

      include 'para.i'

      m = 0
      do mq = 1,meqn
         map_active_com(mq) = 0
         if (nv(mq) .and. .not. has_parabolic_term(mq)) then
            write(6,100) 'setup_parabolic_step : There is no ',
     &            'parabolic term in equation ', mq
            write(6,*) ' '
            stop
         endif
         if (nv(mq)) then
            m = m + 1
            map_active_com(mq) = m
         endif
      enddo
      ntotal = m
  100 format(A,A,I2)

      end


      subroutine set_zeros(mx,my,meqn,rhs)
      implicit none

      integer mx,my,meqn
      double precision rhs(0:mx+1,0:my+1,*)

      integer i,j,m,mq
      logical active_parabolic_term
      integer q_idx


c     # --------------------------------------------
c     # Primary cells
c     # --------------------------------------------
c     # Left and right ghost cells
      do mq = 1,meqn
         if (active_parabolic_term(mq)) then
            m = q_idx(mq)

            do j = 1,my
               rhs(0,j,m)    = 0.d0
               rhs(mx+1,j,m) = 0.d0
            enddo

c           # Bottom and top ghost cells
            do i = 1,mx
               rhs(i,0,m)    = 0.d0
               rhs(i,my+1,m) = 0.d0
            enddo

c           # corners
            rhs(0,0,m) = 0
            rhs(0,my+1,m) = 0
            rhs(mx+1,0,m) = 0
            rhs(mx+1,my+1,m) = 0
         endif
      enddo

      end
