c     # ---------------------------------------------------
c     # Simple driver routine for parabolic equations
c     #
c     # Input data is read in from file 'para.data'.
c     # ---------------------------------------------------
      subroutine parabolic_main(maxmx, maxmy,meqn,mbc,q,plist,nv)
      implicit none

      integer maxmx, maxmy, meqn, mbc
      double precision q(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc,meqn)
      logical nv(meqn)
      logical plist(meqn)

      integer mx,my
      double precision dx,dy
      double precision xlower,ylower,xupper,yupper

      logical rest

      integer nout, nstep, iframe, ierror, n
      integer verbose, get_time_step_counter, method
      double precision dt, t0, tinitial, ti
      logical cart_bc

      double precision t

      integer mq

      include "para.i"


      open(55,file='para.data')

c     domain variables
      read(55,*) mx
      read(55,*) my

c     # Read nstep,ntotal
c     # dt
      read(55,*) nstep,nout
      read(55,*) dt

      read(55,*) xlower
      read(55,*) xupper
      read(55,*) ylower
      read(55,*) yupper

      read(55,*) verbose

      read(55,*) iframe
      read(55,*) method
      close(55)


c     # Done reading in data

c     # ----------------------------------------------
c     # Basic sanity checks

      if (mx .gt. maxmx) then
         write(6,200) 'Increase size of maxmx to ', mx
         stop
      endif

      if (my .gt. maxmy) then
         write(6,200) 'Increase size of maxmy to ', my
         stop
      endif
  200 format(A,I5)

c     # ----------------------------------------------

      rest = iframe .ge. 0

c     # grid spacing
      dx = (xupper - xlower)/mx
      dy = (yupper - ylower)/my

c     # Setup map type. Uses which ever type was compiled.
c     # This allows the user to call the following functions
c     #
c     #       function logical is_cart()
c     #       function logical is_disk()
c     #       function logical is_hemisphere()
c     #       function logical is_sphere()
c     #
c     # The choice is made a compile time and depends on which version
c     # of the file containing the subroutine 'setup_map' is compiled.
c     # Available versions include
c     #
c     #        set_maptype_cart.f, set_maptype_disk.f, set_maptype_hemisphere.f
c     #        set_maptype_sphere.f
c     #

      call initialize_laplacian_call_counter()
      call initialize_timer()
      call initialize_accumulated_time()


      call set_maptype()
      call set_method(method)
      call set_verbosity(verbose)

c     # Specify whether to use Cartesian boundary conditions, i.e. BCs
c     # that do not require solving a tridiagonal system.  The user
c     # must define "apply_cart_bc" so then apply these Cartesian BCs.
c     # The user can set these later ...
      cart_bc = .false.
      call set_cart_bc(cart_bc)

c     # Read in any parameters and other problem specific info.
      call setprob()

      call setup_parabolic(mx,my,meqn,mbc,plist,method,
     &      xlower,ylower,dx,dy)

c     # Initialize solution using either data from a restart file
c     # or by calling initialization routine 'qinit'
      if (rest) then
         call parabolic_restart(maxmx,maxmy,meqn,mbc,mx,my,
     &         q,iframe,t0)
         write(6,400) iframe,t0
  400    format('Restarting from file ',I4,' at t = ',F12.4,/)
      else
         call qinit(maxmx,maxmy,meqn,mbc,mx,my,xlower,ylower,
     &         dx,dy,q)
         t0 = 0
         iframe = 0
         call out2(maxmx,maxmy,meqn,mbc,mx,my,
     &         xlower,ylower,dx,dy,q,work(area_ptr),
     &         t0,iframe)
      endif


c     # This specifies which of the parabolic terms we should solve in our
c     # call to parabolic_step.  Usually, we want to solve all of them.
      do mq = 1,meqn
         nv(mq) = plist(mq)
      enddo

      call initialize_time_step_counter()

c     # Start time stepping
      do n = 0,nout-1
         ti = t0 + dt*n
         call parabolic_step(maxmx, maxmy,meqn,mbc,mx,my,
     &      nv,q,ti,dt,ierror)
         call increment_time_step_counter()
         ti = ti + dt

         if (mod(n+1,nstep) .eq. 0) then
            iframe = iframe + 1
            call out2(maxmx,maxmy,meqn,mbc,mx,my,
     &            xlower,ylower,dx,dy,q,work(area_ptr),ti,iframe)
            write(6,100) iframe, ti, dt
  100       format('Output Frame ',I5,'; t = ',E16.4,'; dt = ',E12.4)
         endif
      enddo
      end
