      subroutine setup_parabolic(mx,my,meqn,mbc,plist,method,
     &      xlower,ylower,dx,dy)
      implicit none

      integer mx,my,meqn, method,mbc
      logical plist(meqn)

      double precision xlower,ylower,dx,dy

      include 'para.i'

c     # Set up pointers in common blocks in 'para.i'
      call setup_common_sphere(mx,my,mbc,xlower,ylower,dx,dy,meqn,
     &      plist,method)

      call setup_parabolic_geo(mx,my,mbc,xlower,ylower,dx,dy)

      call setup_parabolic_equator(mx,my,xlower,ylower,dx,dy)

      end

      subroutine setup_parabolic_geo(mx,my,mbc,xlower,ylower,dx,dy)
      implicit none

      integer mx,my,mbc
      double precision xlower,ylower,dx,dy

      include "para.i"

c     # Set up mesh.  Use pointers defined in para.i
      call setup_mesh(mx,my,mbc,xlower,ylower,dx,dy,
     &      work(xp_ptr),work(yp_ptr),work(zp_ptr),
     &      work(xd_ptr),work(yd_ptr),work(zd_ptr))

c     # Don't seem to need this call
c      call match_mesh(mx,my,mbc,work(xp_ptr),work(xd_ptr))
c      call match_mesh(mx,my,mbc,work(yp_ptr),work(yd_ptr))
c      call match_mesh(mx,my,mbc,work(zp_ptr),work(zd_ptr))

c     # Set up metric terms.
      call compute_area(mx,my,mbc,
     &      work(xd_ptr),work(yd_ptr),work(zd_ptr),
     &      work(area_ptr))

      call compute_coeffs(mx,my,mbc,
     &      work(xp_ptr),work(yp_ptr),work(zp_ptr),
     &      work(xd_ptr),work(yd_ptr),work(zd_ptr),
     &      work(coeffs_ptr))

      call setup_boundary_mesh(mx,my,mbc,xlower,ylower,dx,dy,
     &      work(xp_ptr),work(yp_ptr),work(zp_ptr),
     &      work(xd_ptr),work(yd_ptr),work(zd_ptr),
     &      work(xpi_ptr), work(ypi_ptr), work(zpi_ptr),
     &      work(xdi_ptr), work(ydi_ptr), work(zdi_ptr))
      end


      subroutine setup_parabolic_equator(mx,my,xlower,ylower,dx,dy)
      implicit none

      integer mx,my
      double precision xlower,ylower,dx,dy

      integer ldtri

      include "para.i"

      ldtri = ldtri_com

      call initialize_equator(mx,my,xlower,ylower,dx,dy,
     &      work(xpi_ptr), work(ypi_ptr), work(zpi_ptr),
     &      work(xdi_ptr), work(ydi_ptr), work(zdi_ptr),
     &      work(work_tri_ptr),ldtri,ipiv)

      end

      subroutine setup_common_sphere(mx,my,mbc,xlower,ylower,dx,dy,meqn,
     &      plist,method)
      implicit none

      integer mx,my, meqn, method, mbc
      logical plist(meqn)

      double precision xlower,ylower,dx,dy
      integer np, nd, nbc, total

      integer i, m, mq, mpara, ldtri
      integer get_rkc_mem, get_trbdf_mem, get_rock4_mem
      integer get_cn_mem, get_beuler_mem, get_feuler_mem
      integer get_irkc_mem
      integer mwork_solver, method_mem(7)

      integer mbc1, np1, nd1

      include "para.i"

      nfevals_total = 0

      if (meqn .gt. maxmeqn) then
         write(6,110) 'para.i : Increase size of maxmeqn to ',
     &         meqn
         stop
      endif
  110 format(A,I2)

      m = 0
      do mq = 1,meqn
         plist_com(mq) = plist(mq)
         if (plist(mq)) then
            m = m + 1
         endif
      enddo
      mpara = m

c     # To communicate these through common blocks
      mx_com = mx
      my_com = my
      dx_com = dx
      dy_com = dy
      xlower_com = xlower
      ylower_com = ylower

      np   = (mx+2)*(my+2)
      nd   = (mx+1)*(my+1)
      nbc  = 2*(mx/2+ my)
      ldtri = 25*nbc + 6

      mpara_com = mpara
      meqn_com = meqn
      mbc_com = mbc
      ldtri_com = ldtri

c     # Needed for BiCG solver
      method_mem(1) = get_feuler_mem(mx,my,mpara)
      method_mem(2) = get_beuler_mem(mx,my,mpara)
      method_mem(3) = get_cn_mem(mx,my,mpara)
      method_mem(4) = get_trbdf_mem(mx,my,mpara)
      method_mem(5) = get_rock4_mem(mx,my,mpara)
      method_mem(6) = get_rkc_mem(mx,my,mpara)
      method_mem(7) = get_irkc_mem(mx,my,mpara)
      mwork_solver  = method_mem(method)

c     # We allow for an extra row of ghost cells, which
c     # may be overkill, but well, this is what I do...
      mbc1 = mbc + 1
      np1 = (mx+2*mbc1)*(my+2*mbc1)
      nd1 = (mx+2*mbc1 + 1)*(my+2*mbc1+1)


c     # Indices into work array
      xp_ptr           = 1
      yp_ptr           = xp_ptr         + np1
      zp_ptr           = yp_ptr         + np1
      area_ptr         = zp_ptr         + np1
      coeffs_ptr       = area_ptr       + np1
      xd_ptr           = coeffs_ptr     + 4*np
      yd_ptr           = xd_ptr         + nd1
      zd_ptr           = yd_ptr         + nd1

      xpi_ptr          = zd_ptr         + nd1
      ypi_ptr          = xpi_ptr        + 3*nbc
      zpi_ptr          = ypi_ptr        + 3*nbc

      xdi_ptr          = zpi_ptr        + 3*nbc
      ydi_ptr          = xdi_ptr        + nbc
      zdi_ptr          = ydi_ptr        + nbc

      normals_ptr      = zdi_ptr        + nbc

      qd_ptr           = normals_ptr    + 6*nd1
      work_tri_ptr     = qd_ptr         + nd*mpara
      rhs_ptr          = work_tri_ptr   + ldtri
      q_ptr            = rhs_ptr        + np*mpara
      diff_ptr         = q_ptr            + np*mpara
      reac_ptr         = diff_ptr         + np*mpara
      work_solver_ptr  = reac_ptr         + np*mpara
      total            = work_solver_ptr  + mwork_solver

      total = total - 1

      if (mwork .lt. total) then
         write(6,'(A,A,I10)') 'para.i : Increase memory ',
     &         'in work array to at least ', total
         stop
      endif

c     # Check integer array 'ipiv'.  There is only one
c     # array, so we don't need a pointer into it.
      if (miwork .lt. 2*(mx + my)) then
         write(6,100) 'para.i : Increase size of ipiv ',
     &         'to ', 2*(mx + my)
         stop
      endif
  100 format(A,A,I10)

      end


c     # -------------------------------------------------------
c     # Setup system for solving for dual values at the equator.
c     #
c     # This requires that the tridiagonal system has been
c     # set up by 'initialize_bc(...)'
c     #
c     # ----------------------------------------------------------

      subroutine apply_equator_bc(mx,my,meqn,qp,qd)
      implicit none

      integer mx,my,meqn
      double precision qp(0:mx+1,0:my+1,*)
      double precision qd(mx+1,my+1,*)

      integer m,mq
      logical active_parabolic_term
      integer q_idx, ldtri

      include "para.i"

      ldtri = ldtri_com

      call apply_equator(mx,my,meqn,work(work_tri_ptr),
     &      ldtri,ipiv,qp,qd)
      end

      subroutine apply_equator(mx,my,meqn,work_tri,mwork_tri,
     &      ipiv,qp,qd)
      implicit none

      integer mx,my,meqn,mwork_tri
c     # The last dimension is the number of equations
c     # containing parabolic terms; this is <= meqn
      double precision qp(0:mx+1,0:my+1,*)
      double precision qd(mx+1,my+1,*)

      double precision work_tri(mwork_tri)
      integer ipiv(2*(mx/2+my))

      integer m,mq
      logical active_parabolic_term
      integer q_idx

      do mq = 1,meqn
         if (active_parabolic_term(mq)) then
            m = q_idx(mq)
            call apply_equator_internal(mx,my,work_tri,mwork_tri,
     &            ipiv,qp(0,0,m),qd(1,1,m))
         endif
      enddo

      end


      subroutine apply_equator_internal(mx,my,work_tri,mwork_tri,ipiv,
     &      qp,qd)
      implicit none

      integer mx,my,mwork_tri
      double precision qp(0:mx+1,0:my+1)
      double precision qd(mx+1,my+1)
      double precision work_tri(mwork_tri)
      double precision ipiv(2*(mx/2+my))

      double precision fa

      integer pcval
      integer trisys_du, trisys_d, trisys_dl, trisys_du2, qh
      integer coeffs_alpha, dval, pval, rhs, rv
      integer xpi,ypi,zpi,xdi,ydi,zdi,tv

      integer ntri, nrhs, ldb, info
      character trans

c     # ----------------------------------------------------
c     # Get pointers needed for tridiagonal system and other
c     # set in initialization step
c     # ----------------------------------------------------
      call get_tri_ptrs(mwork_tri,mx,my,pcval,
     &      trisys_du,trisys_d,trisys_dl,trisys_du2,qh,
     &      coeffs_alpha,tv,
     &      pval,dval,rv,rhs)

c     # ----------------------------------------------------
c     # Get time dependent right hand side and scalar fa
c     # ----------------------------------------------------
      call get_tri_rhs(mx,my,work_tri(pcval),work_tri(pval),
     &      qp,work_tri(rv),work_tri(rhs),fa)

c     # ----------------------------------------------------
c     # Solve tridiagonal system to get dual values at the
c     # equator
c     # ----------------------------------------------------
      trans = 'n'
      nrhs = 1
      ntri = 2*(mx/2 + my) - 1
      ldb = ntri
      call dgttrs(trans, ntri,nrhs,work_tri(trisys_dl),
     &      work_tri(trisys_d),work_tri(trisys_du),
     &      work_tri(trisys_du2),ipiv,
     &      work_tri(rhs),ldb,info)

c     # ----------------------------------------------------
c     # Assign dual values at the equator. For the
c     # sphere, we don't use the primal values on the
c     # equator, since they are true ghost values, and are
c     # set using matching conditions with other cells in the
c     # interior of the domain.
c     # ----------------------------------------------------
      call assign_equator_dual(mx,my,work_tri(qh),work_tri(rhs),
     &      work_tri(coeffs_alpha),fa,work_tri(dval),qd)

      end

      subroutine get_tri_rhs(mx,my,pcval,qval,qp,rv,rhs,fa)
      implicit none

      integer mx,my, meqn

      double precision pcval(2*(mx/2+my),2)
      double precision qval(2*(mx/2+my),2)
      double precision qp(0:mx+1,0:my+1)

      double precision rv(0:2*(mx/2 + my))
      double precision rhs(2*(mx/2+my))
      double precision fa

      integer i, j, k, np
      double precision pl, pu

      call match_p(mx,my,qp)

      np = 2*(mx/2 + my)

c     # Index 1 : top
c     # Index 2 : bottom
      k = 1
      do i = mx/2+1,mx
         qval(k,1) = qp(i,1)
         qval(k,2) = qp(i,0)
         k = k + 1
      enddo

      do j = 1,my
         qval(k,1) = qp(mx,j)
         qval(k,2) = qp(mx+1,j)
         k = k + 1
      enddo

      do i = mx,mx/2+1,-1
         qval(k,1) = qp(i,my)
         qval(k,2) = qp(i,my+1)
         k = k + 1
      enddo

      do j = my,1,-1
         qval(k,1) = qp(mx/2+1,j)
         qval(k,2) = qp(mx/2,j)
         k = k + 1
      enddo

c     # Eqn. (3.57)
      do k = 1,np
         pu = pcval(k,1)
         pl = pcval(k,2)
         rv(k) = (pu*qval(k,1) + pl*qval(k,2))/(pu + pl)
      enddo
      rv(0) = rv(np)

      do k = 1,np
         rhs(k) = -(rv(k-1) + rv(k))
      enddo
      fa = rhs(np)

      end

      subroutine assign_equator_dual(mx,my,qh,qinh,coeffs_alpha,fa,
     &      dval,qd)
      implicit none

      integer mx,my
      double precision   qh(2*(mx/2+my))
      double precision qinh(2*(mx/2+my))
      double precision dval(2*(mx/2+my))

      double precision qd(mx+1,my+1)

      double precision fa, alpha, coeffs_alpha(3)

      integer i,j,k,np

      np = 2*(mx/2 + my)

      alpha = coeffs_alpha(1)*fa + coeffs_alpha(2)*qinh(1) +
     &      coeffs_alpha(3)*qinh(np-1)

      if (1./alpha .eq. 0) then
         alpha = 0
      endif

      do k = 1,np-1
         dval(k) = qinh(k) + alpha*qh(k)
      enddo
      dval(np) = alpha

c     # Set dual boundary values
      k = 1
      do i = mx/2+1,mx
         qd(i,1) = dval(k)
         k = k + 1
      enddo

      do j = 1,my
         qd(mx+1,j) = dval(k)
         k = k + 1
      enddo

      do i = mx+1,mx/2+2,-1
         qd(i,my+1) = dval(k)
         k = k + 1
      enddo

      do j = my+1,2,-1
         qd(mx/2+1,j) = dval(k)
         k = k + 1
      enddo

      call match_d(mx,my,qd)
      end


c     # ----------------------------------------------------------------
c     # Setup the tridiagonal system
c     # ----------------------------------------------------------------
      subroutine initialize_equator(mx,my,xlower,ylower,dx,dy,
     &      xpi,ypi,zpi,xdi,ydi,zdi,
     &      work_tri,mwork_tri,ipiv)
      implicit none

      integer mx,my,mwork_tri
      double precision xlower,ylower,dx,dy

      double precision xpi(2*(mx/2+my),-1:1)
      double precision ypi(2*(mx/2+my),-1:1)
      double precision zpi(2*(mx/2+my),-1:1)

      double precision xdi(2*(mx/2+my))
      double precision ydi(2*(mx/2+my))
      double precision zdi(2*(mx/2+my))

      double precision work_tri(mwork_tri)
      integer ipiv(2*(mx/2 + my))

      integer trisys_du,trisys_d,trisys_dl,trisys_du2
      integer pcval, qh, coeffs_alpha
      integer pval,dval,rv,rhs
      integer tv

c     # ----------------------------------------------------
c     # Get pointers needed for tridiagonal system and other
c     # set in initialization step
c     # ----------------------------------------------------
      call get_tri_ptrs(mwork_tri,mx,my,pcval,
     &      trisys_du,trisys_d,trisys_dl,trisys_du2,qh,
     &      coeffs_alpha,tv,pval,dval,rv,rhs)

c     # ----------------------------------------------------
c     # Get coefficients for tridiagonal system
c     # ----------------------------------------------------

      call compute_trisys_coeffs(mx,my,
     &      xpi, ypi, zpi, xdi, ydi, zdi,
     &      work_tri(pcval),work_tri(tv))

c     # ----------------------------------------------------
c     # Get coefficients for tridiagonal system
c     # This could also be used for the bc_phys.f routine.
c     # ----------------------------------------------------
      call construct_sys(mx,my,work_tri(tv),
     &      work_tri(trisys_dl),work_tri(trisys_d),
     &      work_tri(trisys_du),work_tri(trisys_du2),
     &      ipiv,work_tri(qh),
     &      work_tri(coeffs_alpha))

      end

      subroutine construct_sys(mx,my,tv,
     &      trisys_dl,trisys_d,trisys_du,trisys_du2,
     &      ipiv, qh, coeffs_alpha)
      implicit none

      integer mx,my
      double precision  trisys_dl(2*(mx/2+my))
      double precision   trisys_d(2*(mx/2+my))
      double precision  trisys_du(2*(mx/2+my))
      double precision trisys_du2(2*(mx/2+my))
      integer                ipiv(2*(mx/2+my))

      double precision     qh(2*(mx/2+my))
      double precision coeffs_alpha(3)

      double precision tv(0:2*(mx/2 + my))

      double precision v(2),w(2), u
      integer k, km, np, ntri, ldb, nrhs, info
      character trans

      np = 2*(mx/2+my)

      do k = 1,np-1
         trisys_dl(k) = tv(k)
      enddo

      do k = 1,np
         trisys_d(k) = tv(k) - tv(k-1) - 2.d0
      enddo

      do k = 1,np-1
         trisys_du(k) = -tv(k)
      enddo

c     # Define vectors w and v. We only store v(1) and v(np-1); the rest
c     # are zero.  Same for w.
      w(1) = tv(np)
      w(2) = -tv(np-1)
      v(1) = -tv(np)
      v(2) = tv(np-1)
      u = tv(np) - tv(np-1) - 2.d0

c     # Factor tridiagonal system.
      ntri = np - 1
      call dgttrf(ntri,trisys_dl, trisys_d, trisys_du,
     &      trisys_du2, ipiv,info)

      if (info .ne. 0) then
         write(6,'(A,I5)') 'construct_sys(1) : info .ne. 0; ',
     &         info
         stop
      endif

c     # w - equation (2.41)
      do k = 1,np-1
         qh(k) = 0.d0
      enddo
      qh(1)    = -w(1)
      qh(np-1) = -w(2)

      trans = 'n'
      ldb      = ntri
      nrhs     = 1
      call dgttrs(trans,ntri,nrhs,trisys_dl,trisys_d,
     &      trisys_du,trisys_du2,ipiv,qh,ldb,info)

      if (info .ne. 0) then
         write(6,'(A,I5)') 'construct_sys(2) : info .ne. 0; ',
     &         info
         stop
      endif

      coeffs_alpha(1) = 1.d0/(u + v(1)*qh(1) + v(2)*qh(np-1))
      coeffs_alpha(2) = -coeffs_alpha(1)*v(1)
      coeffs_alpha(3) = -coeffs_alpha(1)*v(2)

      end

c     # ----------------------------------------------------------------
c     # Setup the coefficients needed for the boundary conditions operator
c     # ----------------------------------------------------------------
      subroutine compute_trisys_coeffs(mx,my,xpi,ypi,zpi,
     &      xdi,ydi,zdi,pcval,tv)
      implicit none

      integer mx,my
      double precision xpi(2*(mx/2 + my),-1:1)
      double precision ypi(2*(mx/2 + my),-1:1)
      double precision zpi(2*(mx/2 + my),-1:1)

      double precision xdi(2*(mx/2 + my))
      double precision ydi(2*(mx/2 + my))
      double precision zdi(2*(mx/2 + my))

      double precision pcval(2*(mx/2 + my),2)

      double precision tv(0:2*(mx/2 + my))

      integer i,k,m, np, kp1
      double precision taup(3), taud(3), sp
      double precision c1, c2, u(2), p(2)

      np = 2*(mx/2 + my)

c     % Loop over each boundary cells
      do k = 1,np
         do i = -1,1,2

c           # These are the points on nodes of the primary cell
            if (k .eq. np) then
               kp1 = 1
            else
               kp1 = k + 1
            endif

            taud(1) = xpi(k,0) - xpi(k,i)
            taud(2) = ypi(k,0) - ypi(k,i)
            taud(3) = zpi(k,0) - zpi(k,i)

            taup(1) = xdi(kp1) - xdi(k)
            taup(2) = ydi(kp1) - ydi(k)
            taup(3) = zdi(kp1) - zdi(k)

            call get_coeffs(taup,taud,c1,c2,sp)

c           # We don't divide by sp here, since all the
c           # the normalizations cancel when we construct
c           # arrays tv() and rv().

            pcval(k,(1-i)/2+1) = c1

            p((1-i)/2+1) = c1
            u((1-i)/2+1) = c2

         enddo
c        # Eqn. (3.57)
         tv(k) = (u(1) + u(2))/(p(1) + p(2))
      enddo
      tv(0) = tv(np)

      end


c     # -------------------------------------------------------------
c     # Miscellaneous routines for allocating memory
c     # -------------------------------------------------------------

      subroutine get_tri_ptrs(mwork,mx,my,pcval,
     &      trisys_du,trisys_d,trisys_dl,trisys_du2,qh,
     &      coeffs_alpha,tv,pval,dval,rv,rhs)
      implicit none

      integer mwork,mx,my
      integer trisys_du,trisys_d,trisys_dl,trisys_du2,qh
      integer pcval,coeffs_alpha
      integer tv,pval,dval,rv,rhs

      integer total, nbc

      nbc = 2*(mx/2 + my)

      pcval        = 1
      trisys_du    = pcval        + 2*nbc
      trisys_d     = trisys_du    + nbc
      trisys_dl    = trisys_d     + nbc
      trisys_du2   = trisys_dl    + nbc
      qh           = trisys_du2   + nbc
      coeffs_alpha = qh           + nbc

      tv           = coeffs_alpha + 3
      pval         = tv           + nbc + 1
      dval         = pval         + 2*nbc
      rv           = dval         + nbc
      rhs          = rv           + nbc + 1
      total        = rhs          + nbc + 1

      total = total - 1

      if (mwork .lt. 0) then
c        # This is being called from the RKC solver - we already
c        # know that we have enough space
         return
      endif
      if (total .gt. mwork) then
         write(6,*) 'Increase memory in work_tri to ',total
         stop
      endif

      end

      subroutine match(mx,my,up,ud)
      implicit none

      integer mx,my
      double precision up(0:mx+1,0:my+1)
      double precision ud(mx+1,my+1)

      call match_p(mx,my,up)
      call match_d(mx,my,ud)

      end

      subroutine match_p(mx,my,up)
      implicit none

      integer mx,my
      double precision up(0:mx+1,0:my+1)

      integer i,j

c     # copy values in top hemisphere to bottom hemisphere.
c     # We assume that we have set values in the upper hemisphere
c     # and now need values in the lower.
      do j = 1,my
         up(0,j) = up(mx,j)
         up(mx+1,j) = up(1,j)
      enddo

      do i = 0,mx+1
         up(i,0) = up(mx+1-i,1)
         up(i,my+1) = up(mx+1-i,my)
      enddo

      end

      subroutine match_d(mx,my,ud)
      implicit none

      integer mx,my
      double precision ud(mx+1,my+1)

      integer i,j,m

c     # Wrap dual cell values at left edge
      do j = 1,my+1
         ud(1,j) = ud(mx+1,j)
      enddo

      do i = 1,mx/2
         ud(i,1)    = ud(mx-i+2,1)
         ud(i,my+1) = ud(mx-i+2,my+1)
      enddo

      end

      subroutine set_primal_ghost(mx,my,meqn,qp)
      implicit none

      integer mx,my,meqn
      double precision qp(0:mx+1,0:my+1,meqn)

      integer m

      do m = 1,meqn
         call match_p(mx,my,qp(0,0,m))
      enddo
      end

      subroutine match_mesh(mx,my,mbc,sp,sd)
      implicit none

      integer mx,my,mbc
      double precision sp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision sd(-mbc:mx+mbc+2,-mbc:my+mbc+2)

      integer i,j,m

c     # We only match the cells that actually overlap;
c     # assume that all others are okay.

c     # Wrap dual cell values at left edge
      do j = 1,my+1
         sd(1,j) = sd(mx+1,j)
      enddo

      do i = 1,mx/2
         sd(i,1)    = sd(mx-i+2,1)
         sd(i,my+1) = sd(mx-i+2,my+1)
      enddo

      do j = 1,my
         sp(0,j) = sp(mx,j)
         sp(mx+1,j) = sp(1,j)
      enddo

      do i = 0,mx+1
         sp(i,0) = sp(mx+1-i,1)
         sp(i,my+1) = sp(mx+1-i,my)
      enddo



      end
