      subroutine parabolic_step(maxmx, maxmy,meqn,mbc,mx,my,
     &      nv,q_claw,t,dt,ierror)
      implicit none

      integer maxmx, maxmy, meqn,mbc,mx,my
      double precision q_claw(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc,meqn)
      logical nv(meqn)
      double precision t, dt, t_save
      integer method, get_method
      integer lc0, lc1, lc_inc, get_laplacian_call_counter

      integer ierror, itcnt(2),m

      do m = 1,2
         itcnt(m) = 0
      enddo
      call store_itcount(itcnt)

      call start_laplacian_call_counter()
      call start_timer()

      t_save = t

      method = get_method()

      if (method .eq. 1) then
c        # --------------------------------------------------
c        # Forward Euler
c        # --------------------------------------------------
         call parabolic_step_feuler(maxmx, maxmy,meqn,mbc,mx,my,
     &         nv,q_claw,t,dt,ierror)

      elseif (method .eq. 2) then
c        # --------------------------------------------------
c        # Backward Euler
c        # --------------------------------------------------
         call parabolic_step_beuler(maxmx, maxmy,meqn,mbc,mx,my,
     &         nv,q_claw,t,dt,ierror)

      elseif (method .eq. 3) then
c        # --------------------------------------------------
c        # Crank-Nicolson
c        # --------------------------------------------------
         call parabolic_step_cn(maxmx, maxmy,meqn,mbc,mx,my,
     &         nv,q_claw,t,dt,ierror)

      elseif (method .eq. 4) then
c        # --------------------------------------------------
c        # TR-BDF2
c        # --------------------------------------------------
         call parabolic_step_trbdf(maxmx, maxmy,meqn,mbc,mx,my,
     &      nv,q_claw,t,dt,ierror)

      elseif (method .eq. 5) then
c        # --------------------------------------------------
c        # ROCK4
c        # --------------------------------------------------
         call parabolic_step_rock4(maxmx, maxmy,meqn,mbc,mx,my,
     &         nv,q_claw,t,dt,ierror)
         if (ierror .ge. 2) then
            write(6,100)'parabolic_step : Problem with rkc solver; ',
     &            ' ierror = ',ierror
         endif
      elseif (method .eq. 6) then
c        # --------------------------------------------------
c        # RKC
c        # --------------------------------------------------
         call parabolic_step_rkc(maxmx, maxmy,meqn,mbc,mx,my,
     &      nv,q_claw,t,dt,ierror)
         if (ierror .ge. 2) then
            write(6,100)'parabolic_step_rkc : Problem with rkc ',
     &            ' solver ierror = ',ierror
         endif
      elseif (method .eq. 7) then
c        # --------------------------------------------------
c        # IRKC
c        # --------------------------------------------------
         call parabolic_step_irkc(maxmx, maxmy, meqn,mbc,mx,my,
     &         nv,q_claw,t,dt,ierror)
      endif
  100 format(A,A,I5)

      t = t_save

      call stop_laplacian_call_counter()
      call stop_timer()
c      call accumulate_time()
      call print_stats(t,dt)

      end

      subroutine print_stats(t,dt)
      implicit none

      double precision t, dt
      integer method, get_method

      integer verbose
      integer get_verbosity

      method = get_method()
      verbose = get_verbosity()


      if (verbose .eq. 2) then
         call print_stats_short(t,dt)
      elseif (verbose .eq. 3) then
         call print_stats_long(t,dt)
      elseif (verbose .gt. 3) then
         if (method .eq. 1) then
c           # --------------------------------------------------
c           # Forward Euler
c           # --------------------------------------------------
            call feuler_stats(t,dt)

         elseif (method .eq. 2) then
c           # --------------------------------------------------
c           # Backward Euler
c           # --------------------------------------------------
            call beuler_stats(t,dt)

         elseif (method .eq. 3) then
c           # --------------------------------------------------
c           # Crank-Nicolson
c           # --------------------------------------------------
            call cn_stats(t,dt)

         elseif (method .eq. 4) then
c           # --------------------------------------------------
c           # TR-BDF2
c           # --------------------------------------------------
            call trbdf_stats(t,dt)

         elseif (method .eq. 5) then
c           # --------------------------------------------------
c           # ROCK4
c           # --------------------------------------------------
            call rock4_stats(t,dt)

         elseif (method .eq. 6) then
c           # --------------------------------------------------
c           # RKC
c           # --------------------------------------------------
            call rkc_stats(t,dt)
         elseif (method .eq. 7) then
c           # --------------------------------------------------
c           # IRKC
c           # --------------------------------------------------
            call irkc_stats(t,dt)
         endif
      endif


      end
