      subroutine parabolic_step_cn(maxmx, maxmy,meqn,mbc,mx,my,
     &      nv,q_claw,t,dt,ierror)
      implicit none

      integer maxmx, maxmy, meqn,mbc,mx,my, ierror
      double precision q_claw(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc,meqn)
      logical nv(meqn)
      double precision t, dt

      integer neqn, ntotal,mpara
      integer prt_info(2), maxiter
      double precision tol
      integer get_trbdf_mem, mwork_solver

      include 'para.i'

      call check_method(method_cn)

c     # Set the global variable that tells us
c     # which equations have parabolic terms.
      call set_nv(meqn,nv,ntotal)
      method_com = 2

      if (ntotal .eq. 0) then
c        # We have no diffusion terms.
         ierror = 0
         return
      endif
      mpara = ntotal
      neqn = (mx+2)*(my+2)*mpara

      call ps_copy_from_claw(maxmx,maxmy,mbc,meqn,mx,my,
     &      q_claw,work(q_ptr))

      mwork_solver = get_trbdf_mem(mx,my,mpara)
      maxiter = 10000
      tol = 1e-12
      prt_info(1) = 0
      prt_info(2) = 0

      call crank_nicolson(mx,my,meqn,mpara,t,dt,work(q_ptr),maxiter,
     &      tol,work(work_solver_ptr),mwork_solver,prt_info,ierror)

      if (ierror .gt. 0) then
         write(6,*) 'parabolic_step_cn : ierror .ne. 0; ierror = ',
     &         ierror
         stop
      endif

      call ps_copy_to_claw(maxmx,maxmy,mbc,meqn,mx,my,
     &      work(q_ptr),q_claw)

      end

c     # ---------------------------------------------------
c     # Solve using CN
c     # ---------------------------------------------------
      subroutine crank_nicolson(mx,my,meqn,mpara,t,dt,qn,
     &      maxiter,tol,work_solver,mwork_solver,prt_info,
     &      ierror)
      implicit none

      integer mx,my,meqn,mpara, mwork_solver
      integer prt_info(2), maxiter, ierror
      double precision t, dt, tol
      double precision qn(0:mx+1,0:my+1,*)
      double precision work_solver(mwork_solver)
      integer get_bicg_mem, mwork_bicg

      integer rhs,diff,reac,work_bicg,total_its
      double precision lambda

      integer itcnt(2)

      include 'para.i'

      call get_ptrs_cn(mwork_solver,mx,my,mpara,rhs,
     &      diff,reac,work_bicg)

c     # These are needed by the BICG matvec
      current_time_com = t
      current_dt_com = dt

      call compute_rhs_cn(mx,my,meqn,qn,t,dt,work_solver(diff),
     &      work_solver(reac),work_solver(rhs))

      lambda = -2.d0/dt
      total_its = maxiter
      mwork_bicg = get_bicg_mem(mx,my,mpara)
      call bicg(mx,my,meqn,mpara,work_solver(rhs),qn,lambda,
     &      total_its,tol,work_solver(work_bicg),
     &      mwork_bicg,prt_info,ierror)
      maxiter = total_its

      itcnt(1) = total_its
      itcnt(2) = 0

      call store_itcount(itcnt)

      end

      subroutine cn_stats(tip1,dt)
      implicit none

      double precision tip1, dt

      integer lc_total, lc_inc
      integer get_laplacian_calls_total
      integer get_laplacian_calls_elapsed

      integer ts_counter
      integer get_time_step_counter

      integer itcnt(2)

      integer verbose
      integer get_verbosity

      ts_counter = get_time_step_counter()
      verbose  = get_verbosity()
      lc_total = get_laplacian_calls_total()
      lc_inc = get_laplacian_calls_elapsed()
      call get_itcount(itcnt)


      write(6,100) 'Step ', ts_counter
      write(6,101) 'Time ', tip1
      write(6,102) 'dt ',dt
      write(6,100) 'Laplacian calls (this step) ', lc_inc
      write(6,100) 'Laplacian calls (total) ', lc_total
      write(6,100) 'Number of BICG iterations ', itcnt(1)
      write(6,100)

  100 format(A35,I12)
  101 format(A35,F12.6)
  102 format(A35,E12.4)

      end


      subroutine  get_ptrs_cn(mwork,mx,my,mpara,rhs,
     &      diff,reac,work_bicg)
      implicit none

      integer mwork,mx,my,mpara,qnph,rhs,work_bicg
      integer diff,reac
      integer total,np, get_bicg_mem, mwork_bicg

      np = (mx+2)*(my+2)
      mwork_bicg = get_bicg_mem(mx,my,mpara)

      rhs       = 1
      diff      = rhs        +    mpara*np
      reac      = diff       +    mpara*np
      work_bicg = reac       +    mpara*np
      total     = work_bicg  +    mwork_bicg


      total = total - 1

      if (total .gt. mwork) then
         write(6,*) 'get_ptrs_cn : Increase size of mwork to ',
     &         total
         stop
      endif

      end
