      subroutine parabolic_step_trbdf(maxmx, maxmy,meqn,mbc,mx,my,
     &      nv,q_claw,t,dt,ierror)
      implicit none

      integer maxmx, maxmy, meqn,mbc,mx,my, ierror
      double precision q_claw(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc,meqn)
      logical nv(meqn)
      double precision t, dt

      integer neqn, ntotal,mpara
      integer prt_info(2), maxiter
      double precision tol
      integer get_trbdf_mem, mwork_solver

      include 'para.i'

      call check_method(method_trbdf)

c     # Set the global variable that tells us
c     # which equations have parabolic terms.
      call set_nv(meqn,nv,ntotal)

      if (ntotal .eq. 0) then
c        # We have no diffusion terms.
         ierror = 0
         return
      endif
c      mpara = ntotal
c      neqn = (mx+2)*(my+2)*ntotal

      call ps_copy_from_claw(maxmx,maxmy,mbc,meqn,mx,my,
     &      q_claw,work(q_ptr))

      mpara = mpara_com
      mwork_solver = get_trbdf_mem(mx,my,mpara)
      maxiter = 10000
      tol = 1e-12
      prt_info(1) = 0
      prt_info(2) = 0

      call trbdf(mx,my,meqn,mpara,t,dt,work(q_ptr),maxiter,tol,
     &      work(work_solver_ptr),mwork_solver,prt_info,ierror)

      if (ierror .gt. 0) then
         write(6,*) 'parabolic_step_trbdf : ierror .ne. 0; ierror = ',
     &         ierror
         stop
      endif

      call ps_copy_to_claw(maxmx,maxmy,mbc,meqn,mx,my,
     &      work(q_ptr),q_claw)

      end

c     # ---------------------------------------------------
c     # Solve using TR-BDF2
c     # ---------------------------------------------------
      subroutine trbdf(mx,my,meqn,mpara,t,dt,qn,maxiter,
     &      tol,work_solver,mwork_solver,prt_info,ierror)
      implicit none

      integer mx,my,meqn,mpara, mwork_solver
      integer prt_info(2), maxiter, ierror
      double precision t, dt, tol
      double precision qn(0:mx+1,0:my+1,*)
      double precision work_solver(mwork_solver)
      integer get_bicg_mem, mwork_bicg

      integer itcnt(2)

      integer qnph,rhs,reac,work_bicg, work_cn,mwork_cn
      integer get_cn_mem
      double precision dt2

      call get_ptrs_trbdf(mwork_solver,mx,my,mpara,qnph,reac,
     &      rhs,work_cn,work_bicg)

c     # Need to save qn for the BDF2 step
      call copy_qn_qnph(mx,my,meqn,qn,work_solver(qnph))

c     # Solve for qnph
      dt2 = dt/2.d0
      mwork_cn = get_cn_mem(mx,my,mpara)
      itcnt(1) = maxiter
      call crank_nicolson(mx,my,meqn,mpara,t,dt2,work_solver(qnph),
     &      itcnt(1),tol,work_solver(work_cn),mwork_cn,
     &      prt_info,ierror)

c     # Solver for qnp1 = qn
      mwork_bicg = get_bicg_mem(mx,my,mpara)
      itcnt(2) = maxiter
      call solve_bdf2(mx,my,meqn,qn,work_solver(qnph),
     &      work_solver(reac),work_solver(rhs),t,dt,
     &      itcnt(2),tol,work_solver(work_bicg),mwork_bicg,
     &      prt_info,ierror)

      call store_itcount(itcnt)

      end


      subroutine copy_qn_qnph(mx,my,meqn,qn,qnph)
      implicit none
      integer mx,my,meqn
      double precision qn(0:mx+1,0:my+1,*)
      double precision qnph(0:mx+1,0:my+1,*)

      integer i,j,mq,q_idx, m
      logical active_parabolic_term

      do mq = 1,meqn
         if (active_parabolic_term(mq)) then
            m = q_idx(mq)
            do j = 0,my+1
               do i = 0,mx+1
                  qnph(i,j,m) = qn(i,j,m)
               enddo
            enddo
         endif
      enddo

      end


      subroutine solve_bdf2(mx,my,meqn,qn,qnph,reac,rhs,t,dt,
     &      maxiter,tol,work_bicg,mwork_bicg,prt_info,ierror)
      implicit none

      integer mx,my,meqn,mwork_bicg
      integer maxiter,prt_info(2),ierror
      double precision t,dt,tol
      double precision qn(0:mx+1,0:my+1,*)
      double precision qnph(0:mx+1,0:my+1,*)
      double precision rhs(0:mx+1,0:my+1,*)
      double precision reac(0:mx+1,0:my+1,*)
      double precision work_bicg(mwork_bicg)

      integer i,j, q_idx,mq,m, mpara
      double precision lambda, tp
      logical active_parabolic_term
      integer total_its

      include 'para.i'

      current_time_com = t
      current_dt_com = dt

c     # Source term should be evaluated at new time level for BDF2
      tp = t + dt
      call src_reaction(mx,my,meqn,tp,qn,reac)

      do mq = 1,meqn
         if (active_parabolic_term(mq)) then
            m = q_idx(mq)
            do j = 1,my
               do i = 1,mx
                  rhs(i,j,m) = (-1/dt)*(4*qnph(i,j,m) - qn(i,j,m))
     &                  - reac(i,j,m)
               enddo
            enddo
         endif
      enddo

      call set_zeros(mx,my,meqn,rhs)

c     # Overwrite qn with qnp1
      mpara = mpara_com
      lambda = -3.d0/dt
      total_its = maxiter
      call bicg(mx,my,meqn,mpara,rhs,qn,lambda,total_its,tol,
     &      work_bicg,mwork_bicg,prt_info,ierror)
      maxiter = total_its

      end


      subroutine trbdf_stats(tip1,dt)
      implicit none

      double precision tip1, dt
      integer itcnt(2)
      integer ts_counter,get_time_step_counter

      integer lc_inc, lc_total
      integer get_laplacian_calls_total
      integer get_laplacian_calls_elapsed


      call get_itcount(itcnt)
      ts_counter = get_time_step_counter()
      lc_total = get_laplacian_calls_total()
      lc_inc = get_laplacian_calls_elapsed()

      write(6,100) 'Step ', ts_counter
      write(6,101) 'Time ', tip1
      write(6,102) 'dt ',dt
      write(6,100) 'Laplacian calls (this step) ', lc_inc
      write(6,100) 'Laplacian calls (total) ', lc_total
      write(6,100) 'Number of BICG iterations (trap)', itcnt(1)
      write(6,100) 'Number of BICG iterations (bdf)', itcnt(2)
      write(6,100)

  100 format(A35,I12)
  101 format(A35,F12.6)
  102 format(A35,E12.4)

      end


      subroutine  get_ptrs_trbdf(mwork,mx,my,mpara,qnph,reac,
     &      rhs,work_cn,work_bicg)
      implicit none

      integer mwork,mx,my,mpara,qnph,rhs,reac,work_bicg
      integer work_cn, mwork_cn, get_cn_mem
      integer total,np, get_bicg_mem, mwork_bicg

      np = (mx+2)*(my+2)
      mwork_bicg = get_bicg_mem(mx,my,mpara)
      mwork_cn = get_cn_mem(mx,my,mpara)

      qnph = 1
      reac      = qnph       +    mpara*np
      rhs       = reac       +    mpara*np
      work_cn   = rhs        +    mpara*np
      work_bicg = work_cn    +    mwork_cn
      total     = work_bicg  +  mwork_bicg


      total = total - 1

      if (total .gt. mwork) then
         write(6,*) 'get_ptrs_trbdf : Increase size of mwork to ',
     &         total
         stop
      endif

      end
