c     # -------------------------------------------------------------
c     # Function describing ODE to be solved by RKC
c     # -------------------------------------------------------------
      subroutine f_exp(neqn,t,q,rhs)
      implicit none

      integer neqn
      double precision t, q(*), rhs(*)

      integer mx,my,c_bc,meqn, ldtri, qd
      double precision xlower,ylower,dx,dy

      include "para.i"

      mx = mx_com
      my = my_com

      meqn = meqn_com
      xlower = xlower_com
      ylower = ylower_com
      dx = dx_com
      dy = dy_com
      ldtri = ldtri_com

c      call apply_equator(mx,my,meqn,work(work_tri_ptr),
c     &      ldtri,ipiv,q,work(qd_ptr))

c      call match(mx,my,q,work(qd_ptr))

      call compute_avg(mx,my,meqn,q,work(qd_ptr))


c     # Compute Laplacian using primal and dual values
      call apply_LB(mx,my,meqn,work(coeffs_ptr),work(area_ptr),
     &      q,work(qd_ptr),rhs)

c     # Match boundary conditions for sphere
      call src_reaction(mx,my,meqn,t,q,rhs)

c     # Zero out ghost cell values so RKC won't update them
      call set_zeros(mx,my,meqn,rhs)

      end


c     # -----------------------------------------------------
c     # setup_parabolic_sphere
c     # -----------------------------------------------------

      subroutine fix_edges(xc,yc,xlower,ylower,xupper,yupper)
      implicit none

      double precision xc,yc,xlower,ylower,xupper,yupper

c      xc = min(max(xc,xlower),xupper)
c      yc = min(max(yc,ylower),yupper)

      return

      end


      logical function issphere()
      implicit none
      issphere = .true.
      end

      subroutine setup_parabolic(mx,my,meqn,plist,
     &      xlower,ylower,dx,dy)
      implicit none

      integer mx,my,meqn
      logical plist(meqn)

      double precision xlower,ylower,dx,dy

      include 'para.i'

c     # Set up pointers in common blocks in 'para.i'
      call setup_common_sphere(mx,my,xlower,ylower,dx,dy,meqn,plist)

      call setup_parabolic_geo(mx,my,xlower,ylower,dx,dy)

c     # Match up boundary points and area

c      call setup_parabolic_equator(mx,my,xlower,ylower,dx,dy)

      end

      subroutine setup_parabolic_geo(mx,my,xlower,ylower,dx,dy)
      implicit none

      integer mx,my
      double precision xlower,ylower,dx,dy

      include "para.i"

c     # Set up mesh.  Use pointers defined in para.i
      call setup_mesh(mx,my,xlower,ylower,dx,dy,
     &      work(xp_ptr),work(yp_ptr),work(zp_ptr),
     &      work(xd_ptr),work(yd_ptr),work(zd_ptr))

      call match(mx,my,work(xp_ptr),work(xd_ptr))
      call match(mx,my,work(yp_ptr),work(yd_ptr))
      call match(mx,my,work(zp_ptr),work(zd_ptr))

c     # Set up metric terms.
      call setup_metric(mx,my,xlower,ylower,dx,dy,
     &      work(xd_ptr),work(yd_ptr),work(zd_ptr),
     &      work(area_ptr),work(len_ptr))

      call match_p(mx,my,work(area_ptr))

c     # Compute coefficients for Laplacian
      call compute_lb_coeffs(mx,my,
     &      work(xp_ptr),work(yp_ptr),work(zp_ptr),
     &      work(xd_ptr),work(yd_ptr),work(zd_ptr),
     &      work(coeffs_ptr))

      end


c      subroutine setup_parabolic_equator(mx,my,xlower,ylower,dx,dy)
c      implicit none
c
c      integer mx,my
c      double precision xlower,ylower,dx,dy
c
c      integer ldtri
c
c      include "para.i"
c
c      ldtri = ldtri_com
c
c      call initialize_equator(mx,my,xlower,ylower,dx,dy,
c     &      work(xp_ptr),work(yp_ptr),work(zp_ptr),
c     &      work(xd_ptr),work(yd_ptr),work(zd_ptr),
c     &      work(work_tri_ptr),ldtri,ipiv)
c
c      end

      subroutine setup_common_sphere(mx,my,xlower,ylower,dx,dy,meqn,
     &      plist)
      implicit none

      integer mx,my, meqn
      logical plist(meqn)

      double precision xlower,ylower,dx,dy
      integer np, nd, nbc, total

      integer i, m, mq, mpara, ldtri

      include "para.i"

      nfevals_total = 0

      if (meqn .gt. maxmeqn) then
         write(6,110) 'para.i : Increase size of maxmeqn to ',
     &         meqn
         stop
      endif
  110 format(A,I2)

      m = 0
      do mq = 1,meqn
         plist_com(mq) = plist(mq)
         if (plist(mq)) then
            m = m + 1
         endif
      enddo
      mpara = m

c     # To communicate these through common blocks
      mx_com = mx
      my_com = my
      dx_com = dx
      dy_com = dy
      xlower_com = xlower
      ylower_com = ylower

      np   = (mx+2)*(my+2)
      nd   = (mx+1)*(my+1)
      nbc  = 2*(mx/2+ my)
      ldtri = 25*nbc + 6

      mpara_com = mpara
      meqn_com = meqn
      ldtri_com = ldtri

c     # Indices into work array
      xp_ptr        = 1
      yp_ptr        = xp_ptr         + np
      zp_ptr        = yp_ptr         + np
      area_ptr      = zp_ptr         + np
      coeffs_ptr    = area_ptr       + np
      xd_ptr        = coeffs_ptr     + 4*np
      yd_ptr        = xd_ptr         + nd
      zd_ptr        = yd_ptr         + nd
      qd_ptr        = zd_ptr         + nd
      work_tri_ptr  = qd_ptr         + nd*mpara
      rhs_ptr       = work_tri_ptr   + ldtri
      q_ptr         = rhs_ptr        + np*mpara
      work_rkc_ptr  = q_ptr          + np*mpara
      total         = work_rkc_ptr   + 5*np*mpara + 8

      total = total - 1

      if (mwork .lt. total) then
         write(6,'(A,A,I10)') 'para.i : Increase memory ',
     &         'in work array to at least ', total
         stop
      endif

c     # Check integer array 'ipiv'.  There is only one
c     # array, so we don't need a pointer into it.
      if (miwork .lt. 2*(mx + my)) then
         write(6,100) 'para.i : Increase size of ipiv ',
     &         'to ', 2*(mx + my)
         stop
      endif
  100 format(A,A,I10)

      end



      subroutine match(mx,my,qp,qd)
      implicit none

      integer mx,my
      double precision qp(0:mx+1,0:my+1)
      double precision qd(mx+1,my+1)

      call match_p(mx,my,qp)
      call match_d(mx,my,qd)

      end

      subroutine match_p(mx,my,qp)
      implicit none

      integer mx,my
      double precision qp(0:mx+1,0:my+1)

      integer i,j

c     # copy values in top hemisphere to bottom hemisphere.
c     # We assume that we have set values in the upper hemisphere
c     # and now need values in the lower.

c     # Corners
      qp(0,0) = qp(mx,my)
      qp(0,my+1) = qp(mx,1)
      qp(mx+1,my+1)= qp(1,1)
      qp(mx+1,0) = qp(0,my+1)

      do j = 1,my
         qp(0,j) = qp(mx,j)
         qp(mx+1,j) = qp(1,j)
      enddo

      do i = 1,mx
         qp(i,0) = qp(i,my)
         qp(i,my+1) = qp(i,1)
      enddo



      end

      subroutine match_d(mx,my,qd)
      implicit none

      integer mx,my
      double precision qd(mx+1,my+1)

      integer i,j

c     # Wrap dual cell values at left edge
      do j = 1,my+1
         qd(1,j) = qd(mx+1,j)
      enddo

      do i = 1,mx+1
         qd(i,1)    = qd(i,my+1)
      enddo

      end

c c     # -------------------------------------------------------
c c     # Setup system for solving for dual values at the equator.
c c     #
c c     # This requires that the tridiagonal system has been
c c     # set up by 'initialize_bc(...)'
c c     #
c c     # ----------------------------------------------------------
c
c       subroutine apply_equator(mx,my,meqn,work_tri,mwork_tri,
c      &      ipiv,qp,qd)
c       implicit none
c
c       integer mx,my,meqn,mwork_tri
c c     # The last dimension is the number of equations
c c     # containing parabolic terms; this is <= meqn
c       double precision qp(0:mx+1,0:my+1,*)
c       double precision qd(0:mx+1,0:my+1,*)
c
c       double precision work_tri(mwork_tri)
c       integer ipiv(2*(mx/2+my))
c
c       integer m,mq
c       logical active_parabolic_term
c       integer q_idx
c
c       do mq = 1,meqn
c          if (active_parabolic_term(mq)) then
c             m = q_idx(mq)
c             call apply_equator_internal(mx,my,work_tri,mwork_tri,
c      &            ipiv,qp(0,0,m),qd(0,0,m))
c          endif
c       enddo
c
c       end
c
c
c       subroutine apply_equator_internal(mx,my,work_tri,mwork_tri,ipiv,
c      &      qp,qd)
c       implicit none
c
c       integer mx,my,mwork_tri
c       double precision qp(0:mx+1,0:my+1)
c       double precision qd(mx+1,my+1)
c       double precision work_tri(mwork_tri)
c       double precision ipiv(2*(mx/2+my))
c
c       double precision fa
c
c       integer pcval
c       integer trisys_du, trisys_d, trisys_dl, trisys_du2, qh
c       integer coeffs_alpha, dval, pval, rhs, rv
c       integer xpi,ypi,zpi,xdi,ydi,zdi,tv
c
c       integer ntri, nrhs, ldb, info
c       character trans
c
c c     # ----------------------------------------------------
c c     # Get pointers needed for tridiagonal system and other
c c     # set in initialization step
c c     # ----------------------------------------------------
c       call get_tri_ptrs(mwork_tri,mx,my,pcval,
c      &      trisys_du,trisys_d,trisys_dl,trisys_du2,qh,
c      &      coeffs_alpha,xpi,ypi,zpi,xdi,ydi,zdi,tv,
c      &      pval,dval,rv,rhs)
c
c c     # ----------------------------------------------------
c c     # Get time dependent right hand side and scalar fa
c c     # ----------------------------------------------------
c       call get_tri_rhs(mx,my,work_tri(pcval),work_tri(pval),
c      &      qp,work_tri(rv),work_tri(rhs),fa)
c
c c     # ----------------------------------------------------
c c     # Solve tridiagonal system to get dual values at the
c c     # equator
c c     # ----------------------------------------------------
c       trans = 'n'
c       nrhs = 1
c       ntri = 2*(mx/2 + my) - 1
c       ldb = ntri
c       call dgttrs(trans, ntri,nrhs,work_tri(trisys_dl),
c      &      work_tri(trisys_d),work_tri(trisys_du),
c      &      work_tri(trisys_du2),ipiv,
c      &      work_tri(rhs),ldb,info)
c
c c     # ----------------------------------------------------
c c     # Assign dual values at the equator. For the
c c     # sphere, we don't use the primal values on the
c c     # equator, since they are true ghost values, and are
c c     # set using matching conditions with other cells in the
c c     # interior of the domain.
c c     # ----------------------------------------------------
c       call assign_equator_dual(mx,my,work_tri(qh),work_tri(rhs),
c      &      work_tri(coeffs_alpha),fa,work_tri(dval),qd)
c
c       end
c
c       subroutine get_tri_rhs(mx,my,pcval,pval,qp,rv,rhs,fa)
c       implicit none
c
c       integer mx,my
c
c       double precision pcval(2*(mx/2+my),2)
c       double precision pval(2*(mx/2+my),2)
c       double precision qp(0:mx+1,0:my+1)
c
c       double precision rv(0:2*(mx/2 + my))
c       double precision rhs(2*(mx/2+my))
c       double precision fa
c
c       integer i, j, k, np
c       double precision pl, pu
c
c       call match_p(mx,my,qp)
c
c       np = 2*(mx/2 + my)
c
c c     # Index 1 : top
c c     # Index 2 : bottom
c       k = 1
c       do i = mx/2+1,mx
c          pval(k,1) = qp(i,1)
c          pval(k,2) = qp(i,0)
c          k = k + 1
c       enddo
c
c       do j = 1,my
c          pval(k,1) = qp(mx,j)
c          pval(k,2) = qp(mx+1,j)
c          k = k + 1
c       enddo
c
c       do i = mx,mx/2+1,-1
c          pval(k,1) = qp(i,my)
c          pval(k,2) = qp(i,my+1)
c          k = k + 1
c       enddo
c
c       do j = my,1,-1
c          pval(k,1) = qp(mx/2+1,j)
c          pval(k,2) = qp(mx/2,j)
c          k = k + 1
c       enddo
c
c       do k = 1,np
c          pu = pcval(k,1)
c          pl = pcval(k,2)
c          rv(k) = (pu*pval(k,1) + pl*pval(k,2))/(pu + pl)
c       enddo
c       rv(0) = rv(np)
c
c       do k = 1,np
c          rhs(k) = -(rv(k-1) + rv(k))
c       enddo
c       fa = rhs(np)
c
c       end
c
c       subroutine assign_equator_dual(mx,my,qh,qinh,coeffs_alpha,fa,
c      &      dval,qd)
c       implicit none
c
c       integer mx,my
c       double precision   qh(2*(mx/2+my))
c       double precision qinh(2*(mx/2+my))
c       double precision dval(2*(mx/2+my))
c
c       double precision qd(mx+1,my+1)
c
c       double precision fa, alpha, coeffs_alpha(3)
c
c       integer i,j,k,np
c
c       np = 2*(mx/2 + my)
c
c       alpha = coeffs_alpha(1)*fa + coeffs_alpha(2)*qinh(1) +
c      &      coeffs_alpha(3)*qinh(np-1)
c
c       if (1./alpha .eq. 0) then
c          alpha = 0
c       endif
c
c       do k = 1,np-1
c          dval(k) = qinh(k) + alpha*qh(k)
c       enddo
c       dval(np) = alpha
c
c c     # Set dual boundary values
c       k = 1
c       do i = mx/2+1,mx
c          qd(i,1) = dval(k)
c          k = k + 1
c       enddo
c
c       do j = 1,my
c          qd(mx+1,j) = dval(k)
c          k = k + 1
c       enddo
c
c       do i = mx+1,mx/2+2,-1
c          qd(i,my+1) = dval(k)
c          k = k + 1
c       enddo
c
c       do j = my+1,2,-1
c          qd(mx/2+1,j) = dval(k)
c          k = k + 1
c       enddo
c
c       call match_d(mx,my,qd)
c   100 format(I5,3F24.16)
c       end
c
c
c c     # ----------------------------------------------------------------
c c     # Setup the tridiagonal system
c c     # ----------------------------------------------------------------
c       subroutine initialize_equator(mx,my,xlower,ylower,dx,dy,
c      &      xp,yp,zp,xd,yd,zd,work_tri,mwork_tri,ipiv)
c       implicit none
c
c       integer mx,my,mwork_tri
c       double precision xlower,ylower,dx,dy
c
c       double precision xp(0:mx+1,0:my+1)
c       double precision yp(0:mx+1,0:my+1)
c       double precision zp(0:mx+1,0:my+1)
c
c       double precision xd(mx+1,my+1)
c       double precision yd(mx+1,my+1)
c       double precision zd(mx+1,my+1)
c
c       double precision work_tri(mwork_tri)
c       integer ipiv(2*(mx/2 + my))
c
c       integer trisys_du,trisys_d,trisys_dl,trisys_du2
c       integer pcval, qh, coeffs_alpha
c       integer xpi, ypi, zpi, xdi,ydi,zdi
c       integer pval,dval,rv,rhs
c       integer tv
c
c c     # ----------------------------------------------------
c c     # Get pointers needed for tridiagonal system and other
c c     # set in initialization step
c c     # ----------------------------------------------------
c       call get_tri_ptrs(mwork_tri,mx,my,pcval,
c      &      trisys_du,trisys_d,trisys_dl,trisys_du2,qh,
c      &      coeffs_alpha,xpi,ypi,zpi,xdi,ydi,zdi,tv,
c      &      pval,dval,rv,rhs)
c
c c     # ----------------------------------------------------
c c     # Get coefficients for tridiagonal system
c c     # ----------------------------------------------------
c       call setup_equator_mesh(mx,my,
c      &      xlower,ylower,dx,dy,xp,yp,zp,xd,yd,zd,
c      &      work_tri(xpi),work_tri(ypi),work_tri(zpi),
c      &      work_tri(xdi),work_tri(ydi),work_tri(zdi))
c
c       call compute_trisys_coeffs(mx,my,
c      &      work_tri(xpi),work_tri(ypi),work_tri(zpi),
c      &      work_tri(xdi),work_tri(ydi),work_tri(zdi),
c      &      work_tri(pcval),work_tri(tv))
c
c c     # ----------------------------------------------------
c c     # Get coefficients for tridiagonal system
c c     # This could also be used for the bc_phys.f routine.
c c     # ----------------------------------------------------
c       call construct_sys(mx,my,work_tri(tv),
c      &      work_tri(trisys_dl),work_tri(trisys_d),
c      &      work_tri(trisys_du),work_tri(trisys_du2),
c      &      ipiv,work_tri(qh),
c      &      work_tri(coeffs_alpha))
c
c       end
c
c       subroutine construct_sys(mx,my,tv,
c      &      trisys_dl,trisys_d,trisys_du,trisys_du2,
c      &      ipiv, qh, coeffs_alpha)
c       implicit none
c
c       integer mx,my
c       double precision  trisys_dl(2*(mx/2+my))
c       double precision   trisys_d(2*(mx/2+my))
c       double precision  trisys_du(2*(mx/2+my))
c       double precision trisys_du2(2*(mx/2+my))
c       integer                ipiv(2*(mx/2+my))
c
c       double precision     qh(2*(mx/2+my))
c       double precision coeffs_alpha(3)
c
c       double precision tv(0:2*(mx/2 + my))
c
c       double precision v(2),w(2), u
c       integer k, km, np, ntri, ldb, nrhs, info
c       character trans
c
c       np = 2*(mx/2+my)
c
c       do k = 1,np-1
c          trisys_dl(k) = tv(k)
c       enddo
c
c       do k = 1,np
c          trisys_d(k) = tv(k) - tv(k-1) - 2.d0
c       enddo
c
c       do k = 1,np-1
c          trisys_du(k) = -tv(k)
c       enddo
c
c c     # Define vectors w and v. We only store v(1) and v(np-1); the rest
c c     # are zero.  Same for w.
c       w(1) = tv(np)
c       w(2) = -tv(np-1)
c       v(1) = -tv(np)
c       v(2) = tv(np-1)
c       u = tv(np) - tv(np-1) - 2.d0
c
c c     # Factor tridiagonal system.
c       ntri = np - 1
c       call dgttrf(ntri,trisys_dl, trisys_d, trisys_du,
c      &      trisys_du2, ipiv,info)
c
c       if (info .ne. 0) then
c          write(6,'(A,I5)') 'construct_sys(1) : info .ne. 0; ',
c      &         info
c          stop
c       endif
c
c c     # w - equation (2.41)
c       do k = 1,np-1
c          qh(k) = 0.d0
c       enddo
c       qh(1)    = -w(1)
c       qh(np-1) = -w(2)
c
c   100 format(I5,4E24.16)
c
c       trans = 'n'
c       ldb      = ntri
c       nrhs     = 1
c       call dgttrs(trans,ntri,nrhs,trisys_dl,trisys_d,
c      &      trisys_du,trisys_du2,ipiv,qh,ldb,info)
c
c       if (info .ne. 0) then
c          write(6,'(A,I5)') 'construct_sys(2) : info .ne. 0; ',
c      &         info
c          stop
c       endif
c
c       coeffs_alpha(1) = 1.d0/(u + v(1)*qh(1) + v(2)*qh(np-1))
c       coeffs_alpha(2) = -coeffs_alpha(1)*v(1)
c       coeffs_alpha(3) = -coeffs_alpha(1)*v(2)
c
c       end
c
c c     # -------------------------------------------------------
c c     # Setup grid at the equator needed to compute coefficients.
c c     # -------------------------------------------------------
c       subroutine setup_equator_mesh(mx,my,xlower,ylower,dx,dy,
c      &      xp,yp,zp,xd,yd,zd,xpi,ypi,zpi,xdi,ydi,zdi)
c       implicit none
c
c       integer mx,my
c       double precision xlower,ylower,dx,dy
c
c       double precision xp(0:mx+1,0:my+1)
c       double precision yp(0:mx+1,0:my+1)
c       double precision zp(0:mx+1,0:my+1)
c
c       double precision xd(mx+1,my+1)
c       double precision yd(mx+1,my+1)
c       double precision zd(mx+1,my+1)
c
c       double precision xpi(2*(mx/2 + my),-1:1)
c       double precision ypi(2*(mx/2 + my),-1:1)
c       double precision zpi(2*(mx/2 + my),-1:1)
c
c       double precision xdi(2*(mx/2 + my))
c       double precision ydi(2*(mx/2 + my))
c       double precision zdi(2*(mx/2 + my))
c
c       integer i,j,n
c       double precision xc, yc, xd1, yd1, zd1
c       double precision xp1, yp1,zp1
c       double precision xupper, yupper
c
c       double precision d1, d2, d3
c       integer np1, np
c
c       if (mx/2 .ne. my) then
c          write(6,*) 'setup_mesh_interal : mx/2 .ne. my'
c          stop
c       endif
c
c       call match_p(mx,my,xp)
c       call match_p(mx,my,yp)
c       call match_p(mx,my,zp)
c
c       xupper = xlower + mx*dx
c       yupper = ylower + my*dy
c
c       n = 1
c       do i = mx/2 + 1, mx
c          do j = -1,1,2
c             xpi(n,j) = xp(i,(j+1)/2)
c             ypi(n,j) = yp(i,(j+1)/2)
c             zpi(n,j) = zp(i,(j+1)/2)
c          enddo
c          xc = xlower + (i-0.5)*dx
c          yc = ylower
c          call mapc2m(xc,yc,xpi(n,0), ypi(n,0),zpi(n,0))
c
c          n = n + 1
c       enddo
c
c       do j = 1,my
c          do i = -1,1,2
c             xpi(n,i) = xp((1-i)/2 + mx,j)
c             ypi(n,i) = yp((1-i)/2 + mx,j)
c             zpi(n,i) = zp((1-i)/2 + mx,j)
c          enddo
c          xc = xupper
c          yc = ylower + (j-0.5)*dy
c          call mapc2m(xc,yc,xpi(n,0), ypi(n,0),zpi(n,0))
c          n = n + 1
c       enddo
c
c       do i = mx,mx/2+1,-1
c          do j = -1,1,2
c             xpi(n,j) = xp(i,(1-j)/2 + my)
c             ypi(n,j) = yp(i,(1-j)/2 + my)
c             zpi(n,j) = zp(i,(1-j)/2 + my)
c          enddo
c          xc = xlower + (i-0.5)*dx
c          yc = yupper
c          call mapc2m(xc,yc,xpi(n,0), ypi(n,0),zpi(n,0))
c          n = n + 1
c       enddo
c
c       do j = my,1,-1
c          do i = -1,1,2
c             xpi(n,i) = xp((i+1+mx)/2,j)
c             ypi(n,i) = yp((i+1+mx)/2,j)
c             zpi(n,i) = zp((i+1+mx)/2,j)
c          enddo
c          xc = xlower + mx*dx/2.d0
c          yc = ylower + (j-0.5)*dy
c          call mapc2m(xc,yc,xpi(n,0), ypi(n,0),zpi(n,0))
c
c          n = n + 1
c       enddo
c
c c     # Dual cells
c       n = 1
c       do i = mx/2+1,mx
c          xdi(n) = xd(i,1)
c          ydi(n) = yd(i,1)
c          zdi(n) = zd(i,1)
c          n = n + 1
c       enddo
c
c       do j = 1,my
c          xdi(n) = xd(mx+1,j)
c          ydi(n) = yd(mx+1,j)
c          zdi(n) = zd(mx+1,j)
c          n = n + 1
c       enddo
c
c       do i = mx+1,mx/2+2,-1
c          xdi(n) = xd(i,my+1)
c          ydi(n) = yd(i,my+1)
c          zdi(n) = zd(i,my+1)
c          n = n + 1
c       enddo
c
c       do j = my+1,2,-1
c          xdi(n) = xd(mx/2+1,j)
c          ydi(n) = yd(mx/2+1,j)
c          zdi(n) = zd(mx/2+1,j)
c          n = n + 1
c       enddo
c
c   100 format(I5,3E16.8)
c
c       end
c
c c     # ----------------------------------------------------------------
c c     # Setup the coefficients needed for the boundary conditions operator
c c     # ----------------------------------------------------------------
c       subroutine compute_trisys_coeffs(mx,my,xpi,ypi,zpi,
c      &      xdi,ydi,zdi,pcval,tv)
c       implicit none
c
c       integer mx,my
c       double precision xpi(2*(mx/2 + my),-1:1)
c       double precision ypi(2*(mx/2 + my),-1:1)
c       double precision zpi(2*(mx/2 + my),-1:1)
c
c       double precision xdi(2*(mx/2 + my))
c       double precision ydi(2*(mx/2 + my))
c       double precision zdi(2*(mx/2 + my))
c
c       double precision pcval(2*(mx/2 + my),2)
c
c       double precision tv(0:2*(mx/2 + my))
c
c       integer i,k,m, np, kp1
c       double precision taup_head(3),taup_tail(3), taup(3), sp
c       double precision taud_head(3),taud_tail(3), taud(3), sd
c       double precision d_points(2,3)
c       double precision p_points(3)
c       double precision a1, a2, ca, sa, cta
c
c       double precision uu, ul, pu, pl, u(2), p(2)
c
c       np = 2*(mx/2 + my)
c
c c     % Loop over each primary cell
c       do k = 1,np
c          do i = -1,1,2
c             taud_tail(1) = xpi(k,i)
c             taud_tail(2) = ypi(k,i)
c             taud_tail(3) = zpi(k,i)
c
c c           # These are the points on nodes of the primary cell
c             if (k .eq. np) then
c                kp1 = 1
c             else
c                kp1 = k + 1
c             endif
c             d_points(1,1) = xdi(k)
c             d_points(2,1) = xdi(kp1)
c
c             d_points(1,2) = ydi(k)
c             d_points(2,2) = ydi(kp1)
c
c             d_points(1,3) = zdi(k)
c             d_points(2,3) = zdi(kp1)
c
c c           # These are the points at the end of each arm
c             p_points(1) = xpi(k,0)
c             p_points(2) = ypi(k,0)
c             p_points(3) = zpi(k,0)
c
c             sp = 0
c             sd = 0
c             do m = 1,3
c                taup_head(m) = d_points(2,m)
c                taup_tail(m) = d_points(1,m)
c                taud_head(m) = p_points(m)
c
c                taup(m) = taup_head(m) - taup_tail(m)
c                taud(m) = taud_head(m) - taud_tail(m)
c
c                sp = sp + taup(m)*taup(m)
c                sd = sd + taud(m)*taud(m)
c             enddo
c             sp = sqrt(sp)
c             sd = sqrt(sd)
c
c             ca = (taup(1)*taud(1) + taup(2)*taud(2) +
c      &            taup(3)*taud(3))/(sp*sd)
c             sa = sqrt(1.d0 - ca**2)
c             cta = ca/sa
c
c             a1 = 1.d0/sa
c             a2 = -cta
c
c c            pcval(k,(1-i)/2+1) = (sp/sd)*a1/lenpi(k)
c c            ucval(k,(1-i)/2+1) = a2/lenpi(k)
c
c c           # lenpi(k) = sp
c c           # pcval is needed for computing rhs...
c             pcval(k,(1-i)/2+1) = a1/sd
c
c             p((1-i)/2+1) = a1/sd
c             u((1-i)/2+1) = a2/sp
c          enddo
c          tv(k) = (u(1) + u(2))/(p(1) + p(2))
c       enddo
c       tv(0) = tv(np)
c
c   100 format(I5,3E24.16)
c       end
c
c
c
c
c c     # -------------------------------------------------------------
c c     # Miscellaneous routines for allocating memory
c c     # -------------------------------------------------------------
c
c       subroutine get_tri_ptrs(mwork,mx,my,pcval,
c      &      trisys_du,trisys_d,trisys_dl,trisys_du2,qh,
c      &      coeffs_alpha,xpi,ypi,zpi,xdi,ydi,zdi,tv,
c      &      pval,dval,rv,rhs)
c       implicit none
c
c       integer mwork,mx,my
c       integer trisys_du,trisys_d,trisys_dl,trisys_du2,qh
c       integer pcval,coeffs_alpha
c       integer xpi,ypi,zpi,xdi,ydi,zdi,tv,pval,dval,rv,rhs
c
c       integer total, nbc
c
c       nbc = 2*(mx/2 + my)
c
c       pcval        = 1
c       trisys_du    = pcval        + 2*nbc
c       trisys_d     = trisys_du    + nbc
c       trisys_dl    = trisys_d     + nbc
c       trisys_du2   = trisys_dl    + nbc
c       qh           = trisys_du2   + nbc
c       coeffs_alpha = qh           + nbc
c       xpi          = coeffs_alpha + 3
c       ypi          = xpi          + 3*nbc
c       zpi          = ypi          + 3*nbc
c       xdi          = zpi          + 3*nbc
c       ydi          = xdi          + nbc
c       zdi          = ydi          + nbc
c       tv           = zdi          + nbc
c       pval         = tv           + nbc + 1
c       dval         = pval         + 2*nbc
c       rv           = dval         + nbc
c       rhs          = rv           + nbc + 1
c       total        = rhs          + nbc + 1
c
c       total = total - 1
c
c       if (mwork .lt. 0) then
c c        # This is being called from the RKC solver - we already
c c        # know that we have enough space
c          return
c       endif
c       if (total .gt. mwork) then
c          write(6,*) 'Increase memory in work_tri to ',total
c          stop
c       endif
c
c       end
