      subroutine setup_mesh(mx,my,mbc,xlower,ylower,dx,dy,
     &      xp,yp,zp,xd,yd,zd)
      implicit none

      integer mx,my, mbc
      double precision dx,dy,xlower,ylower

      double precision xp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision yp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision zp(-mbc:mx+mbc+1,-mbc:my+mbc+1)

      double precision xd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
      double precision yd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
      double precision zd(-mbc:mx+mbc+2,-mbc:my+mbc+2)

      integer i,j
      double precision dxf,dyf, xc,yc,xd1, yd1,zd1

c     # We need both cell centered and node locations to
c     # compute the normals at cell edges.
      dxf = dx/2.d0
      dyf = dy/2.d0
      do i = -2*mbc-1,2*(mx+mbc+1)+1
         do j = -2*mbc-1,2*(my+mbc+1)+1
            if (abs(mod(i,2)) .ne. abs(mod(j,2))) then
               cycle
            endif
            xc = xlower + (i-1)*dxf
            yc = ylower + (j-1)*dyf

            call mapc2m(xc,yc ,xd1,yd1,zd1)

            if (abs(mod(i,2)) .eq. 1) then
c              # Physical location of cell vertices
               xd((i-1)/2 + 1, (j-1)/2 + 1) = xd1
               yd((i-1)/2 + 1, (j-1)/2 + 1) = yd1
               zd((i-1)/2 + 1, (j-1)/2 + 1) = zd1
            else
c              # Physical locations of cell centers
               xp(i/2,j/2) = xd1
               yp(i/2,j/2) = yd1
               zp(i/2,j/2) = zd1
            endif
         enddo
      enddo


      end


c     # -------------------------------------------------------
c     # Setup grid at the equator needed to compute coefficients.
c     # -------------------------------------------------------
      subroutine setup_boundary_mesh(mx,my,mbc,xlower,ylower,dx,dy,
     &      xp,yp,zp,xd,yd,zd,xpi,ypi,zpi,xdi,ydi,zdi)
      implicit none

      integer mx,my,mbc
      double precision xlower,ylower,dx,dy

      double precision xp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision yp(-mbc:mx+mbc+1,-mbc:my+mbc+1)
      double precision zp(-mbc:mx+mbc+1,-mbc:my+mbc+1)

      double precision xd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
      double precision yd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
      double precision zd(-mbc:mx+mbc+2,-mbc:my+mbc+2)

      double precision xpi(2*(my + my),-1:1)
      double precision ypi(2*(my + my),-1:1)
      double precision zpi(2*(my + my),-1:1)

      double precision xdi(2*(my + my))
      double precision ydi(2*(my + my))
      double precision zdi(2*(my + my))

      integer i,j,n, mx2
      double precision xc, yc, xd1, yd1, zd1
      double precision xp1, yp1,zp1
      double precision xupper, yupper

      double precision d1, d2, d3
      integer np1, np
      logical issphere, isdiamond

      if (issphere() .or. isdiamond()) then
         if (mx/2 .ne. my) then
            write(6,*) 'setup_mesh_internal : mx/2 .ne. my'
            stop
         endif
         mx2 = mx/2
      else
         if (mx .ne. my) then
            write(6,*) 'setup_mesh_internal : mx .ne. my'
            stop
         endif
         mx2 = 0
      endif

      xupper = xlower + mx*dx
      yupper = ylower + my*dy

      n = 1
      do i = mx2 + 1, mx
         do j = -1,1,2
            xpi(n,j) = xp(i,(j+1)/2)
            ypi(n,j) = yp(i,(j+1)/2)
            zpi(n,j) = zp(i,(j+1)/2)
         enddo
         xc = xlower + (i-0.5)*dx
         yc = ylower
         call mapc2m(xc,yc,xpi(n,0), ypi(n,0),zpi(n,0))

         n = n + 1
      enddo

      do j = 1,my
         do i = -1,1,2
            xpi(n,i) = xp((1-i)/2 + mx,j)
            ypi(n,i) = yp((1-i)/2 + mx,j)
            zpi(n,i) = zp((1-i)/2 + mx,j)
         enddo
         xc = xupper
         yc = ylower + (j-0.5)*dy
         call mapc2m(xc,yc,xpi(n,0), ypi(n,0),zpi(n,0))
         n = n + 1
      enddo

      do i = mx,mx2+1,-1
         do j = -1,1,2
            xpi(n,j) = xp(i,(1-j)/2 + my)
            ypi(n,j) = yp(i,(1-j)/2 + my)
            zpi(n,j) = zp(i,(1-j)/2 + my)
         enddo
         xc = xlower + (i-0.5)*dx
         yc = yupper
         call mapc2m(xc,yc,xpi(n,0), ypi(n,0),zpi(n,0))
         n = n + 1
      enddo

      do j = my,1,-1
         do i = -1,1,2
            xpi(n,i) = xp((i+1+2*mx2)/2,j)
            ypi(n,i) = yp((i+1+2*mx2)/2,j)
            zpi(n,i) = zp((i+1+2*mx2)/2,j)
         enddo
         xc = xlower + mx2*dx
         yc = ylower + (j-0.5)*dy
         call mapc2m(xc,yc,xpi(n,0), ypi(n,0),zpi(n,0))

         n = n + 1
      enddo

c     # Dual cells
      n = 1
      do i = mx2+1,mx
         xdi(n) = xd(i,1)
         ydi(n) = yd(i,1)
         zdi(n) = zd(i,1)
         n = n + 1
      enddo

      do j = 1,my
         xdi(n) = xd(mx+1,j)
         ydi(n) = yd(mx+1,j)
         zdi(n) = zd(mx+1,j)
         n = n + 1
      enddo

      do i = mx+1,mx2+2,-1
         xdi(n) = xd(i,my+1)
         ydi(n) = yd(i,my+1)
         zdi(n) = zd(i,my+1)
         n = n + 1
      enddo

      do j = my+1,2,-1
         xdi(n) = xd(mx2+1,j)
         ydi(n) = yd(mx2+1,j)
         zdi(n) = zd(mx2+1,j)
         n = n + 1
      enddo

      end


      subroutine compute_area(mx,my,mbc,xd,yd,zd,area)
      implicit none

      integer mx,my,mbc

      double precision xd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
      double precision yd(-mbc:mx+mbc+2,-mbc:my+mbc+2)
      double precision zd(-mbc:mx+mbc+2,-mbc:my+mbc+2)

      double precision area(-mbc:mx+mbc+1,-mbc:my+mbc+1)

      integer i,j,ii,jj
      double precision quad(0:1,0:1,3)
      double precision get_area_approx

c     # Primary cells.  Note that we don't do anything special
c     # in the diagonal cells - so the areas there are less accurate
c     # than in the rest of the mesh.
      do j = -mbc,my+mbc
         do i = -mbc,mx+mbc
c           # Approximate surfaces of the cells using a trilinear
c           # map
            do ii = 0,1
               do jj = 0,1
                  quad(ii,jj,1) = xd(i+ii,j+jj)
                  quad(ii,jj,2) = yd(i+ii,j+jj)
                  quad(ii,jj,3) = zd(i+ii,j+jj)
               enddo
            enddo
            area(i,j) = get_area_approx(quad)
         enddo
      enddo
      end


c     # This is the area element based on a bilinear approximation to
c     # the surface defined by the corners of the mesh cell.
      double precision function get_area_approx(quad)
      implicit none

      double precision quad(0:1,0:1,3)

      double precision a00(3), a01(3),a10(3), a11(3)
      double precision Txi(3), Teta(3)
      double precision norm_cross, xi,eta

      integer m

c     # Coefficients for bilinear approximation to surface
      do m = 1,3
         a00(m) = quad(0,0,m)
         a01(m) = quad(1,0,m) - quad(0,0,m)
         a10(m) = quad(0,1,m) - quad(0,0,m)
         a11(m) = quad(1,1,m) - quad(1,0,m) -
     &         quad(0,1,m) + quad(0,0,m)
      enddo

      eta = 0.5d0
      xi  = 0.5d0

c     # Mesh square is approximated by
c     #       T(xi,eta) = a00 + a01*xi + a10*eta + a11*xi*eta
c     #
c     # Area is the length of the cross product of T_xi and T_eta.
c     # computed at the center of the mesh cell.
      do m = 1,3
         txi(m)  = (a01(m) + a11(m)*eta)
         teta(m) = (a10(m) + a11(m)*xi)
      enddo

      get_area_approx = norm_cross(txi,teta)

      end

      double precision function norm_cross(u,v)
      implicit none

      double precision u(3), v(3), w(3)

      w(1) =  u(2)*v(3) - u(3)*v(2)
      w(2) = -u(1)*v(3) + u(3)*v(1)
      w(3) =  u(1)*v(2) - u(2)*v(1)

      norm_cross = sqrt(w(1)*w(1) + w(2)*w(2) + w(3)*w(3))

      end
