      subroutine write_line(iunit,len)
      implicit none

      integer iunit,len,i
      character(1000) line

      do i = 1,len
         line(i:i) = '-'
      enddo

      write(iunit,'(A)') line(1:len)

      end

c     # These are kept separate from RKC counters above, since I wanted to
c     # use this for all time stepping routines.
      subroutine initialize_laplacian_call_counter()
      implicit none

      integer lc0_com, lc1_com, lapcalls_total_com
      common /com_stats1/ lc0_com, lc1_com, lapcalls_total_com

      lapcalls_total_com = 0

      end

      subroutine start_laplacian_call_counter()
      implicit none

      integer lc0_com, lc1_com, lapcalls_total_com
      common /com_stats1/ lc0_com, lc1_com, lapcalls_total_com

      lc0_com = lapcalls_total_com

      end

      subroutine stop_laplacian_call_counter()
      implicit none

      integer lc0_com, lc1_com, lapcalls_total_com
      common /com_stats1/ lc0_com, lc1_com, lapcalls_total_com

      lc1_com = lapcalls_total_com

      end

      subroutine increment_laplacian_call_counter()
      implicit none

      integer lc0_com, lc1_com, lapcalls_total_com
      common /com_stats1/ lc0_com, lc1_com, lapcalls_total_com

      lapcalls_total_com = lapcalls_total_com + 1

      end


      integer function get_laplacian_calls_elapsed()
      implicit none

      integer lc0_com, lc1_com, lapcalls_total_com
      common /com_stats1/ lc0_com, lc1_com, lapcalls_total_com

      get_laplacian_calls_elapsed = lc1_com - lc0_com

      end


      integer function get_laplacian_calls_total()
      implicit none

      integer lc0_com, lc1_com, lapcalls_total_com
      common /com_stats1/ lc0_com, lc1_com, lapcalls_total_com

      get_laplacian_calls_total = lapcalls_total_com

      end


c     # These are kept separate from RKC counters above, since I wanted to
c     # use this for all time stepping routines.
      subroutine initialize_time_step_counter()
      implicit none

      integer timestep_total
      common /com_stats2/ timestep_total

      timestep_total = 1

      end

      subroutine increment_time_step_counter()
      implicit none

      integer timestep_total
      common /com_stats2/ timestep_total

      timestep_total = timestep_total + 1

      end


      integer function get_time_step_counter()
      implicit none

      integer timestep_total
      common /com_stats2/ timestep_total

      get_time_step_counter = timestep_total

      end

      subroutine print_stats_short(ti,dt)
      implicit none

      double precision ti, dt, tip1

      integer ts_counter
      integer get_time_step_counter

      integer lc_inc, lc_total
      integer get_laplacian_calls_total
      integer get_laplacian_calls_elapsed

      double precision t_total, t_inc, t_per_inc
      double precision total_time, elapsed_time
      logical timing_stats, laplacian_call_stats

      timing_stats = .true.
      laplacian_call_stats = .false.

      ts_counter = get_time_step_counter()
      lc_total = get_laplacian_calls_total()
      lc_inc = get_laplacian_calls_elapsed()

      t_total = total_time()
      t_inc = elapsed_time()

      tip1 = ti + dt

      if (laplacian_call_stats) then
         write(6,500) ts_counter, tip1, dt, lc_inc, lc_total
  500    format('Step ', i4,
     &         ' at time t = ',E12.4,
     &         '   (dt = ',E10.4,
     &         ';  lap. calls (this call) = 'I3,
     &         ';  lap. calls (total) = ',I10,')')
      endif

      if (timing_stats) then
         if (lc_total .ne. 0) then
            t_per_inc = t_total/lc_total
         else
            t_per_inc = 0.d0
         endif

         write(6,510) ts_counter, tip1, dt, t_per_inc, t_total
  510    format('Step ', i4,
     &         ' at time t = ',E12.4,
     &         '   (dt = ',E10.4,
     &         ';  time/lap. call = 'E12.4,
     &         ';  total_time = ',E16.4,')')
      endif
      end

      subroutine print_stats_long(ti,dt)
      implicit none

      double precision ti, dt, tip1

      integer ts_counter
      integer get_time_step_counter

      integer lc_inc, lc_total
      integer get_laplacian_calls_total
      integer get_laplacian_calls_elapsed

      double precision t_total, t_inc, t_per_inc
      double precision total_time, elapsed_time
      double precision accumulated_time, t_acc
      logical timing_stats, laplacian_call_stats

      ts_counter = get_time_step_counter()
      lc_total = get_laplacian_calls_total()
      lc_inc = get_laplacian_calls_elapsed()

      t_total = total_time()
      t_acc = accumulated_time()
      t_inc = elapsed_time()

      tip1 = ti + dt

      if (lc_total .ne. 0) then
         t_per_inc = t_total/lc_total
      else
         t_per_inc = 0.d0
      endif

      write(6,100) 'Step',ts_counter
      write(6,110) 'time',tip1
      write(6,110) 'dt',dt
      write(6,120) 'Laplacian calls (this step)',lc_inc
      write(6,120) 'Laplacian calls (total)',lc_total
      write(6,110) 'Time per lap. call (s)',t_per_inc
      write(6,110) 'Time (all calls)',t_acc
      write(6,110) 'Total time',t_total
      write(6,*) ' '

  100 format(A35,I12)
  110 format(A35,E12.4)
  120 format(A35,I12)

      end



      subroutine set_method(method)
      implicit none
      integer method

      include 'para.i'

      method_com = method

      end

      integer function get_method()
      implicit none

      include 'para.i'

      get_method = method_com

      end

      subroutine check_method(m_correct)
      implicit none

      integer m_correct, m_actual, get_method

      m_actual = get_method()
      if (m_actual .ne. m_correct) then
         write(6,100) 'check_method : Method should be set to ',
     &         m_correct, '.  Use ''set_method()'' in setprob.f'
         stop
      endif
  100 format(A,I3,A)



      end


      subroutine set_verbosity(verbose)
      implicit none

      integer verbose

      integer verbosity_com
      common /com_verbose/ verbosity_com

      verbosity_com = verbose

      end

      integer function get_verbosity()
      implicit none

      integer verbosity_com
      common /com_verbose/ verbosity_com

      get_verbosity = verbosity_com

      end


      subroutine store_itcount(itcnt)
      implicit none
      integer itcnt(2)

      integer itcnt_com(2), m
      common /com_itcnt/itcnt_com

      do m = 1,2
         itcnt_com(m) = itcnt(m)
      enddo

      end

      integer function get_itcount(itcnt)
      implicit none

      integer itcnt(2)
      integer itcnt_com(2), m
      common /com_itcnt/itcnt_com

      do m = 1,2
         itcnt(m) = itcnt_com(m)
      enddo

      end

      subroutine initialize_timer()
      implicit none

      double precision t0_com,t1_com,time_total_com, tinit_com,
     &      time_accumul_com
      common /comtime/ t0_com,t1_com,time_total_com, tinit_com,
     &      time_accumul_com

      call cpu_time(tinit_com)

      end

      subroutine initialize_accumulated_time()
      implicit none

      double precision t0_com,t1_com,time_total_com, tinit_com,
     &      time_accumul_com
      common /comtime/ t0_com,t1_com,time_total_com, tinit_com,
     &      time_accumul_com

      time_accumul_com = 0

      end

      double precision function total_time()
      implicit none

      double precision t0_com,t1_com,time_total_com, tinit_com,
     &      time_accumul_com
      common /comtime/ t0_com,t1_com,time_total_com, tinit_com,
     &      time_accumul_com

      double precision tnow
      call cpu_time(tnow)

      total_time = tnow - tinit_com

      end


      subroutine start_timer()
      implicit none

      double precision t0_com,t1_com,time_total_com, tinit_com,
     &      time_accumul_com
      common /comtime/ t0_com,t1_com,time_total_com, tinit_com,
     &      time_accumul_com

      call cpu_time(t0_com)

      end


      subroutine stop_timer()
      implicit none

      double precision t0_com,t1_com,time_total_com, tinit_com,
     &      time_accumul_com
      common /comtime/ t0_com,t1_com,time_total_com, tinit_com,
     &      time_accumul_com

      double precision elapsed_time

      call cpu_time(t1_com)
      time_accumul_com = time_accumul_com + elapsed_time()

      end

      double precision function elapsed_time()
      implicit none

      double precision t0_com,t1_com,time_total_com, tinit_com,
     &      time_accumul_com
      common /comtime/ t0_com,t1_com,time_total_com, tinit_com,
     &      time_accumul_com

      elapsed_time = t1_com - t0_com

      end

      double precision function accumulated_time()
      implicit none

      double precision t0_com,t1_com,time_total_com, tinit_com,
     &      time_accumul_com
      common /comtime/ t0_com,t1_com,time_total_com, tinit_com,
     &      time_accumul_com

      accumulated_time = time_accumul_com

      end


C     # ------------------------------------------------------------
C     # These two routines are needed since irkc has a different data
C     # ordering than other routines
C     # ------------------------------------------------------------
      SUBROUTINE copy_to_work_irkc(mx,my,meqn,nactive,q,qwork)
      implicit none

      INTEGER mx,my,meqn,nactive
      DOUBLE PRECISION q(nactive,0:mx+1,0:my+1)
      DOUBLE PRECISION qwork(0:mx+1,0:my+1,nactive)

      INTEGER i,j,mq,m, q_idx
      logical active_parabolic_term


      DO mq = 1,meqn
         IF (active_parabolic_term(mq)) THEN
            m = q_idx(mq)
            DO i = 0,mx+1
               DO j = 0,my+1
                  qwork(i,j,m) = q(m,i,j)
               ENDDO
            ENDDO
         ENDIF
      ENDDO

      END

      SUBROUTINE copy_from_work_irkc(mx,my,meqn,nactive,qwork,q)
        implicit none

        INTEGER mx,my,meqn,nactive
        DOUBLE PRECISION q(nactive,0:mx+1,0:my+1)
        DOUBLE PRECISION qwork(0:mx+1,0:my+1,nactive)

        INTEGER i,j,mq,m, q_idx
        LOGICAL active_parabolic_term


        DO mq = 1,meqn
           IF (active_parabolic_term(mq)) THEN
              m = q_idx(mq)
              DO i = 0,mx+1
                 DO j = 0,my+1
                    q(m,i,j) = qwork(i,j,m)
                 ENDDO
              ENDDO
           ENDIF
        ENDDO

      END
