% Routines for plotting data output from Clawpack package
%
% Basic Clawpack plotting routines and options.
%
%      plotclaw1              - plot 1d results
%      plotclaw2              - plot 2d results
%      plotclaw3              - plot 3d results
%      setplot1               - script for setting parameters for 1d plots
%      setplot2               - script for setting parameters for 2d plots
%      setplot3               - script for setting parameters for 3d plots
%      setplot                - help on using SETPLOT<N>
%      readamrdata            - reads output files produced by Clawpack.
%      plotframe1ez           - functional form of plotframe1.
%      setplotstyle           - sets symbols and colors for line/scatter plots
%      queryframe             - querys user for next frame information
%      getlegendinfo          - returns legend information on line plots.
%      printgif               - prints a gif file from current figure
%      printjpg               - prints a jpg file from current figure
%
% Data analysis routines
%      pressure               - returns pressure given input data
%      xvelocity              - returns x velocity
%      yvelocity              - returns y velocity
%      zvelocity              - returns z velocity
%      mach                   - return mach number.
%
% General graph properties for 2d and 3d plots.
%
%      showcontourlines       - shows contour lines
%      hidecontourlines       - hides contour lines
%      showgridlines          - shows computational grid
%      hidegridlines          - hides computational grid
%      showpatchborders       - shows patch borders
%      hidepatchborders       - hides patch borders
%      showmesh               - shows a coarsened mesh on specified levels
%      hidemesh               - hides a coarsened mesh on specified levels
%      mappedgrid             - information on how to plot using mapped grids.
%      setcolors              - gives user control over how colors are set
%      setopengl              - Sets OpenGL renderer
%      showslices             - shows slices/manifold
%      hideslices             - hides slices/manifold
%      setslicecolor          - sets color of slice/manifold
%      setslicealpha          - set transparency value of slice/manifold
%      yrbcolormap            - yellow/red/blue colormap
%      redwhite               - red/white colormap
%      rybcolormap            - red/yellow/blue colormap
%      getblocknumber         - get block number for plotting results of
%                               multi-block calculations.
%      showlevels             - shows specified levels
%      hidelevels             - hides specified levels
%      setgridlinecolor       - sets color of grid lines
%      setcontourlinecolor    - sets color of contour lines
%      setpatchbordercolor    - sets color of patch borders.
%      drawcontourlines       - draws contour lines on slices.
%      chombo                 - help for graphing using Chombo hdf5 output.
%
% 2d specific graphics routines
%
%      manifold               - information on how to plot a manifold.
%      projectcontours        - projects contour lines to user-specified plane.
%
% 3d specific graphics routines
%
%      sliceloop              - loop over slices on 3d plots.
%      surfloop               - loop over isosurfaces
%      setviews               - sets pre-defined viewing angles
%      showcubes              - shows 3d amr patch cube borders
%      hidecubes              - hides 3d amr patch cube borders
%      showsurfs              - shows isosurfaces created with ISOSURFVALUES
%      hidesurfs              - hides isosurfaces created with ISOSURFVALUES
%      showsurflevels         - show isosurfaces at specified AMR levels
%      hidesurflevels         - hide isosurfaces at specified AMR levels
%      reducesurf             - reduces number of faces on isosurface.
%      showsurfmesh           - shows isosurface mesh
%      hidesurfmesh           - hides isosurface mesh
%      setsurfalpha           - sets isosurface transparency value
%      setsurfcolor           - sets isosurface color
%
% Type 'help' on any one of the individual topics above for more help.
%
